#include <cdefBF537.h>

#include <string.h>

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "lwipopts.h"

#include "lib/emac_bf/emac_bf.h"
#include "lib/emac_bf/lan8700.h"
#include "lib/dma-bf/dma.h"


unsigned g_m = 0;


/*================================================================================================*/
#define RX_BUF_NUM (16)
#if ETH_PAD_SIZE
  #define RX_BUF_SZ (1556 + ETH_PAD_SIZE)
#else
  #define RX_BUF_SZ (1556)
#endif
#define TX_BUF_NUM (1)
#define TX_BUF_SZ (1514)
/*================================================================================================*/

/*================================================================================================*/
struct rx_status
    {
    t_uint32 status;
    };
struct tx_frame
    {
    t_uint16 size;
    t_uint8 data[ TX_BUF_SZ ];
    };
struct tx_status
    {
    t_uint32 status;
    };
/*================================================================================================*/

/*================================================================================================*/
static void f_copy_rxbuf_part(t_uint32 part_len, t_uint8 *buf);
static void f_init_ethDMA(void);
static int f_next_bufnumber(int n);
static void f_poll_mdc_done(void);
static int f_prev_bufnumber(int n);
static void f_read_PHYreg(t_uint16 PHYaddr, t_uint16 reg_addr, t_uint16 *data);
static void f_set_rxDMAdescriptor(void);
static void f_write_PHYreg(t_uint16 PHYaddr, t_uint16 reg_addr, t_uint16 data);
/*================================================================================================*/

/*================================================================================================*/
static xSemaphoreHandle f_h_datarx_smphr = NULL;
static xSemaphoreHandle f_h_datatx_smphr = NULL;
static int f_n_start_descriptor = 0;
static const unsigned char f_PHY_addr = 0x01;
static int f_queue_full = 0;
static t_uint8 f_rx_buffers[ RX_BUF_NUM ][ RX_BUF_SZ ];
static volatile struct rx_status f_rx_bufsstatus[ RX_BUF_NUM ];
static int f_rxbuf_number;
static t_uint32 f_rxread_curbuf = RX_BUF_NUM - 1;
static t_uint32 f_rxreaded_buf = RX_BUF_NUM - 1;
static t_uint32 f_rxread_idx;
static t_uint32 f_rxread_totallen = 0;
static int f_rxrecv_buf = RX_BUF_NUM;
static struct tx_frame f_tx_buffer;
static int f_tx_run = 0;
static volatile struct tx_status f_tx_bufstatus;
static struct dma_desc_list f_rx_DMAdescrs[ RX_BUF_NUM ][ 2 ];
static struct dma_desc_list f_tx_DMAdescrs[ 2 ];
t_uint8 g_emac_hwaddr[ EMAC_HWADDR_SZ ] = {0x00, 0xCD, 0xCD, 0xCD, 0xCD, 0x00};
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
enum e_EMACErr emac_init
    (
    void
    )
    {
    /*   Ethernet MAC . */
    const portTickType init_delay = 5000 / portTICK_RATE_MS;
    enum e_EMACErr init_status = e_EMACErr_NoErr;
    t_uint32 opmode = RE | TE;
    t_uint16 PHY_status;
    
    /*  MAC- */
    *pEMAC_ADDRLO = *(t_uint32 *)&g_emac_hwaddr[ 0 ];
    *pEMAC_ADDRHI = *(t_uint32 *)&g_emac_hwaddr[ 4 ];
    
    /*     MII (2.5 ) */
    *pEMAC_SYSCTL = 26 << 8;
    
        
    /*  PHY- */
    /*  PHY- */
    f_write_PHYreg( f_PHY_addr, LAN8700_CTRL_REG, LAN8700_CTRL_RST_MASK );
    vTaskDelay( init_delay );
    
    f_write_PHYreg( f_PHY_addr, LAN8700_CTRL_REG, LAN8700_CTRL_ANEN_MASK );
    
    /*     */
    //f_write_PHYreg( f_PHY_addr, LAN8700_ID1_REG, (t_uint16)g_emac_hwaddr[ 0 ] |
    //    ((t_uint16)g_emac_hwaddr[ 1 ] << 8) );
    //f_write_PHYreg( f_PHY_addr, LAN8700_ID2_REG, (t_uint16)g_emac_hwaddr[ 2 ] << 10 );
    
    /*    */
    f_read_PHYreg( f_PHY_addr, LAN8700_STATUS_REG, &PHY_status );
    f_read_PHYreg( f_PHY_addr, LAN8700_STATUS_REG, &PHY_status );
    if (!(PHY_status & LAN8700_STATUS_LNKSTAT_MASK))
        {
        init_status = e_EMACErr_NoLink;
        }
    
    f_read_PHYreg( f_PHY_addr, LAN8700_SPECMODE_REG, &PHY_status );
        
    if (init_status == e_EMACErr_NoErr)
        {
        f_write_PHYreg( f_PHY_addr, LAN8700_CTRL_REG, LAN8700_CTRL_ANEN_MASK |
            LAN8700_CTRL_RESTARTAN_MASK );
        do
            {
            vTaskDelay( 10 );
            f_read_PHYreg( f_PHY_addr, LAN8700_STATUS_REG, &PHY_status );
            }
        while (!(PHY_status & LAN8700_STATUS_ANCOMPLETE_MASK));
        }
        
    f_read_PHYreg( f_PHY_addr, LAN8700_SPEC_CTRLSTAT_REG, &PHY_status );
    
    switch( (PHY_status >> 2) & 0x07)
        {
        case 0x01:
        case 0x02:
            *pEMAC_OPMODE = 0x00;
            break;
        case 0x05:
        case 0x06:
            *pEMAC_OPMODE = FDMODE;
            break;
        }
        
    
    /*     Ethernet */    
//    if (PHY_status & LAN8700_STATUS_10BTFDUPLEX_MASK)
//        {
//        *pEMAC_OPMODE = FDMODE;
//        }
        
    vSemaphoreCreateBinary( f_h_datarx_smphr );
    vSemaphoreCreateBinary( f_h_datatx_smphr );

    f_init_ethDMA();
    
    return init_status;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
t_uint32 emac_input_length
    (
    void
    )
    {
    /*
     *   .
     */
    int i_buf;
    t_uint32 in_len = 0;
    
    for (i_buf = f_next_bufnumber( f_rxreaded_buf );
         (i_buf != f_rxreaded_buf);
         i_buf = f_next_bufnumber( i_buf )
        )
        {
        if (f_rx_bufsstatus[ i_buf ].status != 0)
            {
            if (f_rx_bufsstatus[ i_buf ].status & RX_COMP)
                {
                f_rxread_curbuf = i_buf;
                in_len = f_rx_bufsstatus[ i_buf ].status & 0x7FF;
                break;
                }
            f_rx_bufsstatus[ i_buf ].status = 0;
            }
        }
        
    return in_len;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void emac_read_first
    (
    t_uint32 total_len,                     /* () -     */
    t_uint32 part_len,                      /* () -     */
    t_uint8 *buf                            /* () -   */
    )
    {
    /*
     *   .
     * 
     *               
     *   .
     *  
     *      ,      .
     *   total_len     ,    emac_input_length.
     */
    f_rxread_totallen = total_len;
    f_rxread_idx = 0;
    
    f_copy_rxbuf_part( part_len, buf );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void emac_read_next
    (
    t_uint32 part_len,                      /* () -     */
    t_uint8 *buf                            /* () -   */
    )
    {
    /*
     *   ( emac_read_first   )  
     * .
     */
     
    if (f_rxread_totallen == 0)
        return;
        
    f_copy_rxbuf_part( part_len, buf );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
enum e_EMACErr emac_send
    (
    t_uint8 *buf,
    t_uint32 len,
    int end_frame
    )
    {
    /*
     *    Ethrnet.
     */
    ptrdiff_t buf_space;
    static t_uint8 *pbufdata = f_tx_buffer.data;
    enum e_EMACErr send_status = e_EMACErr_NoErr;
        
    /*      */
    buf_space = (&f_tx_buffer.data[ TX_BUF_SZ ] - pbufdata);
    if (len > buf_space)
        {
        pbufdata = f_tx_buffer.data;
        send_status = e_EMACErr_Mem;
        }
        
    
    if (send_status == e_EMACErr_NoErr)
        {
        t_uint32 i_len = len;
        int start_frame = (pbufdata == f_tx_buffer.data);
        if (start_frame) f_tx_buffer.size = len;
        else             f_tx_buffer.size += len;
        (void)memcpy( pbufdata, buf, len * sizeof(t_uint8) );
        pbufdata += len;
        }
    
    
    
    /*   */
    if (send_status == e_EMACErr_NoErr)
        {
        f_tx_DMAdescrs[ 0 ].pNext = &f_tx_DMAdescrs[ 1 ];
        f_tx_DMAdescrs[ 0 ].pStart = &f_tx_buffer;
        f_tx_DMAdescrs[ 1 ].pStart = (void *)(&f_tx_bufstatus);
        f_tx_DMAdescrs[ 0 ].dConfig = FLOW_LARGE | NDSIZE_7 | SYNC | WDSIZE_32 | DMAEN;
        f_tx_DMAdescrs[ 1 ].dConfig = FLOW_STOP | NDSIZE_0 | DI_EN | SYNC | WDSIZE_32 | WNR | DMAEN;
        f_tx_DMAdescrs[ 0 ].dXCount = 0;
        f_tx_DMAdescrs[ 1 ].dXCount = 0;
        f_tx_DMAdescrs[ 0 ].dXModify = 4;
        f_tx_DMAdescrs[ 1 ].dXModify = 4;
        f_tx_bufstatus.status = 0;
        }
        
    
    /*   (  ) */
    if (send_status == e_EMACErr_NoErr)
        {
        if (end_frame)
            {
            *pDMA2_NEXT_DESC_PTR = &f_tx_DMAdescrs[ 0 ];
            *pDMA2_CONFIG = FLOW_LARGE | NDSIZE_7 | SYNC | WDSIZE_32 | DMAEN;
            
            *pEMAC_OPMODE |= TE;
            
            xSemaphoreTake( f_h_datatx_smphr, portMAX_DELAY );
            
#if 0
            /*    */
            while ((f_tx_bufstatus.status & TX_COMP) == 0)
                {
                vTaskDelay( 2 );
                //continue;
                }
#endif
                
            pbufdata = f_tx_buffer.data;
            }
        }
        
    return send_status;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void emac_wait_input
    (
    void
    )
    {
    /*     Ethernet
     * 
     *    ,     .  ,    
     *     .
     */
    t_uint16 DMA_stopped;
    int i_buf;
     
    
    //DMA_status = *pDMA1_IRQ_STATUS;
    //if ((DMA_status & (DMA_RUN | DFETCH)) == 0)
    DMA_stopped = ((*pDMA1_CONFIG ==
        (FLOW_STOP | NDSIZE_0 | DI_EN | SYNC | WDSIZE_32 | WNR | DMAEN)) &&
        ((unsigned long)*pDMA1_CURR_ADDR - (unsigned long)*pDMA1_START_ADDR == 4));
    if (DMA_stopped)
        {
        if (f_queue_full)
            f_set_rxDMAdescriptor();
        if (!f_queue_full)
            {
            *pDMA1_NEXT_DESC_PTR = &f_rx_DMAdescrs[ f_n_start_descriptor ][ 0 ];
            *pDMA1_CONFIG = FLOW_LARGE | NDSIZE_7 | SYNC | WDSIZE_32 | WNR | DMAEN;
            }
        }
    
    if (!f_queue_full)
        xSemaphoreTake( f_h_datarx_smphr, portMAX_DELAY );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_copy_rxbuf_part
    (
    t_uint32 part_len,
    t_uint8 *buf
    )
    {
    int len = (part_len < f_rxread_totallen) ? part_len : f_rxread_totallen;
        
    (void)memcpy( buf, &f_rx_buffers[ f_rxread_curbuf ][ f_rxread_idx ], len );
    
    f_rxread_idx += len;
        
    if (len > f_rxread_totallen) f_rxread_totallen = 0;
    else                         f_rxread_totallen -= len;
    
    if (f_rxread_totallen == 0)
        {
        f_rx_bufsstatus[ f_rxread_curbuf ].status = 0;
        f_rxreaded_buf = f_rxread_curbuf;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_init_ethDMA
    (
    void
    )
    {
    /*  DMA    Ethernet */
    int i_buf;
    
    /*   DMA (  ): DMA1 - EMAC receive; DMA2 - EMAC transmit */
    *pDMA1_PERIPHERAL_MAP = 0x1000U;
    *pDMA2_PERIPHERAL_MAP = 0x2000U;
    
    *pDMA1_IRQ_STATUS = DMA_DONE | DMA_ERR;
    *pDMA2_IRQ_STATUS = DMA_DONE | DMA_ERR;
    
    /*    DMA1 (EMAC RX)  IVG8 */
    *pSIC_IMASK |= IRQ_DMA1 | IRQ_DMA2;
    
    /*   */
    for (i_buf = 0; (i_buf < RX_BUF_NUM); i_buf++)
        {
        f_rx_bufsstatus[ i_buf ].status = 0;
        
        f_rx_DMAdescrs[ i_buf ][ 0 ].pNext = &f_rx_DMAdescrs[ i_buf ][ 1 ];
        f_rx_DMAdescrs[ i_buf ][ 1 ].pNext = &f_rx_DMAdescrs[ i_buf+1 ][ 0 ];
        f_rx_DMAdescrs[ i_buf ][ 0 ].pStart = f_rx_buffers[ i_buf ];
        f_rx_DMAdescrs[ i_buf ][ 1 ].pStart = (void *)(&f_rx_bufsstatus[ i_buf ]);
        f_rx_DMAdescrs[ i_buf ][ 0 ].dConfig = FLOW_LARGE | NDSIZE_7 | DI_EN | SYNC | WDSIZE_32 |
            WNR | DMAEN;
        f_rx_DMAdescrs[ i_buf ][ 1 ].dConfig = FLOW_LARGE | NDSIZE_7 | DI_EN | SYNC | WDSIZE_32 |
            WNR | DMAEN;
        f_rx_DMAdescrs[ i_buf ][ 0 ].dXCount = 0;
        f_rx_DMAdescrs[ i_buf ][ 1 ].dXCount = 0;
        f_rx_DMAdescrs[ i_buf ][ 0 ].dXModify = 4;
        f_rx_DMAdescrs[ i_buf ][ 1 ].dXModify = 4;
        }
    /*        */
    f_rx_DMAdescrs[ RX_BUF_NUM-1 ][ 1 ].pNext = &f_rx_DMAdescrs[ 0 ][ 0 ]; 
    f_rx_DMAdescrs[ RX_BUF_NUM-1 ][ 1 ].dConfig = FLOW_STOP | NDSIZE_0 | DI_EN | SYNC | WDSIZE_32 |
        WNR | DMAEN;
    f_rxbuf_number = 0;
    
    *pDMA1_NEXT_DESC_PTR = &f_rx_DMAdescrs[ 0 ][ 0 ];
    *pDMA1_CONFIG = FLOW_LARGE | NDSIZE_7 | SYNC | WDSIZE_32 | WNR | DMAEN;
    *pEMAC_OPMODE |= RE;

    f_tx_bufstatus.status = 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_next_bufnumber
    (
    int n                                   /* () -   */
    )
    {
    /*
     *   .
     * 
     *           RX_BUF_NUM.
     *  
     *     
     */
    int n_buf;
    
    if (n + 1 < RX_BUF_NUM) n_buf = n + 1;
    else                    n_buf = 0;
    
    return n_buf;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_poll_mdc_done
    (
    void
    )
    {
    /*     MDC/MDIO */
    while (*pEMAC_STAADD & STABUSY)
        {
        continue;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_prev_bufnumber
    (
    int n                                   /* () -   */
    )
    {
    /*
     *   .
     * 
     *           0.
     *  
     *     
     */
    int n_buf;
    
    if (n > 0) n_buf = n - 1;
    else       n_buf = RX_BUF_NUM - 1;
    
    return n_buf;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_read_PHYreg
    (
    t_uint16 PHYaddr,                       /* () -  PHY- */
    t_uint16 reg_addr,                      /* () -   */
    t_uint16 *data                          /* () -  */
    )
    {
    f_poll_mdc_done();
    *pEMAC_STAADD = SET_PHYAD( PHYaddr ) | SET_REGAD( reg_addr ) | STABUSY;
    f_poll_mdc_done();
    *data = *pEMAC_STADAT;    
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_set_rxDMAdescriptor
    (
    void
    )
    {
    /*
     *   DMA1 (EMAC RX).
     * 
     *      ( FLOW_STOP)    
     *  .     ,   FLOW_STOP , ..  
     *     .   ,    
     *   .
     */
    int nonempty_buf_found;
    int i_buf;
    int n_descr_buf;
    int n_nonempty_buf;
    
    /*    */
    for (i_buf = 0; (i_buf < RX_BUF_NUM); i_buf++)
        {
        if (f_rx_DMAdescrs[ i_buf ][ 1 ].dConfig ==
            (FLOW_STOP | NDSIZE_0 | DI_EN | SYNC | WDSIZE_32 | WNR | DMAEN)
           )
            {
            n_descr_buf = i_buf;
            break;
            }
        }
        
        if (n_descr_buf >= RX_BUF_NUM)
            i_buf = 1;
        
    /*        */
    nonempty_buf_found = 0;
    for (i_buf = f_next_bufnumber( n_descr_buf );
         (i_buf != n_descr_buf);
         i_buf = f_next_bufnumber( i_buf )
        )
        {
        if (f_rx_bufsstatus[ i_buf ].status & RX_COMP)
            {
            nonempty_buf_found = 1;
            n_nonempty_buf = i_buf;
            break;
            }
        }

    /*    ,       */
    if (!nonempty_buf_found)
        n_nonempty_buf = n_descr_buf;
    /* ,     */
    if (n_nonempty_buf != f_next_bufnumber( n_descr_buf ))
        {
        /*   */
        int n_last_descr = f_prev_bufnumber( n_nonempty_buf );
        f_rx_DMAdescrs[ n_last_descr ][ 1 ].dConfig = FLOW_STOP | NDSIZE_0 | DI_EN | SYNC |
            WDSIZE_32 | WNR | DMAEN;
        f_rx_DMAdescrs[ n_descr_buf ][ 1 ].dConfig = FLOW_LARGE | NDSIZE_7 | DI_EN | SYNC |
            WDSIZE_32 | WNR | DMAEN;
        f_n_start_descriptor = f_next_bufnumber( n_descr_buf );
        f_queue_full = 0;
        }
     else
        {
        f_queue_full = 1;
        }
    }   
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_write_PHYreg
    (
    t_uint16 PHYaddr,                       /* () -  PHY- */
    t_uint16 reg_addr,                      /* () -   */
    t_uint16 data                           /* () -  */
    )
    {
    f_poll_mdc_done();
    *pEMAC_STADAT = data;
    *pEMAC_STAADD = SET_PHYAD( PHYaddr ) | SET_REGAD( reg_addr ) | STAOP | STABUSY;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void vIVG8_handle
    (
    void
    )
    {
    /*
     *   IVG8 (DMA1 - EMAC RX).
     */
    portBASE_TYPE switch_require = pdFALSE;
     
    if (*pDMA1_IRQ_STATUS & DMA_DONE)
        {
        *pDMA1_IRQ_STATUS = DMA_DONE;
        ssync();
        
        xSemaphoreGiveFromISR( f_h_datarx_smphr, &switch_require );
    
        f_set_rxDMAdescriptor();
        }
        
    if (*pSIC_ISR & IRQ_DMA2)
        {
        *pDMA2_IRQ_STATUS = DMA_DONE;
        ssync();
        
        xSemaphoreGiveFromISR( f_h_datatx_smphr, &switch_require );
        }

            
    portEND_SWITCHING_ISR( switch_require );
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
