/* Part of FreeRTOS porting code on Blackfin processor */
/* Luke Yang <magic.yyang@gmail.com> */

#include "FreeRTOSConfig.h"
#include "context_sl_asm.h"
#include "FreeRTOSPortConfig.h"

.extern _vTaskSwitchContext;
.extern _configUSE_PREEMPTION;
.extern _vTaskIncrementTick;
#if IVG7_EN
.extern _vIVG7_handle;
#endif
#if IVG8_EN
.extern _vIVG8_handle;
#endif
#if IVG9_EN
.extern _vIVG9_handle;
#endif
#if IVG10_EN
.extern _vIVG10_handle;
#endif
#if IVG11_EN
.extern _vIVG11_handle;
#endif
#if IVG12_EN
.extern _vIVG12_handle;
#endif
#if IVG13_EN
.extern _vIVG13_handle;
#endif

.section L1_code;/*program;*/

/*
 * Manual context switch.  The first thing we do is save the registers so we
 * can use a naked attribute.
 */
 /*  This function is also the ISR for interrupt 2 */
 .global _vContextSwitch;
_vContextSwitch:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
	LINK 12;
	call.X _vTaskSwitchContext;
	UNLINK;

    _portRESTORE_CONTEXT;
_vContextSwitch.end:


/* 
 * The ISR used for the scheduler tick depends on whether the cooperative or
 * the preemptive scheduler is being used.
 */
/* The cooperative scheduler requires a normal IRQ service routine to 
simply increment the system tick. */

/* This is the ISR for core timer interrupt (default level 6)*/
.global _vTickISR;
_vTickISR:

#if (configUSE_PREEMPTION == 0)

		LINK 12;
		call.X _vTaskIncrementTick;
		UNLINK;

#else

		/* Save the context of the interrupted task. */
		_portSAVE_CONTEXT;	
		
		LINK 12;
		
		/* Increment the RTOS tick count, then look for the highest priority 
		task that is ready to run. */
		call.X _vTaskIncrementTick;
		
		call.X _vTaskSwitchContext;
		
		UNLINK;
		
		/* Restore the context of the new task. */
		_portRESTORE_CONTEXT;
	
#endif
_vTickISR.end:


.global _vIVG7_ISR;
_vIVG7_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG7_EN
	LINK 12;
	call.X _vIVG7_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG7_ISR.end:

.global _vIVG8_ISR;
_vIVG8_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG8_EN
	LINK 12;
	call.X _vIVG8_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG8_ISR.end:

.global _vIVG9_ISR;
_vIVG9_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG9_EN
	LINK 12;
	call.X _vIVG9_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG9_ISR.end:

.global _vIVG10_ISR;
_vIVG10_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG10_EN
	LINK 12;
	call.X _vIVG10_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG10_ISR.end:

.global _vIVG11_ISR;
_vIVG11_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG11_EN
	LINK 12;
	call.X _vIVG11_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG11_ISR.end:

.global _vIVG12_ISR;
_vIVG12_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG12_EN
	LINK 12;
	call.X _vIVG12_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG12_ISR.end:

.global _vIVG13_ISR;
_vIVG13_ISR:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
#if IVG13_EN
	LINK 12;
	call.X _vIVG13_handle;
	UNLINK;
#endif

    _portRESTORE_CONTEXT;
_vIVG13_ISR.end:
