/* 
	FreeRTOS.org V4.1.3 - Copyright (C) 2003-2006 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section 
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license 
	and contact details.  Please ensure to read the configuration and relevant 
	port sections of the online documentation.
	***************************************************************************
*/

#ifndef PORTMACRO_H
#define PORTMACRO_H


#include <ccblkfn.h>

/*-----------------------------------------------------------
 * Port specific definitions.  
 *
 * The settings in this file configure FreeRTOS correctly for the
 * given hardware and compiler.
 *
 * These settings should not be altered.
 *-----------------------------------------------------------
 */
 
/* Type definitions. */
#define portCHAR		char
#define portFLOAT		float
#define portDOUBLE		double
#define portLONG		long
#define portSHORT		short
#define portSTACK_TYPE	unsigned portLONG
#define portBASE_TYPE	portLONG

#if( configUSE_16_BIT_TICKS == 1 )
	typedef unsigned portSHORT portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffff
#else
	typedef unsigned portLONG portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffffff
#endif
/*-----------------------------------------------------------*/	

/* Scheduler utilities. */

extern unsigned int freertos_global_imask;
extern int g_cnt_save_imask;
extern unsigned g_save_imask;
extern unsigned g_save_imask1;

#define portDISABLE_INTERRUPTS() do {g_save_imask1 = cli();}while(0)
#define portENABLE_INTERRUPTS() do{sti(g_save_imask);}while(0)

#define portENTER_CRITICAL()		portDISABLE_INTERRUPTS()
#define portEXIT_CRITICAL()			portENABLE_INTERRUPTS()

/*-----------------------------------------------------------*/

/* Task utilities. */
extern void vContextSwitch(void);	/* interrupt 14 on BFin */
extern void vTickISR(void);
/* FreeRTOS does YIELD() in critical section... 
 * So we have to re-enable soft int 14 here
 */
#define portYIELD() 						\
	asm volatile ("[ -- SP ] = R0;	\n\t	\
					CLI R0;			\n\t	\
					BITSET (R0, 14);\n\t	\
					STI R0;			\n\t	\
					R0 = [ SP ++ ];	\n\t	\
					RAISE 14;")
					
#define vStartFirstTask() portYIELD()

/*-----------------------------------------------------------*/

/* Hardware specifics. */
#define portBYTE_ALIGNMENT			4
#define portSTACK_GROWTH			( -1 )
/* time per tick in ms */
#define portTICK_RATE_MS			( ( portTickType ) 1000 / configTICK_RATE_HZ )		
#define portNOP()					asm volatile ( "NOP;" )
/*-----------------------------------------------------------*/

/* Task utilities. */
#define portEND_SWITCHING_ISR( xSwitchRequired ) 	\
{													\
extern void vTaskSwitchContext( void );				\
													\
	if( xSwitchRequired ) 							\
	{												\
		vTaskSwitchContext();						\
	}												\
}
/*-----------------------------------------------------------*/

/* Task function macros as described on the FreeRTOS.org WEB site. */
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )
#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )

void vPortEnterCritical(void);
void vPortExitCritical(void);

extern void vIVG7_ISR(void);
extern void vIVG8_ISR(void);
extern void vIVG9_ISR(void);
extern void vIVG10_ISR(void);
extern void vIVG11_ISR(void);
extern void vIVG12_ISR(void);
extern void vIVG13_ISR(void);

#endif /* PORTMACRO_H */

