/*****************************************************************************\
*              efs - General purpose Embedded Filesystem library              *
*          --------------------- -----------------------------------          *
*                                                                             *
* Filename :  hcs12cw_spi.h                                                   *
* Description : Headerfile for hcs12cw_spi.c                                  *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; version 2                     *
* of the License.                                                             *
                                                                              *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* As a special exception, if other files instantiate templates or             *
* use macros or inline functions from this file, or you compile this          *
* file and link it with other works to produce a work based on this file,     *
* this file does not by itself cause the resulting work to be covered         *
* by the GNU General Public License. However the source code for this         *
* file must still be made available in accordance with section (3) of         *
* the GNU General Public License.                                             *
*                                                                             *
* This exception does not invalidate any other reasons why a work based       *
* on this file might be covered by the GNU General Public License.            *
*                                                                             *
*                                                    (c)2007 Konstantin Les   *
\*****************************************************************************/

#ifndef HCS12CW_SPI_H_
#define HCS12CW_SPI_H_


#include "config_efsl.h"
#include "types.h"


/*================================================================================================*/
#define SPIBR_6000KHZ (1)
#define SPIBR_750KHZ (0x13)
#define SPIBR_500KHZ (0x23)
#define SPIBR_200KHZ (0x06)
/*================================================================================================*/

/*================================================================================================*/
/*************************************************************\
              hwInterface
               ----------
* FILE* 	imagefile		File emulation of hw interface.
* long		sectorCount		Number of sectors on the file.
* euitn8    numDevice       Used SPI device number
\*************************************************************/
struct hwInterface
    {
    /*FILE *imageFile;*/
    euint32 addressWritten;
	eint32 sectorCount;
    };
typedef struct hwInterface hwInterface;
/*================================================================================================*/

/*================================================================================================*/
void if_assert_device(hwInterface *iface);
void if_deassert_device(hwInterface *iface);
esint8 if_flush(hwInterface *file);
esint8 if_initInterface(hwInterface *file, eint8 *opts);
esint8 if_readBuf(hwInterface *file, euint32 address, euint8 *buf);
esint8 if_release_device(hwInterface *iface);
esint8 if_setPos(hwInterface *file, euint32 address);
void if_spiInit(hwInterface *iface);
void if_spiReceiveBuf(hwInterface *iface, euint16 size, euint16 buflen, euint8 *buf);
euint8 if_spiSend(hwInterface *iface, euint8 outgoing);
void if_spiSendBuf(hwInterface *iface, euint16 bufsize, const euint8 *buf);
esint8 if_take_device(hwInterface *iface);
esint8 if_writeBuf(hwInterface *file, euint32 address, euint8 *buf);
/*================================================================================================*/

#endif /*#ifndef HCS12CW_SPI_H_*/
