/*****************************************************************************\
*              efs - General purpose Embedded Filesystem library              *
*          --------------------- -----------------------------------          *
*                                                                             *
* Filename : hcs12cw_spi.c                                                    *
* Description : This file contains the functions needed to use efs for        *
*               accessing files on an SD-card connected to an Freescale       *
*               HCS12 via SPI.                                                *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; version 2                     *
* of the License.                                                             *
                                                                              *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* As a special exception, if other files instantiate templates or             *
* use macros or inline functions from this file, or you compile this          *
* file and link it with other works to produce a work based on this file,     *
* this file does not by itself cause the resulting work to be covered         *
* by the GNU General Public License. However the source code for this         *
* file must still be made available in accordance with section (3) of         *
* the GNU General Public License.                                             *
*                                                                             *
* This exception does not invalidate any other reasons why a work based       *
* on this file might be covered by the GNU General Public License.            *
*                                                                             *
*                                                    (c)2006 Lennart Yseboodt *
*                                                    (c)2006 Michael De Nil   *
*                                                    (c)2007 Konstantin Les   *
\*****************************************************************************/


#include "GlobDefs.h"

#include "interfaces/bf537_spi.h"
#include "interfaces/sd.h"


/*================================================================================================*/
#define SDCARD_SPI_CS (FLG4)
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void if_assert_device
    (
    hwInterface *iface
    )
    {
    Select_CS_Slave( SDCARD_SPI_CS );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void if_deassert_device
    (
    hwInterface *iface
    )
    {
    Select_CS_Slave( 0 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_flush
    (
    hwInterface *file
    )
    {
    esint8 ret_val = 0;
    
    if (if_take_device( file ) != 0)
        return -1;
    
    ret_val = sd_Flush( file );
    
    (void)if_release_device( file );
    
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_initInterface
    (
    hwInterface *file,
    eint8 *opts                        /* (in) - used SPI device number */
    )
    {
    esint8 ret_val = 0;
    euint32 sc;
    
    if (if_take_device( file ) != 0)
        return -2;
    
    if_spiInit( file );
    
	if (sd_Init( file ) < 0)
	    {
	    ret_val = -3;
		goto ret_initInterface;
	    }
	
	if (sd_State( file ) < 0)
		{
	    ret_val = -4;
		goto ret_initInterface;
	    }
	
	(void)sd_getDriveSize( file, &sc );
	file->sectorCount = sc / 512;
	if (sc % 512 != 0)
		file->sectorCount--;
	
	/*if (!g_f)
	    goto ret_initInterface;
	aa = file->sectorCount * 512;
	for (ii = 0; (ii < aa); ii += 512)
	    {
	    euint8 resp;
	    
    	if_assert_device( file );
    	sd_Command( file, 29, (ii >> 16) & 0xFFFF, ii & 0xFFFF );
    	resp = sd_Resp8b( file );
    	if (resp != 0)
    	    {
    	    sd_Resp8bError( file, resp );
    	    return -4;
    	    }
    	    
    	while (if_spiSend( file, 0xFF ) == 0);
    	if_deassert_device( file );
	    }*/
	
	
ret_initInterface:
	if_release_device( file );
	
	return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_readBuf
    (
    hwInterface *file,
    euint32 address,
    euint8 *buf
    )
    {
    esint8 ret_val = -1;
    
    if (if_take_device( file ) != 0)
        return -1;
    
    if_spiInit( file );
    
	ret_val = sd_readSector( file, address, buf, 512 );
	
	(void)if_release_device( file );
	
	return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void if_spiReceiveBuf
    (
    hwInterface *iface,
    euint16 size,
    euint16 buflen,
    euint8 *buf
    )
    {
    while (size-- > 0)
        {
        unsigned char c;
        c = ExchangeSPI( 0xFF );
        if (buflen > 0)
            {
            *buf++ = c;
            buflen--;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_release_device
    (
    hwInterface *iface
    )
    {
    UnDumpSPISettingsStore( SPI_DEVICE_SDCARD );
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_setPos
    (
    hwInterface *file,
    euint32 address
    )
    {
	return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void if_spiInit
    (
    hwInterface *iface
    )
    {
    const euint32 ct_sdspi_clk = 10000000;
    ConfigureSPIBaud( ct_sdspi_clk );
    ConfigureSPI( START_TRANS_WITH_WRITE, true, false, false, true,	false, false, true, true, true,
        false);
    EnableSPI;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
euint8 if_spiSend
    (
    hwInterface *iface,
    euint8 outgoing
    )
    {
	euint8 incoming = 0;
	
	incoming = ExchangeSPI( outgoing );
	    
	return incoming;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void if_spiSendBuf
    (
    hwInterface *iface,
    euint16 bufsize,
    const euint8 *buf
    )
    {
    while (bufsize-- > 0)
        {
        (void)ExchangeSPI( *buf++ );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_take_device
    (
    hwInterface *iface
    )
    {
    DumpSPISettingsStore( SPI_DEVICE_SDCARD );
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
esint8 if_writeBuf
    (
    hwInterface *file,
    euint32 address,
    euint8 *buf
    )
    {
    esint8 ret_val;
    
    if (if_take_device( file ) != 0)
        return -1;
    
    if_spiInit( file );
    
	ret_val = sd_writeSector( file, address, buf );
	
	(void)if_release_device( file );
	
	return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
