#include "lwip/opt.h"
#include "lwip/sys.h"


/*================================================================================================*/
#define POST_BLOCK_TIME_MS (10000)
#define SYS_THREAD_MAX (10)
/*================================================================================================*/

/*================================================================================================*/
//struct timeoutlist
//    {
//    struct sys_timeouts timeouts;
//    xTaskHandle pid;
//    };
/*================================================================================================*/

/*================================================================================================*/
static int f_nextthread = 0;
//static struct timeoutlist f_timeoutlist[ SYS_THREAD_MAX ];
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
u32_t sys_arch_mbox_fetch
    (
    sys_mbox_t mbox,                        /* () -    */
    void **msg,                             /* () - , NULL -   */
    u32_t timeout                           /* () - -  , 0 -  */
    )
    {
    /*      (      
     * )
     *  
     *       .
     *   SYS_ARCH_TIMEOUT -    -
     */
    void *dummyptr;
    u32_t ret_val;
    portTickType start_time;
    
    start_time = xTaskGetTickCount();
    
    if (msg == NULL)
        msg = &dummyptr;

    if (timeout != 0)
        {
        if (xQueueReceive( mbox, &(*msg), timeout ) == pdTRUE)
            {
            portTickType elapsed;
            portTickType end_time;
            end_time = xTaskGetTickCount();
            if ((elapsed = (end_time - start_time) / portTICK_RATE_MS) == 0)
                elapsed = 1;
            ret_val = elapsed;
            }
        else
            {
            *msg = NULL;
            ret_val = SYS_ARCH_TIMEOUT;
            }
        }
    else
        {
        const portTickType def_timeout = 10000 / portTICK_RATE_MS;
        portTickType elapsed;
        portTickType end_time;
        while (xQueueReceive( mbox, &(*msg), def_timeout ) != pdTRUE)
            {
            continue;
            }
        end_time = xTaskGetTickCount();
        if ((elapsed = (end_time - start_time) / portTICK_RATE_MS) == 0)
            elapsed = 1;
        ret_val = elapsed;
        }
    
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
sys_prot_t sys_arch_protect
    (
    void
    )
    {
    /*     */
    vPortEnterCritical();
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

#if 0
/*------------------------------------------------------------------------------------------------*/
struct sys_timeouts *sys_arch_timeouts
    (
    void
    )
    {
    /*     -   .
     *  
     *      -  .
     */
    
    int i;
    xTaskHandle pid;
    struct sys_timeouts *ptimeout = NULL;
    
    
    pid = xTaskGetCurrentTaskHandle();
    for (i = 0; (i < f_nextthread); i++)
        {
        struct timeoutlist *tl;
        tl = &f_timeoutlist[ i ];
        if (tl->pid == pid)
            ptimeout = &(tl->timeouts);
        }
         
    return ptimeout;
    }
/*------------------------------------------------------------------------------------------------*/
#endif

/*------------------------------------------------------------------------------------------------*/
u32_t sys_arch_mbox_tryfetch
    (
    sys_mbox_t mbox,                        /* () -    */
    void **msg                              /* () - , NULL -   */
    )
    {
    /*       (   )
     *  
     *   0 -  
     *   SYS_MBOX_EMPTY -   
     */
    u32_t tryfetch_status = 0;
     
    if (xQueueReceive( mbox, &(*msg), 0 ) != pdTRUE)
        tryfetch_status = SYS_MBOX_EMPTY;
     
    return tryfetch_status;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
u32_t sys_arch_sem_wait
    (
    sys_sem_t sem,                          /* () -   */
    u32_t timeout                           /* () - -  , 0 -  */
    )
    {
    /*  ( )        - (
     * -  0 - )
     *  
     *           .
     *   SYS_ARCH_TIMEOUT -    -
     */
    u32_t ret_val = 0;
    portTickType start_time;
    
    start_time = xTaskGetTickCount();
    
    if (timeout != 0)
        {
        if (xSemaphoreTake( sem, timeout / portTICK_RATE_MS ) == pdTRUE)
            {
            portTickType elapsed;
            portTickType end_time = xTaskGetTickCount();
            elapsed = (end_time - start_time) / portTICK_RATE_MS;
            if (elapsed == 0)
                elapsed = 1;
            ret_val = elapsed;
            }
        else
            {
            ret_val = SYS_ARCH_TIMEOUT;
            }
        }
    else
        {
        /*   -,    - */
        const portTickType def_timeout = 10000 / portTICK_RATE_MS;
        portTickType elapsed;
        portTickType end_time;
        while (xSemaphoreTake( sem, def_timeout ) != pdTRUE)
            {
            continue;
            }
        end_time = xTaskGetTickCount();
        elapsed = end_time - start_time;
        if (elapsed == 0)
            elapsed = 1;
        ret_val = elapsed;
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_arch_unprotect
    (
    sys_prot_t pval
    )
    {
    /*     */
    vPortExitCritical();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_init
    (
    void
    )
    {
    /*   sys_arch */
    int i_thread;
    
//    for (i_thread = 0; (i_thread < SYS_THREAD_MAX); i_thread++)
//        {
//        f_timeoutlist[ i_thread ].pid = 0;
//        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_mbox_free
    (
    sys_mbox_t mbox                         /* () -    */
    )
    {
    /*   
     * 
     *      .    lwIP     .
     */
    vQueueDelete( mbox );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
sys_mbox_t sys_mbox_new
    (
    int size                                /* () -      */
    )
    {
    /*      
     *  
     *     .
     */
    return xQueueCreate( size, sizeof (void *) );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_mbox_post
    (
    sys_mbox_t mbox,                        /* () -    */
    void *msg                               /* () -  */
    )
    {
    /*      (   ) */
    while (xQueueSend( mbox, &msg, POST_BLOCK_TIME_MS / portTICK_RATE_MS ) != pdTRUE)
        {
        continue;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
err_t sys_mbox_trypost
    (
    sys_mbox_t mbox,                        /* () -    */
    void *msg                               /* () -  */
    )
    {
    /*       (   )
     *  
     *   ERR_MEM -   
     *   ERR_OK -      
     */
    err_t ret_val;
    
    ret_val = (xQueueSend( mbox, &msg, POST_BLOCK_TIME_MS / portTICK_RATE_MS ) == pdTRUE) ? ERR_OK :
        ERR_MEM;
    
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_sem_free
    (
    sys_sem_t sem                           /* () -   */
    )
    {
    /*  
     * 
     *    FreeRTOS     ,   .
     */
    
    vQueueDelete( sem );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
sys_sem_t sys_sem_new
    (
    u8_t count                              /* () -    */
    )
    {
    /*   
     *  
     *    .
     */
    xSemaphoreHandle xSemaphore;
    
    portENTER_CRITICAL();
    vSemaphoreCreateBinary( xSemaphore );
    if (count == 0)
        {
        xSemaphoreTake( xSemaphore, 1 );
        }
    portEXIT_CRITICAL();
    
    return xSemaphore;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void sys_sem_signal
    (
    sys_sem_t sem                           /* () -   */
    )
    {
    /*   */
    
    xSemaphoreGive( sem );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
sys_thread_t sys_thread_new
    (
    char *name,                             /* () -   */
    void (*thread)(void *arg),              /* () - ,     */
    void *arg,                              /* () -     */
    int stacksize,                          /* () -    */
    int prio                                /* () -   */
    )
    {
    /*  
     *  
     *    .
     */
    xTaskHandle h_created_task;
    
    (void)xTaskCreate( thread, name, stacksize, arg, prio, &h_created_task );
        
//    f_timeoutlist[ f_nextthread++ ].pid = h_created_task;
    
    return h_created_task;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
