#include "HAL.h" 

void OpenATFlashInterface(void)
{
    while (*pSPI_STAT&TXS); // ,    
    
    DisableSPI;
    Select_CS_Slave(0); //   CS Flash     ,         
    PF_PINS_SET(PF2|PF3|PF4|PF5|PF6|PF7); //        
    ConfigureSPIBaud(ATFlashClkFreq); // 
    ConfigureSPI(START_TRANS_WITH_WRITE,
				  false,//bool SendLastWord,
				  true, //bool OverwriteDataOnOverflow,
				  true, //bool SlaveDetectEnable,
				  true, //bool EnableMISO,
				  false,//bool WordSize16Bit,
				  false, //bool LSB1, 
				  true,//bool CPHA1,	
				  true,//bool CPOL1,
				  true, //bool Master,
				  false //bool WriteOpenDrainMaster
				  );
	EnableSPI;
	CLR_ATFlash_CS;
	WaitFlashReady();		
} 

void SetFlashToPowerOfTwo(void)
{	
	SET_ATFlash_CS;	
	SendFlashCommandOnly(0xa6802a3d);
	CLR_ATFlash_CS;		
}

void Disable_Sector_Protection(void)
{
	SET_ATFlash_CS;		
	SendFlashCommandOnly(0x9a7f2a3d);		
	CLR_ATFlash_CS;	
}

void Enable_SectorProtection(void)
{
	SET_ATFlash_CS;		
	SendFlashCommandOnly(0xa97f2a3d);
	CLR_ATFlash_CS;	
}


//------------------------------------------------------------

void SendFlashCommand(BYTE Command, DWORD Address)
{
	BYTE Cmd[4];
	Cmd[0]=Command;
	Cmd[1]=(BYTE)(Address>>16);
	Cmd[2]=(BYTE)(Address>>8);
	Cmd[3]=(BYTE)(Address);
	WriteSPIBytes_asm(Cmd, 4);
}

void SendFlashCommandOnly(DWORD Command)
{	
	WriteSPIBytes_asm((BYTE *)&Command, sizeof(Command));
}


void ReadFlashPageIntoBufer1(DWORD Address)
{
	SET_ATFlash_CS;	
	SendFlashCommand(FLASH_CMD_MAIN_MEM_TO_BUFFER1, Address);		
	CLR_ATFlash_CS;			
	WaitFlashReady();						
}

void WriteFlashBuffer1(DWORD Address, BYTE * Data, DWORD NumData)
{
	SET_ATFlash_CS;
	SendFlashCommand(FLASH_CMD_BUFFER1_WRITE, Address);
	WriteSPIBytes_asm(Data,NumData);			
	CLR_ATFlash_CS;	
}

void ProgrammBuffer1_WithErase(DWORD Address)
{
	SET_ATFlash_CS;
	SendFlashCommand(FLASH_CMD_BUFFER1_PROGRAMM_WITH_ERASE, Address);		
	CLR_ATFlash_CS;	
}

void ProgrammFlash(DWORD Address, BYTE * Data, DWORD NumData)
{	
	if (Address&(FLASH_PAGE_MASK))
	{ //    
		DWORD NumDataToWrite = Address&(FLASH_PAGE_MASK);
		DWORD PageAddress = Address&(~FLASH_PAGE_MASK);

		ReadFlashPageIntoBufer1(PageAddress);
		WriteFlashBuffer1(PageAddress,Data,NumDataToWrite);
		ProgrammBuffer1_WithErase(Address);
		WaitFlashReady();

		NumData-=NumDataToWrite;
		Data+=NumDataToWrite;
		Address+=NumDataToWrite;		
	}

	while (NumData>=FLASH_PAGE_SIZE)
	{
		DWORD NumDataToWrite = min(FLASH_PAGE_SIZE, NumData);		
		WriteFlashBuffer1(0,Data,NumDataToWrite);
		ProgrammBuffer1_WithErase(Address);
		WaitFlashReady();

		NumData-=NumDataToWrite;
		Data+=NumDataToWrite;
		Address+=NumDataToWrite;		
	}   

	if(NumData)
	{
		ReadFlashPageIntoBufer1(Address);
		WriteFlashBuffer1(0,Data,NumData);
		ProgrammBuffer1_WithErase(Address);
		WaitFlashReady();				
	}	
}


void ContiniousReadFlash(DWORD Address, BYTE * Data, DWORD NumData)
{
	SET_ATFlash_CS;
	SendFlashCommand(FLASH_CMD_CONTINIUOS_ARRAY_READ_LEGACY, Address);
	SendFlashCommand(FLASH_CMD_CONTINIUOS_ARRAY_READ_LEGACY, Address); // Don't Care Byte				
	ReadSPIBytes_asm(Data, NumData);
	CLR_ATFlash_CS;	
}

void GetFlashChipId(BYTE * pData)
{
	SET_ATFlash_CS;
	WriteSPI_asm(FLASH_CMD_GET_CHIP_ID);
	ReadSPIBytes_asm(pData, 4);					
	CLR_ATFlash_CS;	
}

void EraseFlashSector(BYTE Sector)
{
	SET_ATFlash_CS;
	DWORD Address=(DWORD)Sector<<12;
	SendFlashCommand(FLASH_CMD_SECTOR_ERASE, Address);		
	CLR_ATFlash_CS;			
	WaitFlashReady();						
}

void GetFlashStatus(BYTE * Data)
{
	SET_ATFlash_CS;
	WriteSPI_asm(FLASH_CMD_STATUS_REGISTER_READ);
	*Data=ReadSPI_asm();			
	CLR_ATFlash_CS;	
}

void WaitFlashReady(void)
{
	BYTE Data=0;	
	while (!(Data&(1<<7)))
	{
		GetFlashStatus(&Data);		
	}	
}

BYTE GetFlashSectorProtectionRegister(void)
{
    SET_ATFlash_CS;
	WriteSPI_asm(FLASH_CMD_SECTOR_PROTECTION_READ);
	WriteSPI_asm(0);
	BYTE Data=ReadSPI_asm();					
	CLR_ATFlash_CS;	
	return Data;
}

BYTE GetFlashSectorLockDownRegister(void)
{	
	SET_ATFlash_CS;
	WriteSPI_asm(FLASH_CMD_SECTOR_LOCKDOWN_READ);
	WriteSPI_asm(0);
	BYTE Data=ReadSPI_asm();
	CLR_ATFlash_CS;	
	return Data;
}

BYTE GetFlashSecurityRegister(void)
{
    SET_ATFlash_CS;
	WriteSPI_asm(FLASH_CMD_SECURITY_READ);
	WriteSPI_asm(0);
	BYTE Data=ReadSPI_asm();
	CLR_ATFlash_CS;	
	return Data;
}

