#include "HAL.h" 

void PrepareDescrDMAAndStart(DMA_CHANNEL_ID Channel,  DMA_FLEX_DESCRIPTOR_SIZE DescriptorSize, DMA_FLOW FlowType)
{ //  -      
    WORD * pReg=(WORD *)(DMA0_CONFIG+Channel);
    *pReg=0; //  DMa   
    ssync();
    *(WORD *)(DMA0_IRQ_STATUS+Channel)=0x3; //   
    *pReg=PrepareDMAConfig(false, 0, false, DescriptorSize ,FlowType, true);
}

void ClearDMARegisters(DMA_CHANNEL_ID Channel)
{    
    *(WORD *)(DMA0_CONFIG+Channel)=0; //   
    *(WORD *)(DMA0_IRQ_STATUS+Channel)=0x3; //   
    *(DWORD *)(DMA0_START_ADDR+Channel)=0; 	
 	*(WORD *)(DMA0_X_MODIFY+Channel)=0;
 	*(WORD *)(DMA0_X_COUNT+Channel)=0;
 	*(WORD *)(DMA0_Y_MODIFY+Channel)=0;
 	*(WORD *)(DMA0_Y_COUNT+Channel)=0;
 	*(DWORD *)(DMA0_CURR_ADDR+Channel)=0;
}

DWORD GetDMACurrAddress(DMA_CHANNEL_ID Channel)
{
    return *(DWORD *)(DMA0_CURR_ADDR+Channel);
}

void SetDMACurrAddress(DMA_CHANNEL_ID Channel, DWORD Value)
{
    *(DWORD *)(DMA0_CURR_ADDR+Channel)=Value;
}

WORD PauseDMA(DMA_CHANNEL_ID Channel, bool Pause)
{
    if (Pause)
    {
     (*(WORD *)(DMA0_CONFIG+Channel))&=~DMAEN;
    }
    else
    {
        (*(WORD *)(DMA0_CONFIG+Channel))|=DMAEN;
    }
    WORD TempData = (*(WORD *)(DMA0_CONFIG+Channel));
    ssync();
    return TempData;
}

void PrepareRegDMAAndStart(DMA_CHANNEL_ID Channel, bool DirDestination,
			DMA_TRANSFER_WORD_SIZE WordSize,
			bool AllowInterrupt, DMA_FLEX_DESCRIPTOR_SIZE DescriptorSize,
			DMA_FLOW FlowType, bool Start)
{ //  -  
    WORD * pReg=(WORD *)(DMA0_CONFIG+Channel);
    *pReg=0; //  DMa   
    *(WORD *)(DMA0_IRQ_STATUS+Channel)=0x3; //   
    WORD Config = PrepareDMAConfig(DirDestination, WordSize, AllowInterrupt, DescriptorSize ,FlowType, true);    
    if (!Start) Config&=~DMAEN;
    *pReg=Config;
}


void ConfigureDMAChannelDesc(DMA_CHANNEL_ID Channel, DMA_DESCRIPTOR_UNION * pCurrentDescriptor)
{ 	 	
 	*(DWORD *)(DMA0_NEXT_DESC_PTR+Channel)=(DWORD)pCurrentDescriptor; 	
}

void ConfigureDMAChannelPerif(DMA_CHANNEL_ID Channel,DMA_PMAP Perif)
{
	*(WORD *)(DMA0_PERIPHERAL_MAP+Channel)=Perif;
}

void ConfigureDMAChannelReg(DMA_CHANNEL_ID Channel, DWORD pStartAddress, 
							WORD Counter, WORD CounterModify)
{	
 	*(DWORD *)(DMA0_START_ADDR+Channel)=pStartAddress; 	
 	*(WORD *)(DMA0_X_MODIFY+Channel)=CounterModify;
 	*(WORD *)(DMA0_X_COUNT+Channel)=Counter;
}

void ConfigureDMAChannelRegAndStart(DMA_CHANNEL_ID Channel, DWORD pStartAddress, 
							WORD Counter)
{	
    (*(WORD *)(DMA0_CONFIG+Channel))&=~DMAEN;
    ssync();
 	*(DWORD *)(DMA0_START_ADDR+Channel)=pStartAddress; 	
 	*(WORD *)(DMA0_X_COUNT+Channel)=Counter;
 	(*(WORD *)(DMA0_CONFIG+Channel))|=DMAEN;
}


void StartDMA(DMA_CHANNEL_ID Channel, DMA_FLOW FlowType)
{    
    (*(WORD *)(DMA0_CONFIG+Channel))|=DMAEN|(FlowType<<12);
}

WORD StopDMA(DMA_CHANNEL_ID Channel)
{
    (*(WORD *)(DMA0_CONFIG+Channel))=0;
    WORD TempData = (*(WORD *)(DMA0_CONFIG+Channel));
    ssync();
    return TempData;
}

#if (defined (__ADSPBF534__)||defined (__ADSPBF536__)||defined (__ADSPBF537__))

void SetupHDMA0(HDMA_CHANNEL_ID Channel, bool RequestPolarity, bool ReloadBCount, t_HDMA_Request Request)
{	
	(*(WORD *)(HMDMA0_CONTROL+Channel))=0;
	(*(WORD *)(HMDMA0_CONTROL+Channel))=HMDMAEN|(RequestPolarity<<1)|(Request<<8)|(ReloadBCount<<12);
}

void SetHDMAParam(HDMA_CHANNEL_ID Channel, WORD ECInit, WORD BCInit)
{
	(*(WORD *)(HMDMA0_BCINIT+Channel))=BCInit;
	(*(WORD *)(HMDMA0_ECINIT+Channel))=ECInit;
}

#endif



// EXAMPLES
/*
   DMA    , 
   ,    
 FLOW_STOP -    

*/

	/*    
	// single transmit mode
    ConfigureDMAChannelReg(ADI_DMA_MDMA_S0, (DWORD)TestRingArray, 32, 2);	       	
	ConfigureDMAChannelReg(ADI_DMA_MDMA_D0, (DWORD)pBULK_DATA, 32, 0);	       
	            	            
	PrepareRegDMAAndStart(ADI_DMA_MDMA_S0, false, DMA_WORD_16,
					false, DMA_DESCR_0, ADI_DMA_FLOW_STOP);
    PrepareRegDMAAndStart(ADI_DMA_MDMA_D0, true, DMA_WORD_16,
					false, DMA_DESCR_0, ADI_DMA_FLOW_STOP);	     
					
	// autobuffer mode
    ConfigureDMAChannelReg(ADI_DMA_MDMA_S0, (DWORD)Buf1, 32, 2);	       	
	ConfigureDMAChannelReg(ADI_DMA_MDMA_D0, (DWORD)TestRingArray, 32, 2);	       
	            	            
	PrepareRegDMA(ADI_DMA_MDMA_S0, false, DMA_WORD_16,
					false, DMA_DESCR_0, ADI_DMA_FLOW_AUTOBUFFER);
    PrepareRegDMA(ADI_DMA_MDMA_D0, true, DMA_WORD_16,
					false, DMA_DESCR_0, ADI_DMA_FLOW_AUTOBUFFER);	     
					
	 // Large array
	DMA_DESCRIPTOR_UNION BufDescrUn;
	DMA_DESCRIPTOR_UNION ArrayDescrUn;
	
	BufDescrUn.Large.pNext=NULL;
	BufDescrUn.Large.StartAddress=TestRingArray;
	BufDescrUn.Large.Config.VALUE=PrepareDMAConfig(false, DMA_WORD_16, false, DMA_DESCR_0, ADI_DMA_FLOW_STOP);
	BufDescrUn.Large.XCount=32;
	BufDescrUn.Large.XModify=2;
	
	ArrayDescrUn.Large.pNext=NULL;
	ArrayDescrUn.Large.StartAddress=(void *)pBULK_DATA;
	ArrayDescrUn.Large.Config.VALUE=PrepareDMAConfig(true, DMA_WORD_16, false, DMA_DESCR_0, ADI_DMA_FLOW_STOP);
	ArrayDescrUn.Large.XCount=32;
	ArrayDescrUn.Large.XModify=2;    
    
    ConfigureDMAChannelDesc(ADI_DMA_MDMA_S0, &BufDescrUn);
	ConfigureDMAChannelDesc(ADI_DMA_MDMA_D0, &ArrayDescrUn);
	
	PrepareDescrDMAAndStart(ADI_DMA_MDMA_S0,  DMA_DESCR_7, ADI_DMA_FLOW_LARGE);
	PrepareDescrDMAAndStart(ADI_DMA_MDMA_D0,  DMA_DESCR_7, ADI_DMA_FLOW_LARGE);    	   
    	

    */
