#ifndef DMA_H_
#define DMA_H_

typedef enum 
{
    DMA_WORD_8=WDSIZE_8,
    DMA_WORD_16=WDSIZE_16,
    DMA_WORD_32=WDSIZE_32
} DMA_TRANSFER_WORD_SIZE;

typedef enum
{
    DMA_DESCR_0,
    DMA_DESCR_1,
    DMA_DESCR_2,
    DMA_DESCR_3,
    DMA_DESCR_4,
    DMA_DESCR_5,
    DMA_DESCR_6,
    DMA_DESCR_7,
    DMA_DESCR_8,
    DMA_DESCR_9
}DMA_FLEX_DESCRIPTOR_SIZE;

typedef enum {                          // DMA Channel ID
#if defined (__ADSPBF533__)
    DMA_DMA0=0,                         // DMA 0 - default channel for PPI.
    DMA_DMA1=1*0x40,                    // DMA 1 - default channel for SPORT0 receive.
    DMA_DMA2=2*0x40,                    // DMA 2 - default channel for SPORT0 transmit.
    DMA_DMA3=3*0x40,                    // DMA 3 - default channel for SPORT1 receive.
    DMA_DMA4=4*0x40,                    // DMA 4 - default channel for SPORT1 transmit
    DMA_DMA5=5*0x40,                    // DMA 5 - default channel for SPI.
    DMA_DMA6=6*0x40,                    // DMA 6 - default channel for UART receive.
    DMA_DMA7=7*0x40,                    // DMA 7 - default channel for UART transmit.
	DMA_MDMA_D0=8*0x40,                 // Memory DMA Stream 0 Destination.
    DMA_MDMA_S0=9*0x40,                 // Memory DMA Stream 0 Source.
    DMA_MDMA_D1=10*0x40,                // Memory DMA Stream 1 Destination.
    DMA_MDMA_S1=11*0x40                 // Memory DMA Stream 1 Source.    
#elif (defined (__ADSPBF534__)||defined (__ADSPBF536__)||defined (__ADSPBF537__))
	DMA_DMA0=0,                          // DMA 0 - default channel for PPI.
    DMA_DMA1=1*0x40,                     // DMA 1 - MAC recieve
    DMA_DMA2=2*0x40,                     // DMA 2 - MAC transmit
    DMA_DMA3=3*0x40,                     // DMA 3 - default channel for SPORT0 receive.
    DMA_DMA4=4*0x40,                     // DMA 4 - default channel for SPORT0 transmit.
    DMA_DMA5=5*0x40,                     // DMA 5 - default channel for SPORT1 receive.
    DMA_DMA6=6*0x40,                     // DMA 6 - default channel for SPORT1 transmit
    DMA_DMA7=7*0x40,                     // DMA 7 - default channel for SPI.
	DMA_DMA8=8*0x40,                     // DMA 8 - default channel for UART0 recieve
	DMA_DMA9=9*0x40,                     // DMA 9 - default channel for UART0 transmit.
	DMA_DMA10=10*0x40,                   // DMA 10 - default channel for UART1 recieve.
	DMA_DMA11=11*0x40,                   // DMA 11 - default channel for UART1 transmit.
	DMA_MDMA_D0=12*0x40,                 // Memory DMA Stream 0 Destination.
    DMA_MDMA_S0=13*0x40,                 // Memory DMA Stream 0 Source.
    DMA_MDMA_D1=14*0x40,                 // Memory DMA Stream 1 Destination.
    DMA_MDMA_S1=15*0x40,                 // Memory DMA Stream 1 Source.        
#endif    
} DMA_CHANNEL_ID;

typedef enum 
{                          // Values to use when setting/sensing mapping	
#if defined (__ADSPBF533__)    
	DMA_PMAP_PPI        = 0x0000,  // PPI
    DMA_PMAP_SPORT0_RX  = 0x1000,  // SPORT0 RX
    DMA_PMAP_SPORT0_TX  = 0x2000,  // SPORT0 TX
    DMA_PMAP_SPORT1_RX  = 0x3000,  // SPORT1 RX
    DMA_PMAP_SPORT1_TX  = 0x4000,  // SPORT1 TX
    DMA_PMAP_SPI        = 0x5000,  // SPI
    DMA_PMAP_UART_RX    = 0x6000,  // UART RX
    DMA_PMAP_UART_TX    = 0x7000,  // UART TX    
#elif (defined (__ADSPBF534__)||defined (__ADSPBF536__)||defined (__ADSPBF537__))
	DMA_PMAP_PPI        = 0x0000,  // PPI
    DMA_PMAP_MAC_RX  	= 0x1000,  // SPORT0 RX
    DMA_PMAP_MAC_TX  	= 0x2000,  // SPORT0 TX
    DMA_PMAP_SPORT0_RX  = 0x3000,  // SPORT0 RX
    DMA_PMAP_SPORT0_TX  = 0x4000,  // SPORT0 TX
    DMA_PMAP_SPORT1_RX  = 0x5000,  // SPORT1 RX
    DMA_PMAP_SPORT1_TX  = 0x6000,  // SPORT1 TX
    DMA_PMAP_SPI        = 0x7000,  // SPI
    DMA_PMAP_UART0_RX    = 0x8000, // UART RX
    DMA_PMAP_UART0_TX    = 0x900,  // UART TX
    DMA_PMAP_UART1_RX    = 0xA000, // UART RX
    DMA_PMAP_UART1_TX    = 0xB000, // UART TX    
#endif       
    DMA_PMAP_UNDEFINED  = 0xf000   // undefined (available for use)
} DMA_PMAP;


typedef enum {
    DMA_FLOW_STOP       =0,
    DMA_FLOW_AUTOBUFFER =1,
    DMA_FLOW_ARRAY      =4,
    DMA_FLOW_SMALL      =6,
    DMA_FLOW_LARGE      =7
} DMA_FLOW;



#pragma pack (2)

typedef union  
{    // union of all descriptor types
   	struct 
	{
	    volatile WORD b_DMA_EN:      1;  //0     Enabled
	    volatile WORD b_WNR:         1;  //1     Direction
	    volatile WORD b_WDSIZE:      2;  //2:3   Transfer word size
	    volatile WORD b_DMA2D:       1;  //4     DMA mode
	    volatile WORD b_RESTART:     1;  //5     Retain FIFO
	    volatile WORD b_DI_SEL:      1;  //6     Data interrupt timing select
	    volatile WORD b_DI_EN:       1;  //7     Data interrupt enabled
	    volatile WORD b_NDSIZE:      4;  //8:11  Flex descriptor size
	    volatile WORD b_FLOW:        3;  //12:14 Flow
	    volatile WORD b_RESERVED:    1;  //15    Reserved
   	};
   	WORD VALUE;
} DMA_CONFIG_REG;

typedef struct DMA_DESCRIPTOR_SMALL 
{           // DMA descriptor small model
    WORD                             pNext;              // next descriptor in the chain
    WORD                             StartAddressLow;    // lower 16 bits of the start address of the data
    WORD                             StartAddressHigh;   // upper 16 bits of the start address of the data
    DMA_CONFIG_REG              	Config;             // configuration control register
    WORD                             XCount;             // X (column) count
    short                             XModify;            // X (column) stride
    WORD                             YCount;             // Y (row) count
    short                             YModify;            // Y (row) stride
} DMA_DESCRIPTOR_SMALL;

typedef struct DMA_DESCRIPTOR_LARGE_Struct
{           // DMA descriptor large model
    struct DMA_DESCRIPTOR_LARGE_Struct *pNext;             // next descriptor in the chain
    void                            *StartAddress;      // start address of the data
    DMA_CONFIG_REG                  Config;             // configuration control register
    WORD                             XCount;             // X (column) count
    short                             XModify;            // X (column) stride
    WORD                             YCount;             // Y (row) count
    short                             YModify;            // Y (row) stride
    WORD                             Padding;            // necessary for 32 bit alignment
} DMA_DESCRIPTOR_LARGE;

typedef struct 
{           // DMA descriptor array model
    void                            *StartAddress;      // start address of the data
    DMA_CONFIG_REG              	Config;             // configuration control register
    WORD                             XCount;             // X (column) count
    short                             XModify;            // X (column) stride
    WORD                             YCount;             // Y (row) count
    short                             YModify;            // Y (row) stride
    WORD                             Padding;            // necessary for 32 bit alignment
} DMA_DESCRIPTOR_ARRAY;

typedef struct DMA_DESCRIPTOR_TINY_Struct
{
	struct DMA_DESCRIPTOR_TINY_Struct* 	pNext;
	void * 					StartAddress;
	DMA_CONFIG_REG			Config;
	WORD                    XCount;
} DMA_DESCRIPTOR_TINY;

#pragma pack ()



typedef union  
{    // union of all descriptor types
    DMA_DESCRIPTOR_SMALL    Small;          // small model descriptor
    DMA_DESCRIPTOR_LARGE    Large;          // large model descriptor
    DMA_DESCRIPTOR_ARRAY    Array;          // array of descriptors
} DMA_DESCRIPTOR_UNION;

#if (defined (__ADSPBF534__)||defined (__ADSPBF536__)||defined (__ADSPBF537__))
typedef enum
{
	MDMA_No_Transfer=0,
	MDMA_Single_Transfer=1,
	MDMA_Multy_Transfer=2,
	MDMA_Urgent_Transfer=3
}t_HDMA_Request;

typedef enum
{
	HDMA_CHANNEL0=0,
	HDMA_CHANNEL1=1*0x40
} HDMA_CHANNEL_ID;

void SetupHDMA0(HDMA_CHANNEL_ID Channel, bool RequestPolarity, bool ReloadBCount, t_HDMA_Request Request);
void SetHDMAParam(HDMA_CHANNEL_ID Channel, WORD ECInit, WORD BCInit);
#endif

#define DMA_MAX_TRANSFER_COUNT	(1024*64)

extern DMA_DESCRIPTOR_UNION STOPDescr;

inline bool DMA_ENABLED(DMA_CHANNEL_ID Channel)
{
    return (*(WORD *)(DMA0_CONFIG+Channel))&DMAEN;
}

#define PrepareDMAConfig(DirDestination, WordSize,  AllowInterrupt, DescriptorSize, FlowType, RestartDMA) (DMAEN|(DirDestination<<1)|(WordSize)|(RestartDMA<<5)|(AllowInterrupt<<7)|(DescriptorSize<<8)|(FlowType<<12))



void ConfigureDMAChannelPerif(DMA_CHANNEL_ID Channel,DMA_PMAP Perif);

void ConfigureDMAChannelReg(DMA_CHANNEL_ID Channel, DWORD pStartAddress, 
							WORD Counter, WORD CounterModify);

void ConfigureDMAChannelRegAndStart(DMA_CHANNEL_ID Channel, DWORD pStartAddress, 
							WORD Counter);							
														
void ConfigureDMAChannelDesc(DMA_CHANNEL_ID Channel, DMA_DESCRIPTOR_UNION * pCurrentDescriptor);


			
void StartDMA(DMA_CHANNEL_ID Channel, DMA_FLOW FlowType);
WORD StopDMA(DMA_CHANNEL_ID Channel);

WORD PauseDMA(DMA_CHANNEL_ID Channel, bool Pause);

void PrepareRegDMAAndStart(DMA_CHANNEL_ID Channel, bool DirDestination,
			DMA_TRANSFER_WORD_SIZE WordSize,
			bool AllowInterrupt, DMA_FLEX_DESCRIPTOR_SIZE DescriptorSize,
			DMA_FLOW FlowType , bool Start);
void PrepareDescrDMAAndStart(DMA_CHANNEL_ID Channel,  DMA_FLEX_DESCRIPTOR_SIZE DescriptorSize, DMA_FLOW FlowType);			


void ClearDMARegisters(DMA_CHANNEL_ID Channel);

DWORD GetDMACurrAddress(DMA_CHANNEL_ID Channel);
void SetDMACurrAddress(DMA_CHANNEL_ID Channel, DWORD Value);



#endif /*#ifndef DMA_H_*/
