#include "HAL.h"

#if (defined (__ADSPBF536__)||defined (__ADSPBF537__))
//
//		Read an off-chip register in a PHY through the MDC/MDIO port
//


WORD RdPHYReg(WORD PHYAddr, WORD RegAddr)
{
    PollMdcDone;
    
	*pEMAC_STAADD = SET_PHYAD(PHYAddr) | SET_REGAD(RegAddr) |
		EMAC2_STAOP_RD | STABUSY;

	PollMdcDone;

	return (WORD)*pEMAC_STADAT;
}

//
//		Write an off-chip register in a PHY through the MDC/MDIO port
//

void WrPHYReg(WORD PHYAddr, WORD RegAddr, WORD Data)
{    
	*pEMAC_STADAT = Data;

	*pEMAC_STAADD = SET_PHYAD(PHYAddr) | SET_REGAD(RegAddr) |
		EMAC2_STAOP_WR | STABUSY;

	PollMdcDone;
}


void ProcessNegotiationResult(t_ETHERNET_BF537_CONF *dev)
{    
    unsigned short phydat;
    if (dev->Negotiate) 
	{ // start auto negotiation											
		while (!(RdPHYReg(dev->PhyAddr, PHYREG_MODESTAT)&0x20))
		{ // wait negotiation complited		    
			phydat=RdPHYReg(dev->PhyAddr, PHYREG_ANAR);				    
			if(phydat&(1<<13))
			{
			    dev->Negotiate=false; // remote fault detected
			    break; 
			}
		}
		
		if (!(RdPHYReg(dev->PhyAddr, PHYREG_ANER)&0x1))
		{ // auto Negotiation Impossible
			dev->Negotiate=false;	
		}	
	
		if(dev->Negotiate)
		{ //    
			phydat=RdPHYReg(dev->PhyAddr, PHYREG_ANLPAR);
			if(phydat&((1<<6)|(1<<8))) dev->FullDuplex=true; // full duplex able			
			else dev->FullDuplex=false;	
	
			if(phydat&((1<<7)|(1<<8)|(1<<9))) dev->Port10 = false; // 100 Mbit		
			else dev->Port10 = true;		
			
			dev->FlowControl = (phydat &(1<<10));			
		}
		else
		{ //   10 Mbit half duplex
		    dev->FullDuplex=false;		
			dev->Port10 = true;			
		}	
	}	
	else
	{			
		while (RdPHYReg(dev->PhyAddr, PHYREG_MODESTAT)==0x8000); //  	
	}   
	
	if(dev->FullDuplex)
	{
	    dev->OpMode|=FDMODE;
	    dev->OpMode&=~DRO;
	}
	else dev->OpMode&=~FDMODE;		
	if (dev->RMIIType) dev->OpMode|=RMII;
	else dev->OpMode&=~RMII;					
	*pEMAC_OPMODE=dev->OpMode;	
	
	if (dev->FlowControl) *pEMAC_FLC = FLCE;		/* flow control enabled */										 
	else *pEMAC_FLC&= ~FLCE;
}


//
//		set up the phy
//
void SetPhy(t_ETHERNET_BF537_CONF *dev)
{   
    WORD phydat=0;			
    
	//	Program PHY registers			
	
	// issue a reset
	WrPHYReg(dev->PhyAddr, PHYREG_MODECTL, MODE_RST);	
	DelayMs(1); //   
	while (RdPHYReg(dev->PhyAddr, PHYREG_MODESTAT)==0x8000); //  			
	
	DWORD PhyID=RdPHYReg(dev->PhyAddr, PHYREG_PHYID1);
    PhyID|=RdPHYReg(dev->PhyAddr, PHYREG_PHYID2)<<16;		
	
    dev->PhyID=PhyID;
	if (!dev->EnableAutoMDIX)
	{
		WrPHYReg(dev->PhyAddr, 27, (1<<15)|(0<<13));
	}    
	
	if (dev->FullDuplex)    phydat |= (1 << 8);		// full duplex		
	if (!dev->Port10)       phydat |= (1 << 13);	// 100 Mbps	
	if (dev->Loopback)      phydat |= (1 << 14);	// enable TX->RX loopback
	WrPHYReg(dev->PhyAddr, PHYREG_MODECTL, phydat);		
	
	if (dev->Negotiate)
	{		
		WORD DeviceCapabilities = RdPHYReg(dev->PhyAddr, PHYREG_ANAR);
		DeviceCapabilities&=~0x1E0;
		DeviceCapabilities|=(1<<5); // 10 Mbit
		if (dev->FullDuplex)    DeviceCapabilities|=(1<<6);
		if (!dev->Port10)       DeviceCapabilities|=(1<<7);
		if (dev->FullDuplex&&!dev->Port10) DeviceCapabilities|=(1<<8);
		WrPHYReg(dev->PhyAddr, PHYREG_ANAR, DeviceCapabilities);
		
		phydat |= ((1<<12)|(1<<9));		// enable auto negotiation and restart
		WrPHYReg(dev->PhyAddr, PHYREG_MODECTL, phydat);						    
	    
		while (!(RdPHYReg(dev->PhyAddr, PHYREG_MODESTAT)&(1<<5)))
		{ // wait negotiation complited					    
			if(RdPHYReg(dev->PhyAddr, PHYREG_ANAR)&(1<<13)) break; // remote fault detected
		}
	}	
		
	ProcessNegotiationResult(dev);	

	// check for SMSC PHY
	if ((dev->PhyID&0xFFFF)==0x0007&&(((dev->PhyID&0xFFF00000)==0xC0A00000)||((dev->PhyID&0xFFF00000)==0xC0C00000)))
	{ //  93185  LAN8700	 
		// we have SMSC PHY so reqest interrupt on link down condition
		WrPHYReg(dev->PhyAddr, PHYREG_INT_SOURMASK, 0x0ff); // enable interrupts
		// enable PHY_INT
		DWORD sysctl = *pEMAC_SYSCTL;

		sysctl |= 0x1;
		*pEMAC_SYSCTL = sysctl;
	}
	ssync();
}


//
//		Set MAC address
//
void SetMacAddr(BYTE *mac)
{
	*pEMAC_ADDRLO = GetDWORDFromAddr_asm(&mac[0]);
	*pEMAC_ADDRHI = GetWORDFromAddr_asm(&mac[4]);
}

//
//		Get MAC address
//
void GetMacAddr(BYTE *mac)
{
	SetDWORDToAddr_asm(&mac[0], *pEMAC_ADDRLO);
	SetWORDToAddr_asm(&mac[4], *pEMAC_ADDRHI);
}

void WaitLinkUp(t_ETHERNET_BF537_CONF *dev)
{
    while (!(RdPHYReg(dev->PhyAddr, PHYREG_MODESTAT)&0x4));
}


#endif

