#ifndef EMAC_H_
#define EMAC_H_


// -----------------------------------------------------------------------
//                     PHY REGISTER NAMES				//
// -----------------------------------------------------------------------
#define PHYREG_MODECTL		0x0000
#define PHYREG_MODESTAT		0x0001
#define PHYREG_PHYID1		0x0002
#define PHYREG_PHYID2		0x0003
#define PHYREG_ANAR			0x0004
#define PHYREG_ANLPAR		0x0005
#define PHYREG_ANER			0x0006
#define PHYREG_NSR			0x0010
#define PHYREG_LBREMR		0x0011
#define PHYREG_REC			0x0012
#define PHYREG_10CFG		0x0013
#define PHYREG_PHY1_1		0x0014
#define PHYREG_PHY1_2		0x0015
#define PHYREG_PHY2			0x0016
#define PHYREG_TW_1			0x0017
#define PHYREG_TW_2			0x0018
#define PHYREG_TEST			0x0019
#define PHYREG_INT_SOURCE	0x001D
#define PHYREG_INT_SOURMASK 0x001E
#define PHYREG_SPECIAL      0x001F

#define	EMAC2_STAOP_RD	( 0<< 1 )	// Station Management Operation=Read
#define	EMAC2_STAOP_WR	( 1<< 1 )	// Station Management Operation=Write

#define MODE_RST			(1<<15)
//   LAN 8700  LAN83c185
#define PHY_INT_AN_Page_recv	(1<<1)
#define PHY_INT_Par_Det_Fall	(1<<2)
#define PHY_INT_AN_LP_Ack		(1<<3)
#define PHY_INT_Link_Down		(1<<4)
#define PHY_INT_Remote_Fall		(1<<5)
#define PHY_INT_AN_Complete		(1<<6)
#define PHY_INT_ENERGYON		(1<<7)


typedef struct 
    {
    DWORD NumBytes;
    WORD NumBuffers;
    } t_BUFFER_STATUS;

#pragma pack(4)
typedef struct
    {	
	bool 					Open;			// device open
	bool					Started;		// device started
	bool					Closing;		// refuse new requests		
	
	DWORD 					OpMode;
	DWORD					MDC_Freq;		// clock in value in MHZ
	
	bool					FlowEnabled;	// record if data flow is active	
	WORD					PhyAddr;		// PHY address
	DWORD                   PhyID;	
	bool 					RMIIType;
	bool					Port10;			// set port speed to 10 Mbit/s	
	bool                    EnableAutoMDIX; //  Crossover  
		
	bool					GenChksums;		// IP checksums to be calculated	
	bool					StripPads;		// remove trailing pad bytes
	
	bool 					RxWordAligment;
	bool 					TxWordAligment;
		
	bool					FullDuplex;		// set full duplex mode
	bool					Negotiate;		// enable auto negotiation	
	bool					Loopback;		// loopback at the PHY
	bool					Cache;			// Buffers may be cached
	bool					FlowControl;	// flow control active
	WORD					IntMask;		// interrupt mask
	BYTE					Mac[6];			// MAC address of the board
	
	INTERRUPT_IVG			EtherIntIVG;	// IVG for the ethernet interrupt
	INTERRUPT_IVG			RXIVG;			// IVG for the RX completion
	INTERRUPT_IVG			TXIVG;			// IVG for the TX completion
	
	t_BUFFER_STATUS			RX_Status;
	t_BUFFER_STATUS			TX_Status;
	
	WORD 					CurTXFrame;
	WORD					CurRXFrame;
	
	WORD 					NumFramesRecv;
	WORD 					NumFramesTran;
	DWORD 					NumBytesRecv;
	DWORD 					NumBytesFreeForTran;
	
    } t_ETHERNET_BF537_CONF;

#pragma pack()
//
//		Wait until the previous MDC/MDIO transaction has completed
//
#define PollMdcDone	{while((*pEMAC_STAADD) & STABUSY);}
#define INVALID_SOCKET	0xFFFFFFFF


WORD RdPHYReg(WORD PHYAddr, WORD RegAddr);
void WrPHYReg(WORD PHYAddr, WORD RegAddr, WORD Data);

void SetMacAddr(BYTE *mac);
void GetMacAddr(BYTE *mac);

void SetPhy(t_ETHERNET_BF537_CONF *dev);
void ProcessNegotiationResult(t_ETHERNET_BF537_CONF *dev);
void WaitLinkUp(t_ETHERNET_BF537_CONF *dev);

#endif /*#ifndef EMAC_H_*/
