#include "HAL.h"

void ConfigureSPI(SPI_TRANSFER_MOD_INIT TransferInit,
				  bool SendLastWord,
				  bool OverwriteDataOnOverflow,
				  bool SlaveDetectEnable,
				  bool EnableMISO,
				  bool WordSize16Bit,
				  bool LSB1, 
				  bool CPHA1,  
				  bool CPOL1,
				  bool Master,
				  bool WriteOpenDrainMaster
				  )
{
    *pSPI_CTL=TransferInit|(SendLastWord<<2)|(OverwriteDataOnOverflow<<3)|
    			(SlaveDetectEnable<<4)|(EnableMISO<<5)|(WordSize16Bit<<8)|
    			(LSB1<<9)|(CPHA1<<10)|(CPOL1<<11)|(Master<<12)|(WriteOpenDrainMaster<<13);
}

void ChangeSPI_TRansferMode(SPI_TRANSFER_MOD_INIT TransferInit)
{
    Wait_SPI_Single_Word_Transfer_Complited;
    
    WORD TempValue = *pSPI_CTL;
    *pSPI_CTL=(TempValue&(~0x3))|TransferInit;
}

void ConfigureSPIBaud(DWORD SpiClkHz)
{    
    float SPI_Baudf=(float)CurSystClkHz/(SpiClkHz*2);    
    WORD SPI_Baud = SPI_Baudf;
    if ((SPI_Baudf-SPI_Baud)>0.5f)SPI_Baud++;  
    SPI_Baud = max(SPI_Baud,2); //    SPI   1/4  
     *pSPI_BAUD=SPI_Baud;     
}

void DumpSPISettings(t_SPI_CONTROL_SET * SPI_CONTROL_SET)
{	
	SPI_CONTROL_SET->SPI_Control=*pSPI_CTL;
	SPI_CONTROL_SET->SPIFlag=*pSPI_FLG;
	SPI_CONTROL_SET->SPI_Baud=*pSPI_BAUD;
	if(*pSPI_CTL&SPE) while (*pSPI_STAT&TXS); // ,    ,      SPI	
}

void UnDumpSPISettings(t_SPI_CONTROL_SET * SPI_CONTROL_SET)
{	
	if(*pSPI_CTL&SPE) while (*pSPI_STAT&TXS); // ,    ,      SPI	
	
	DisableSPI;
	*pSPI_FLG=SPI_CONTROL_SET->SPIFlag;
	*pSPI_BAUD=SPI_CONTROL_SET->SPI_Baud;		
	*pSPI_CTL=SPI_CONTROL_SET->SPI_Control;	
}


WORD WriteSPI(WORD Data)
{
   *pSPI_TDBR=Data;
   Wait_SPI_RegistersReady; // ,        	
   return *pSPI_RDBR;  	
}

WORD ReadSPI(void)
{
   *pSPI_TDBR=0xFF; 
   Wait_SPI_RegistersReady; // ,        	 
   return *pSPI_RDBR;
}

WORD WriteSPIBytes(BYTE * Data, WORD NumData)
{
    WORD TempData;
   	while (NumData--)
    {    	
    	*pSPI_TDBR=*Data++;
    	Wait_SPI_RegistersReady; // ,        	
    	TempData=*pSPI_RDBR;
    }  
    return TempData;
}

void ReadSPIBytes(BYTE * Data, WORD NumData)
{ //  Exchange,    ,      0xFF,
	//   SD    -     
   while(NumData--)
   {      
       *pSPI_TDBR=0xFF; 
       Wait_SPI_RegistersReady; // ,        	
       *Data++=*pSPI_RDBR;
   }
}

WORD ExchangeSPI(WORD Data)
{	//  WriteSPI,       
    *pSPI_TDBR=Data;    
    Wait_SPI_RegistersReady; // ,        	   
	return *pSPI_RDBR; 
}
