#include "HAL.h"
/********************************************************************************/
/***** Set_PLL(CoreCLOCK_multiplier, SCLK_divider)							*****/
/***** Sets core and system clock 											*****/
/********************************************************************************/
DWORD CurCoreClkHz=1;
DWORD CurSystClkHz=1;
DWORD CurPLLClkHz=1;
DWORD CurQuartzClkHz=1;

segment ("L1_code")
void InitializeQuartz(DWORD QuartzFreqHz)
{
    CurQuartzClkHz = QuartzFreqHz;
}

int GetCoreClkDiv(void)
{
    WORD Data =  *pPLL_DIV;
    return (Data>>4)&0x3;
}
/*
   ,   ,        
*/
segment ("L1_code")
WORD Set_PLL(bool Bypass, bool DivF, BYTE MultSEL, DWORD QuartzFreq)
{   
   CurQuartzClkHz=QuartzFreq;
   int InterruptLatch = disable_interrupt;
   if(!(*pEBIU_SDSTAT&SDRS))
   { // SDRAM  
        Put_SDRAM_Into_SelfRefresh();
   } 
   
   WORD previous_PLL= *pPLL_CTL;
   DWORD previous_SIC_IWR = *pSIC_IWR;
   WORD new_PLL= (previous_PLL & 0x00FA) | (MultSEL<<9)|((Bypass&0x1)<<8)|(DivF&0x1);  
   CurPLLClkHz = (MultSEL*CurQuartzClkHz)/(1<<DivF);   
   if (new_PLL != previous_PLL) // skip if multiplier has not changed
   {		      	
		*pPLL_LOCKCNT=0x200; 			  //    	
		*pSIC_IWR = 0x1; 				  // enable PLL Wakeup Interrupt			
		*pPLL_CTL = new_PLL;			
		ssync();  		
	 	idle();							  // put in idle
	 	*pSIC_IWR = previous_SIC_IWR;	  // continue here after idle, restore previous IWR content
		ssync();	
    }   
    
    enable_interrupt(InterruptLatch);
    return new_PLL;
}

segment ("L1_code")
WORD SetSystemAndCoreCLKDiv(BYTE SystemCLK_divider, BYTE CoreCLK_divider)
{      
    int InterruptLatch = disable_interrupt;
    
    if(!(*pEBIU_SDSTAT&SDRS))
    { // SDRAM  
        Put_SDRAM_Into_SelfRefresh();
    }   
    
    //         
    SystemCLK_divider = max(SystemCLK_divider, (1<<CoreCLK_divider));    
    
    CurCoreClkHz = CurPLLClkHz/(1<<CoreCLK_divider);
    DWORD NewSystemClk= CurPLLClkHz/SystemCLK_divider;
    WORD NewPLL_DIV = (CoreCLK_divider<<4)| SystemCLK_divider;
        
    *pPLL_DIV = NewPLL_DIV;
    NewPLL_DIV = *pPLL_DIV;
  	ssync();  	      
    
    if(!(*pEBIU_SDSTAT&SDRS))
    { // SDRAM      
    	if (NewSystemClk!=CurSystClkHz&&CurSystClkHz!=1)
    	{ //    
    	    WORD LastSDRefresh = *pEBIU_SDRRC;
    	    *pEBIU_SDRRC =(int)(LastSDRefresh*((float)NewSystemClk/CurSystClkHz));
    	}
        Get_SDRAM_From_SelfRefresh();
    }    

    CurSystClkHz = NewSystemClk;
  	enable_interrupt(InterruptLatch);
  	
  	return NewPLL_DIV;
}
WORD Set_Pll_PowerManagement(bool PllOff, bool StopClock, bool PowerDown,
							bool InputDelay, bool OutputDelay)
{   
   WORD previous_PLL= *pPLL_CTL;
   DWORD previous_SIC_IWR = *pSIC_IWR;
   WORD new_PLL= (previous_PLL & 0xFF01) | ((PllOff&1)<<1)|
   					((StopClock&1)<<3)|((PowerDown&1)<<5)|
   					((InputDelay&1)<<6)|((OutputDelay&1)<<7);  
   
   if (new_PLL != previous_PLL) // skip if multiplier has not changed
   {			  
		*pPLL_LOCKCNT=0x200; //    	
		*pSIC_IWR = 0x1; // enable PLL Wakeup Interrupt
		*pPLL_CTL = new_PLL;
		new_PLL = *pPLL_CTL;
		ssync();		
  		
	 	idle();							  // put in idle

	 	*pSIC_IWR = previous_SIC_IWR;		  // continue here after idle, restore previous IWR content
		ssync();
		
    }
    
    return new_PLL;
}

void Idle_State(DWORD WakeUpReason, bool UseSleepSDRAM)
{   
    DWORD InterruptLatch = disable_interrupt;   
    if (UseSleepSDRAM&&(!(*pEBIU_SDSTAT&SDRS)))
    {
        Put_SDRAM_Into_SelfRefresh();
    }
 	DWORD previous_SIC_IWR = *pSIC_IWR;
 	*pSIC_IWR = WakeUpReason;
 	ssync();
 	
    idle();
    
    *pSIC_IWR = previous_SIC_IWR;     
    
    if (UseSleepSDRAM&&(!(*pEBIU_SDSTAT&SDRS)))
    {
        Get_SDRAM_From_SelfRefresh();
    }
    enable_interrupt(InterruptLatch);
}

segment ("L1_code")
void Put_SDRAM_Into_SelfRefresh(void)
{
    ssync();
    *pEBIU_SDGCTL|=SRFS; 
    ssync();   
    while (!(*pEBIU_SDSTAT&SDSRA)); //       self refresh
}
segment ("L1_code")
WORD Get_SDRAM_From_SelfRefresh(void)
{
    ssync();
    *pEBIU_SDGCTL&=~SRFS;        
    *pEBIU_SDGCTL|=PSS;
    ssync();    
    return *(WORD *)0x0;
}

void SystemReset(bool NoBoot)
{
	if(NoBoot)*pSYSCR=NOBOOT; //     ,    
	ssync();
	*pSWRST=SYSTEM_RESET;
	ssync();
	*pSWRST=0;
	ssync();
	raise_intr(1);
}


#if (defined (__ADSPBF534__)||defined (__ADSPBF536__)||defined (__ADSPBF537__))
void EnableClkBufOut(void)
{
    WORD vrctl = *pVR_CTL;
	if (!(vrctl&CLKBUFOE)) 
	{
	    int InterruptLatch = disable_interrupt;     	    
		*pVR_CTL = CLKBUFOE|vrctl; //enable CLKBFOE to enable clock for the PHY				
		ssync();
		idle();		
		enable_interrupt(InterruptLatch);
	}
}
#endif



