#include "hardware.h"


/*================================================================================================*/
struct altera_register g_altera_reg[ ALTERA_REGS_NUMBER ];
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void OpenAlteraForBurn
    (
    void
    )
    {    
    /*    */
    DumpSPISettingsStore( SPI_DEVICE_ALTERA_BURN );
    
    DisableSPI;
    /*  CS   */
    Select_CS_Slave( 0 );
    /*     */
    PF_PINS_SET( PF4|PF5|PF6|PF10 );
    ConfigureSPIBaud( AlteraMaxBurnFreq );
    ConfigureSPI( START_TRANS_WITH_WRITE,
				  false,                                /* bool SendLastWord, */
				  true,                                 /* bool OverwriteDataOnOverflow, */
				  false,                                /* bool SlaveDetectEnable, */
				  true,                                 /* bool EnableMISO, */
				  false,                                /* bool WordSize16Bit, */
				  true,                                 /* bool LSB1, */
				  false,                                /* bool CPHA1, */
				  false,                                /* bool CPOL1, */
				  true,                                 /* bool Master, */
				  false                                 /* bool WriteOpenDrainMaster */
				  );
	EnableSPI;	
    
    PG_PINS_CLR( ALTERA_NCONF_G );
	while (PG_PINS_DATA & ALTERA_NSTAT_G)
	    {
	    continue;
	    }    
	Sleep( 10 );
    PG_PINS_SET( ALTERA_NCONF_G );
    while (!(PG_PINS_DATA & ALTERA_NSTAT_G))
        {
        continue;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
DWORD LoadAlteraBroachingFromFlash
    (
    BYTE *Data
    )
    {
    /*    ,     */
    DWORD BroachingSize = 0;
    DWORD CRC = 0;
    DWORD CurrCRC = 0;
    DWORD i = 0;
    
    OpenATFlashInterface();    
    
    
    ContiniousReadFlash( ALTERA_BROACHING_FLASH_START_ADDRESS, (BYTE *)&BroachingSize,
        sizeof(BroachingSize) );
    ContiniousReadFlash( ALTERA_BROACHING_FLASH_START_ADDRESS+4, (BYTE *)&CRC, sizeof(CRC) );
    ContiniousReadFlash( ALTERA_BROACHING_FLASH_START_ADDRESS+8, (BYTE *)Data, BroachingSize );
    CloseATFlashInterface();    
    
    for (i = 0; (i < BroachingSize); i++)
        {
        CurrCRC += Data[ i ];
        }
    
    if (CRC != CurrCRC)
        return 0xFFFFFFFF;
        
    /*      WORD */
    return BroachingSize;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool BurnAltera
    (
    BYTE *Data,
    DWORD NumData
    )
    {
    OpenAlteraForBurn();    
    WriteSPIBytes_asm( Data, NumData );
    CloseAlteraForBurn();
    
    
    OpenAlteraInterface();
    AlteraWriteCommand( 0, 0x01 );
    g_board_version = AlteraReadCommand( 0x07 );
    
    return AlteraTest();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CloseAlteraForBurn
    (
    void
    )
    {    
    WORD i;
        
    while (!(PF_PINS_DATA & ALTERA_CONF_DONE_F))
        {
        WriteSPI( 0 );
        }
    
    for (i = 0; (i < 30); i++)
        {
        /*     */
        WriteSPI_asm( 0 );
        }
    DelayMks( 400 );
    UnDumpSPISettingsStore( SPI_DEVICE_ALTERA_BURN );
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void OpenAlteraInterface(void)
{
    /*      - User Mode */

    DumpSPISettingsStore( SPI_DEVICE_ALTERA_USER );
    
    DisableSPI;
    Select_CS_Slave( ALTERA_REG_CS_F ); 
    /*     */
    PF_PINS_SET( PF4|PF5|PF6|PF10 );
    ConfigureSPIBaud( AlteraMaxWorkFreq );
    ConfigureSPI( START_TRANS_WITH_WRITE,
				  false,                                /* bool SendLastWord, */
				  true,                                 /* bool OverwriteDataOnOverflow, */
				  false,                                /* bool SlaveDetectEnable, */
				  true,                                 /* bool EnableMISO, */
				  true,                                 /* bool WordSize16Bit, */
				  false,                                /* bool LSB1, */
				  false,                                /* bool CPHA1, */
				  true,                                 /* bool CPOL1, */
				  true,                                 /* bool Master, */
				  false                                 /* bool WriteOpenDrainMaster */
				  );
	EnableSPI;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool AlteraTest
    (
    void
    )
    {
    unsigned char d;
        
    AlteraWriteCommand( ALTERA_TEST_REG, 0x55 );
    if ((d = AlteraReadCommand( ALTERA_TEST_REG )) != 0x55)
        return false;     
    
    /*    */
    AlteraWriteCommand( ALTERA_RESET_REG, 0 );
    AlteraWriteCommand( ALTERA_RESET_REG, 1 );
    DelayMks( 400 );
    /*  Ring Mode */
    AlteraWriteCommand( ALTERA_TEST_RING_REG, 0 );
    /*      */
    AlteraWriteCommand( ALTERA_MODULE_SPEED_LO, 0x00 );
    AlteraWriteCommand( ALTERA_MODULE_SPEED_HI, 0x00 );
    /*    ,        */
    SetAlteraLed( LED_Green );
    return true;    
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void SetAlteraLed
    (
    t_AlteraLedType AlteraLedType
    )
    {
	OpenAlteraInterface();   
	AlteraWriteCommand( ALTERA_LED_REG, AlteraLedType );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
BYTE AlteraReadCommand
    (
    WORD Address
    )
    {
   	WriteSPI_asm( Address );
    return ExchangeSPI_asm( 0x8000 ) & 0xFF;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void AlteraWriteCommand
    (
    WORD Address,
    WORD Data
    )
    {
    WriteSPI_asm( Address );
    ExchangeSPI_asm( Data|0xC000 );
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
