#include "hardware.h"


/*================================================================================================*/
#ifdef _EZKIT_
  #define MainMultiplier 16
  #define SystemCLKDiv 3
  #define QuartzClkFreq (25 * 1000000)
#else
  #define MainMultiplier 20
  #define SystemCLKDiv 3
  #define QuartzClkFreq (20*1000000)
#endif

#define CoreCLKDiv 0
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void InitializeRealtimeClock(void)
    {
    /*  
     *        2      ,
     *      SDRAM,    data corrupt
     *  
     */
    *pRTC_ICTL = 0;                         /*    */
    *pRTC_SWCNT = 0;                        /*  =0; */
    *pRTC_PREN = 0;                         /*     ,    */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeClock
    (
    void
    )
    {
    /*  -   32768 
     *   -  590 M
     * core_clock -  ,      PLL - 590 M
     * system_clock - 118 	
     */
    /* bypass   div -    */
    Set_PLL( false, false, MainMultiplier, QuartzClkFreq);
    SetSystemAndCoreCLKDiv( SystemCLKDiv, CoreCLKDiv );
    Set_Pll_PowerManagement( false, false, false, false, false );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeVoltage
    (
    void
    )
    {
    SetNewVoltage( VOL_1_20, VOL_FREQ_1_MHz, VOL_GAIN_20 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
segment ("L1_code")
WORD InitializeSDRAM
    (
    void
    )
    {
    bool SDRAMInittialized = !(*pEBIU_SDSTAT & SDRS); 
    if (SDRAMInittialized)
        {
    	// SDRAM      	
        Put_SDRAM_Into_SelfRefresh();
        }        
    /*  SDRAM
     * RDIV=((117964,8*64ms)/8192)-(6+3)=912 //   -   
     *  -   8192,  4096,    1834
     */
    *pEBIU_SDRRC = ((CurSystClkHz / 1000) * 64) / 8192 - (6 + 3);
    /*
     *   - 32 
     * 9  -   
     */
    if (!SDRAMInittialized)
        {
        *pEBIU_SDBCTL = EBE | EBSZ_32 | EBCAW_9;
        }

    /* CAS latency=3,    2 -     (    !!!!)
     * PASR_ALL -   SDRAM  2.5  -  ,   
     * tRAS(min)=45  (  120  - 6 )
     * tRP(min)=20  (  120  - 3 )
     * tRCD(min)=20  (  120  - 3 )
     * tWR - ,   2
     * POWER startup delay -  
     * PSS - power SDRAM -  
     * SRFS -    SDRAM     0  
     * EBUFE=0 -    SDRAM
     * FBBRW=0 -  ,      ,    -  
     * EMREN=0 -   SDRAM  2.5  -  
     * TCSR=0 -   SDRAM  2.5  -  
     * CDDBG=0 -    signals  	
     */	  

    *pEBIU_SDGCTL = SCTLE | CL_3 | PASR_ALL | TRAS_6 | TRP_3 | TRCD_3 | TWR_2 | PSS;		
    ssync();
    return *(WORD *)0x00;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeASYNCMemory
    (
    void
    )
    {
    /*  ASYNC */
    /*
     * BANK0 -    ISP
     * ARDY - disable
     * ARDY polarity -  
     *    -  - 1 
     *   - 2 
     *   - 2 
     *   -  - 3 

     * BANK1 -    ISP DMA
     * ARDY - disable
     * ARDY polarity -  
     *    -  - 6  -   4  
     *   - 2 
     *   - 3 
     *   -  - 5 ,   4 -  4
     */		

    /*  0  RAT  WAT   */
    *pEBIU_AMBCTL0 = B0TT_1 | B0ST_2 | B0HT_2 | B0RAT_5 | B0WAT_5 | B1TT_1 | B1ST_3 | B1HT_2 |
        B1RAT_5 | B1WAT_5;

    /*
     * BANK2 -  
     * BANK3 -  
     */
    *pEBIU_AMBCTL1 = B2TT_4 | B2ST_2 | B2HT_3 | B2RAT_8 | B2WAT_4 | B3TT_4 | B3ST_3 | B3HT_3 |
        B3RAT_5 | B3WAT_5;

    /*
     * AMCKEN-   		
     * AMBEN_ALL -   	
     * Bit 8 of the EBIU_AMGCTL register is also set to ensure that DMA gets more priority over 
     * the processor while accessing external memory. If this is not done then frequent
     * DMA under and over runs occur when executing instructions from SDRAM	
     */
    *pEBIU_AMGCTL = 0x01F0 | AMCKEN | AMBEN_ALL;
    ssync();	
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializePPI
    (
    void
    )
    {
    /*
     * PPI -  
     *  
     */
    *pPPI_CONTROL = DLEN_8;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeProgrammableFlagPins
    (
    void
    )
    {
    /*    */
    SWITCH_PORT_PERIF = 0;

    /* PORTF   	
     * PF0 - DMAR
     * PF1 - DMAR
     * PF2 - ISP Interrupt
     * PF3 -  bootloader,    
     * PF4 - SSel6
     * PF5 - SSel5
     * PF6 - SSel4
     * PF7 - TMR2
     * PF8 - 
     * PF9 - FS1
     * PF10 - CS Boot Flash, SPISSEL1
     * PF11 - PF12 - PF13 - SPI
     * PF14 - CONF_DONE -  
     * PF15 - PPI_CLK    	
     */        
    SWITCH_PORT_PERIF |= PFDE | PFS6E | PFS5E | PFS4E;
    PF_PINS_PERIF_ENABLE( PF4|PF5|PF6|PF7|PF9|PF11|PF12|PF13|PF15 );
    PF_PINS_INPUT_ENABLE = PF0 | PF1 | PF7;
    SET_PF_PINS_DIRECTION( ATFlash_CS_F|ALTERA_REG_CS_F | PF4);
    PF_PINS_DATA = PF3 | ATFlash_CS_F | ALTERA_REG_CS_F;
    PF_PINS_POLARITY = 0;                   /* ,      */
    PF_PINS_EDGE = 0;
    PF_PINS_INTERRUPT_BOTH = 0;
    PF_PINS_INTERRUPT_A_SET( 0 );

    /* PORTG
     * PG0 - PG7 - PPI Data    	
     * PG8 - NCONF //     
     * PG9 - NSTAT  - 
     * PG10 - ISP_RESET
     * PG11 - 
     * PG12 -  	
     * PG13 -     	 
     * PG14 - 
     * PG15 - 
     */        
    SWITCH_PORT_PERIF |= 0;
    PG_PINS_PERIF_ENABLE( PG0|PG1|PG2|PG3|PG4|PG5|PG6|PG7 );
    PG_PINS_INPUT_ENABLE = PG13;
    SET_PG_PINS_DIRECTION( ALTERA_NCONF_G|ISP1583_RESET_G );
    PG_PINS_DATA = 0;
    PG_PINS_POLARITY = 0;                   /* ,      */
    PG_PINS_EDGE = ISP1583_READY_G;
    PG_PINS_INTERRUPT_BOTH = 0;    
    PG_PINS_INTERRUPT_A_SET( 0 ); 

    /* PORTH   	
     * PH0 - PH15 - Ethernet
     */   
    PH_PINS_PERIF_ENABLE( PH0|PH1|PH2|PH3|PH4|PH5|PH6|PH7|PH8|PH9|PH10|PH11|PH12|PH13|PH14|PH15 );

    /* PORTG
     *  SPORT0 TRANSMIT PRI SEC
     */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeDMA(void)
{
    StopDMA(DMA_DMA5);// anomaly workaround - DMA 5    ,  
    
    ConfigureDMAChannelPerif(DMA_DMA1,DMA_PMAP_MAC_RX);
    ConfigureDMAChannelPerif(DMA_DMA2,DMA_PMAP_MAC_TX);
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeSPORT
    (
    void
    )
    {
    return;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeSPI
    (
    void
    )
    {
    /*
     * FLS1 -  
     * FLS4 - 
     * FLS6 - -  
     *   SPI   -    -  SPI Slave Boot SPI  
     * 
     */
    Select_CS_Slave( FLS2|FLS3|FLS4|FLS5|FLS6|FLS7 );


    ConfigureSPIBaud( CurSystClkHz/10 );    /*   */

    ConfigureSPI( START_TRANS_WITH_WRITE, false, true, true, true, true, false, false, false, true,
        false);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeUART
    (
    void
    )
    {
    DisableUART_Clock;
    ConfigureUART( UART_DATA8, UART_STOP1, UART_PARITY_ODD );
    ConfigureUART_Interrupt( true, false, false );
    SetUARTBaudRate( 9600 );
    EnableUART_Clock;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
segment ("L1_code")
void InitializeHardware
    (
    void
    )
    {
    int InterruptLatch = disable_interrupt;
    sysreg_write( reg_SYSCFG, 0x32 );       /* Initialize System Configuration Register */
    InitializeClock();
    InitializeASYNCMemory(); 
#ifndef _EZKIT_
    InitializeSDRAM();                      /*     Init_Code */
#endif
    InitializeVoltage(); 
    InitializeRealtimeClock();
    InitializePPI();        
    InitializeProgrammableFlagPins();   
    InitializeDMA(); 
    InitializeSPORT(); 
    InitializeSPI();  
    ssync();
    InterruptLatch |= disable_interrupt;
    enable_interrupt( InterruptLatch );
    }
/*------------------------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------------------------*/
void InitializeTimers
    (
    void
    )
    {
    /*    
     *  
     */
    /*     */
    ConfigureTimer( TIMER0, TIMER_PWM_OUT, false, true, true, false, true, false, false );

    /*          Sleep */
    ConfigureTimer( TIMER1, TIMER_PWM_OUT, false, true, false, false, true, false, false );

    /*   Sleep */
    ConfigureTimer( TIMER2, TIMER_PWM_OUT, false, true, false, false, true, false, false );   

    /*      PPI */
    ConfigureTimer( TIMER7, TIMER_PWM_OUT, false, true, false, false, true, false, false );  						

    ConfigureTimer( TIMER3, TIMER_PWM_OUT, false, true, true, false, true, false, false );   			

    ConfiGureTimerPeriodAndWidth( TIMER1, 200, 100 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitializeAndEnableInterrupts
    (
    void
    )
    {
    ResetPerifDefaultIVG();
    /*
     * 7,8,13 -  Ethernet
     * IVG9 -  USB
     * 10,11,12 - altera
     * IVG14 - VDK
     * IVG15 - VDK
     *   ,    IVG        	
     */
    }
/*------------------------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------------------------*/
void InitializeSoftware
    (
    void
    )
    {   
    InitializeTimers();     
    GlobalStop = 0;
    InitializeAndEnableInterrupts();        /*   */
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
