#include "../hardware.h" 


/*================================================================================================*/
static void Init_D14_SFR(void);
static void InitializeEndpoints(void);
/*================================================================================================*/

/*================================================================================================*/
t_USB_DevConfStruct *USBDevConf;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void ACKNOWLEDGE
    (
    WORD ep
    )
    {    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = ep );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.STATUS1 = 1 );
    USBDevConf->ep0.state = EP_IDLE;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CloseISP1583
    (
    void
    )
    {	
	Disable_Interrupt_Perif( PortF_B_Interrupt );
	PF_PINS_INTERRUPT_B_CLR( USBDevConf_PORTF_INT | USBDevConf_PORTF_DREQ );
	//PF_PINS_PERIF_DISABLE( USBDevConf_PORTF_DREQ_DMAR );
	USBDevConf->Opened = false;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ConfigUSB
    (
    BYTE usb_state
    )
    {      
    /* globale interrupt disable */
    /* disable Interrupt */
    DisableISPInterrupt;
    
    USBDevConf->ep2rxEn = 0;

    switch (usb_state)
        {
        case FLAG_USB_POWERED:
            /* switch to full_speed D14_MODE */
            USBDevConf->USB_FLAGS = FLAG_USB_POWERED;
            Init_D14_SFR();
            InitISP1583DMAC();
            /* disable device */
            AccessISPRegWrite( ISP1583.D14_ADDRESS.VALUE = 0 );
            break;
        case FLAG_USB_DEFAULT:
            /* switch to full_speed D14_MODE */
            USBDevConf->USB_FLAGS = FLAG_USB_DEFAULT | FLAG_USB_POWERED;
            Init_D14_SFR();
            /* enable device and reset the device D14_ADDRESS */
            AccessISPRegWrite( ISP1583.D14_ADDRESS.VALUE = 0x80 );
            break;
        case FLAG_USB_ADDRESS:
            USBDevConf->USB_FLAGS = USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED | FLAG_USB_ADDRESS |
                FLAG_USB_DEFAULT | FLAG_USB_POWERED;
            USBDevConf->ep0.state = USBDevConf->ep1.state = USBDevConf->ep2rx.state =
                USBDevConf->ep2tx.state = EP_IDLE;
            /* set the interrupt mask register */
            /* reset, suspend, resume, hs-status, ep0setup, ep0in, ep0out */
            AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE = 0x0D39 );
            break;
        case FLAG_USB_CONFIGURATION1:
            USBDevConf->USB_FLAGS = USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED |
                FLAG_USB_CONFIGURATION1 | FLAG_USB_ADDRESS | FLAG_USB_DEFAULT | FLAG_USB_POWERED;
            /*    */
            /* reset, suspend, resume, hs-status, dma, ep0setup, ep0in, ep0out, ep2in, ep2out */

    		//AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE = 0x0D79 );

    		USBDevConf->ep2rxEn = 1;
    		/*    BulkRx */
    		AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE = 0x4D79 );
	
            InitializeEndpoints();           
        
            InitISP1583DMAC();
        
            break;
        default:
            break;
        }
    /* globale interrupt enable */
    /* enable Interrupt */
    EnableISPInterrupt;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
WORD GetEPBulkDataRecieved
    (
    void
    )
    {
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
   	return (ISP1583.D14_BUFFER_LENGTH >> 1);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void Init_D14_SFR
    (
    void
    )
    {	
    /* enable Global Interrupt enable
     * enable soft connect and other feature
     * enable wakeup on chip select    
     * DMACLK ON            
    */
    AccessISPRegWrite( ISP1583.D14_MODE.VALUE = 0x0281 );
    
    /* set all interrupt source to mode 1 */
    
    
	//AccessISPRegWrite( ISP1583.D14_INT_CONFIG.VALUE = 0xFD );
	//AccessISPRegWrite( ISP1583.D14_INT_CONFIG.VALUE = 0xF5 );
	AccessISPRegWrite( ISP1583.D14_INT_CONFIG.VALUE = 0x55 );
	
    /* set the interrupt mask register */
    AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE = 0x0D39 );

    USBDevConf->MaxBulkSize = FS_BULK_MAX_PACK_SZ;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void InitializeEndpoints
    (
    void
    )
    {
    /*     */
    unsigned short i;
    
    
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 6 );
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 2 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = 0 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 3 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = 0 );
    
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 2 );
    for (i = 4; (i < 32); i++)
        {
        AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = i );
        AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = 0 );
        }
        
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 6 );
        
    /*     */
    /* interrupt-out endpoint (MaxPacketSize=64, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 2 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_MAXPKTSIZE.VALUE = 0x40 );
    /* interrupt-in  endpoint (MaxPacketSize=64, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 3 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_MAXPKTSIZE.VALUE = 0x40 );
    
    /* bulk-out endpoint (MaxPacketSize=512, double buffering, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4);
    AccessISPRegWrite( ISP1583.D14_ENDPT_MAXPKTSIZE.VALUE = USBDevConf->MaxBulkSize );
    /* bulk-in  endpoint (MaxPacketSize=512, double buffering, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_MAXPKTSIZE.VALUE = USBDevConf->MaxBulkSize );

    /*    */
    /* interrupt-out endpoint (MaxPacketSize=64, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 2 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = (0 << 3) | (0 << 2) | (3 << 0) );
    /* interrupt-in  endpoint (MaxPacketSize=64, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 3 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = (0 << 3) | (0 << 2) | (3 << 0) );
    
    /* bulk-out endpoint (MaxPacketSize=512, double buffering, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
    /*  FULL SPEED -     NOEMPTY =0 */
/*	AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE =
	    ((USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED) ? (1 << 4) : (1 << 4)) |
	    (0 << 3) | (1 << 2) | (2 << 0) );*/
	AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = (0 << 4) | (0 << 3) | (1 << 2) | (2 << 0) );
//	AccessISPRegWrite(ISP1583.D14_ENDPT_TYPE.VALUE=((USBDevConf->USB_FLAGS&FLAG_USB_HIGH_SPEED) ? (1<<4) : (0<<4))|(0<<3)|(0<<2)|(2<<0)); //    BulkRx
    /* bulk-in  endpoint (MaxPacketSize=512, double buffering, enable) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    /*   ,           */
    /*AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE =
        ((USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED) ? (1 << 4) : (0 << 4)) |
        (0 << 3) | (1 << 2) | (2 << 0) );*/
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE = (0 << 4) | (0 << 3) | (1 << 2) | (2 << 0) );
        

    /*     */
    for (i = 2; (i < 6); i++)
        {
        AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = i );
        AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.VALUE |= (1 << 3) );
        }
        
    
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    
        
    
    USBDevConf->ep1.state = USBDevConf->ep2rx.state = USBDevConf->ep2tx.state = EP_IDLE;
    USBDevConf->DMAReady = true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool ISP_DMA_Running
    (
    void
    )
    {
	return (!USBDevConf->DMAReady);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void OpenISP1583
    (
    t_USB_DevConfStruct *USBDevConfThis
    )
    {
    int InterruptLatch = disable_interrupt;
	volatile unsigned short ReadValue = 0;
	volatile unsigned short WriteValue = 0;
    
	USBDevConf = USBDevConfThis;
    /* assert isp1583 reset */
    SET_ISP1583_RES();
    do
        {
        /* assert isp1583 reset */
        SET_ISP1583_RES();
        /* reset pulse width >500mks */
        Sleep( 1 );
        /* deassert isp1583 reset */
        CLR_ISP1583_RES();
        Sleep( 1 );
        }
    while (ISP1583.D14_CHIP_ID != 0x00158230);

    /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    /*          */
    /*
    while (1)
        {
		if (WriteValue == 0)
		    WriteValue = 1;
		AccessISPRegWrite( ISP1583.D14_SCRATCH_REGISTER = WriteValue );
		ReadValue = ISP1583.D14_SCRATCH_REGISTER;
		WriteValue = WriteValue << 1;
    }
    */
    /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
         
    Init_D14_SFR();
    InitISP1583DMAC();
    ConfigUSB( FLAG_USB_POWERED );
    
    PG_PINS_EDGE |= USBDevConf_PORTG_READY_WRITE_EDGE;
    
    PF_PINS_EDGE |= USBDevConf_PORTF_DREQ;
    PF_PINS_CLR( USBDevConf_PORTF_INT | USBDevConf_PORTF_DREQ );// | USBDevConf_PORTF_DREQ_DMAR );
        
    
	PF_PINS_INTERRUPT_B_SET( USBDevConf_PORTF_INT );
    PF_PINS_INTERRUPT_B_CLR( USBDevConf_PORTF_DREQ );
    //PF_PINS_PERIF_ENABLE( USBDevConf_PORTF_DREQ_DMAR );
    
    USBDevConf->Opened = true;
   
    /*     DMA */
    Set_Enable_InterruptIVG_Perif( PortF_B_Interrupt, USBDevConfThis->USBIVG,
        NULL/*USB_InterruptHandler*/ );
        
    /*      */
    InterruptLatch |= disable_interrupt;
    enable_interrupt( InterruptLatch );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void SetDefaultUSBParameters
    (
    t_USB_DevConfStruct *USBDevConfThis
    )
    {
	memset( USBDevConfThis, 0, sizeof(t_USB_DevConfStruct) );
	
	USBDevConf->MaxBulkSize = FS_BULK_MAX_PACK_SZ;
	USBDevConfThis->USBIVG = IVG9;

	/*  DMA     
     *  -   -   USB
     */
    USBDevConfThis->ISP_RW.Large.pNext = (void *)&USBDevConfThis->ISP_RW;
	USBDevConfThis->ISP_RW.Large.StartAddress = (void *)pBULK_DATA;
	USBDevConfThis->ISP_RW.Large.Config.VALUE =
	    PrepareDMAConfig( false, DMA_WORD_16, true, DMA_DESCR_0, DMA_FLOW_STOP, true );
	USBDevConfThis->ISP_RW.Large.XCount = 0;
	USBDevConfThis->ISP_RW.Large.XModify = 0;
	
	USBDevConfThis->MEMDSP_RW.Large.pNext = (void *)&USBDevConfThis->MEMDSP_RW;
	USBDevConfThis->MEMDSP_RW.Large.StartAddress = NULL;
	USBDevConfThis->MEMDSP_RW.Large.Config.VALUE =
	    PrepareDMAConfig( false, DMA_WORD_16, true, DMA_DESCR_0, DMA_FLOW_STOP, true );
	USBDevConfThis->MEMDSP_RW.Large.XCount = 0;
	USBDevConfThis->MEMDSP_RW.Large.XModify = 2;

	USBDevConfThis->BufferLengthBulkRx[ 0 ] = 0;
	USBDevConfThis->BufferLengthBulkRx[ 1 ] = 0;
	USBDevConfThis->BufferLengthBulkRx[ 2 ] = 0;
	USBDevConfThis->BufferLengthBulkRx[ 3 ] = 0;
    USBDevConfThis->BufferLengthBulkRxIndex = 0;
    USBDevConfThis->BufferLengthBulkRxIndexProc = 0;
	
    USBDevConfThis->BufferRemaind = 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void STALL
    (
    WORD ep
    )
    {
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = ep );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.STALL = 1 );
    USBDevConf->ep0.state = EP_IDLE;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void start_readDMA_ISP
    (
    WORD *Data,
    DWORD NumData
    )
    {
    unsigned InterruptLatch;
    
	if (!NumData)
	    return;
	/* ,       ISP DMA */
    while (!USBDevConf->DMAReady)
        {
        if (GlobalStop)
            return;
        }
        
    InterruptLatch = cli();
	ssync();        
    
    USBDevConf->epDMA.ptr = Data;
    USBDevConf->epDMA.cntr = NumData;
    
    USBDevConf->ISP_RW.Large.Config.b_WNR = 0;
    USBDevConf->MEMDSP_RW.Large.Config.b_WNR = 1;
    
    /*  -     ISP */
	USBDevConf->WriteDMAISPFlag = 0;
	
	PF_PINS_CLR( USBDevConf_PORTF_DREQ );
	PF_PINS_INTERRUPT_B_SET( USBDevConf_PORTF_DREQ );
	
	USBDevConf->DMAReady = false;
    
	/* start write */
	/*  DSP   ,    */
    AccessISPRegWrite( ISP1583.D14_DMA_TRANSFER_COUNTER = (NumData << 1) );
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 4 );
    AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = GDMA_Write_Command );
    
    //InterruptLatch |= cli();
	//ssync();
	sti( InterruptLatch );
	ssync();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void start_writeDMA_ISP
    (
    WORD *Data,
    DWORD NumData
    )
    {
    /*    ISP  DMA */
	unsigned InterruptLatch;
	
	if (!NumData)
	    return;
	/* ,       ISP DMA */
    while (!USBDevConf->DMAReady)
        {
        if (GlobalStop)
            return;
        }
        
    InterruptLatch = cli();
	ssync();        
   
	USBDevConf->epDMA.ptr = Data;
	USBDevConf->epDMA.cntr = NumData;
    
	USBDevConf->ISP_RW.Large.Config.b_WNR = 1;
	USBDevConf->MEMDSP_RW.Large.Config.b_WNR = 0;
         
	/*  -     ISP */
	USBDevConf->WriteDMAISPFlag = 1;

	/*  PF8 */
	PF_PINS_CLR( USBDevConf_PORTF_DREQ );
	/*   */
	PF_PINS_INTERRUPT_B_SET( USBDevConf_PORTF_DREQ );
	       	  
	USBDevConf->DMAReady = false;
	
	/* start read DMA ISP */
	/*  DSP   ,    */
    AccessISPRegWrite( ISP1583.D14_DMA_TRANSFER_COUNTER = (NumData << 1) );
    /* endpoint=5   Bulk */
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 5 );
    /*  DMA */
    AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = GDMA_Read_Command );
     
    //InterruptLatch |= cli();
	//ssync();
	sti( InterruptLatch );
	ssync();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void start_writeDMA_ISP_DMAR
    (
    WORD *Data,
    DWORD NumData
    )
    {
    /*   */
    /*    ISP  DMA */
    WORD DataToSend;
	unsigned InterruptLatch;
	
	InterruptLatch = cli();
	ssync();

    if (!NumData)
        return;
    /* ,       ISP DMA */
    while (!USBDevConf->DMAReady)
        {
        if (GlobalStop)
            return;
        }
     

    USBDevConf->ISP_RW.Large.Config.b_WNR = 1;
    USBDevConf->MEMDSP_RW.Large.Config.b_WNR = 0;   
      	    
    USBDevConf->MEMDSP_RW.Large.StartAddress = Data;
    USBDevConf->MEMDSP_RW.Large.XCount = NumData;
    USBDevConf->ISP_RW.Large.XCount = NumData;
     
    SetHDMAParam( USBDevConf_HDMA_Channel, 0, 0 );
	SetupHDMA0( USBDevConf_HDMA_Channel, true, false, MDMA_Multy_Transfer );
      		
	ConfigureDMAChannelDesc( USBDevConf_MEMDMA_Source, &USBDevConf->MEMDSP_RW );
	ConfigureDMAChannelDesc( USBDevConf_MEMDMA_Dest, &USBDevConf->ISP_RW );
    /*        destination */
    PrepareDescrDMAAndStart( USBDevConf_MEMDMA_Source, DMA_DESCR_7, DMA_FLOW_LARGE );
	PrepareDescrDMAAndStart( USBDevConf_MEMDMA_Dest, DMA_DESCR_7, DMA_FLOW_LARGE );
	 
	DataToSend = NumData >> 6;
	USBDevConf->epDMA.cntr = NumData & FS_BULK_MAX_PACK_SZ_MASK;
	if (USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED)
	    {
		DataToSend = NumData >> 8;
		USBDevConf->epDMA.cntr = NumData & HS_BULK_MAX_PACK_SZ_MASK;
	    }
	USBDevConf->epDMA.ptr = &Data[ NumData-USBDevConf->epDMA.cntr ];
	 
	SetHDMAParam( USBDevConf_HDMA_Channel, DataToSend, USBDevConf->MaxBulkSize >> 1 );
	 
	USBDevConf->DMAReady = false;
	 
	/* start read DMA ISP */
	/*  ISP   ,    */
	AccessISPRegWrite( ISP1583.D14_DMA_TRANSFER_COUNTER = (NumData << 1) );
	AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 5 );
	AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = GDMA_Read_Command );
     
    InterruptLatch |= cli();
	ssync();
	sti( InterruptLatch );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void UNSTALL
    (
    WORD ep
    )
    {
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = ep );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.STALL = 0 );
    DelayNs( 250 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.BITS.ENABLE = 0 );
    DelayNs( 250 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_TYPE.BITS.ENABLE = 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
//EX_INTERRUPT_HANDLER( USB_InterruptHandler )
void vIVG9_handle(void)
    {
    t_uint16 DMA_IntStatus;
	WORD PFStatus = PF_PINS_DATA;
	PFStatus &= PF_PINS_INTERRUPT_B_DATA;
	
	if (PFStatus & USBDevConf_PORTF_DREQ)
	    {
		PF_PINS_CLR( USBDevConf_PORTF_DREQ );
		
		if (USBDevConf->epDMA.cntr > 0)
		    {
		    WORD size;
		    /*    ,   DMA */
    		/*  16   */
    		
          	size = min( USBDevConf->MaxBulkSize >> 1, USBDevConf->epDMA.cntr );
      	
			*pMDMA_S0_START_ADDR = USBDevConf->epDMA.ptr;
			*pMDMA_S0_X_COUNT = size;
			*pMDMA_S0_X_MODIFY = 2;
			*pMDMA_D0_START_ADDR = (void *)pBULK_DATA;
			*pMDMA_D0_X_COUNT = size;
			*pMDMA_D0_X_MODIFY = 0;

          	USBDevConf->epDMA.ptr += size;
          	USBDevConf->epDMA.cntr -= size;
      	
          	if (USBDevConf->epDMA.cntr == 0)
    		    {
    		    /*    */
    			PF_PINS_INTERRUPT_B_CLR( USBDevConf_PORTF_DREQ );
    		    }
      	
    		/*  DMA  */
    		*pMDMA_S0_CONFIG = (0 << 12) |       /*   */
    						   (0 << 8) |		 /*   */
    						   (0 << 7) |		 /*   */
                               (0 << 6) |        /*      */
                               (1 << 5) |        /*  FIFO DMA    */
                               (0 << 4) |        /*  DMA -  */
                               (1 << 2) |        /* 16   */
                               (0 << 1) |        /*  -    */
                               (1 << 0);         /*    */
		    ssync();

    		/*  DMA  */
    		*pMDMA_D0_CONFIG = (0 << 12) |       /*   */
                               (0 << 8) |        /*   */
                               (0 << 7) |        /*   */
                               (0 << 6) |        /*      */
                               (1 << 5) |        /*  FIFO DMA    */
                               (0 << 4) |        /*  DMA -  */
                               (1 << 2) |        /* 16   */
                               (1 << 1) |        /*  -    */
                               (1 << 0);         /*    */
    		ssync();
		    }
		else
		    {
		    /*   */
			PF_PINS_INTERRUPT_B_CLR( USBDevConf_PORTF_DREQ );
		    }
	    } /*if (PFStatus & USBDevConf_PORTF_DREQ)*/
	

	if (PFStatus & USBDevConf_PORTF_INT)
	    {
	    DWORD Status;
		
		DMA_IntStatus = ISP1583.D14_DMA_INT.VALUE;
        DMA_IntStatus &= ISP1583.D14_DMA_INT_ENABLE.VALUE;

		/*   ISP */
		Status = ISP1583.D14_INT.VALUE;
		Status &= ISP1583.D14_INT_ENABLE.VALUE;
		
		if ((Status & (1 << 6)) != 0)
		    {
		    /*    DMA (  ).
		     *      DMA     .
		     */
		    DMA_IntStatus = ISP1583.D14_DMA_INT.VALUE;
            //DMA_IntStatus &= ISP1583.D14_DMA_INT_ENABLE.VALUE;
            AccessISPRegWrite( ISP1583.D14_DMA_INT.VALUE = DMA_IntStatus );            		        
		    }

		AccessISPRegWrite( ISP1583.D14_INT.VALUE = Status );
		
        /*       */
		if (Status)
            {	       	
        	WORD Counter;
        	for (Counter = 0; (Counter < 16); Counter++)
        	    {
        		if (Status & (1 << Counter))
        		    {
        		    /* check usb activity */
        			USBEventHandler[ Counter ]();	        		        					        			        			
        		    }
        	    }
            }
	    }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
