#ifndef USB_H_
#define USB_H_



typedef void (*USB_request_Handler)(void);

#include "descriptor.h" // ISP1583 reg
#include "d14.h" // ISP1583 reg
#include "ISP.h"
#include "std_requests.h"
#include "ven_requests.h"


/*================================================================================================*/
/* signals for host */
#define SIGNAL_BUFFER_ERROR (1 << 0)
#define SIGNAL_MODULE_CONFIG (1 << 1)
/* usb state flags */
#define FLAG_USB_POWERED (1 << 0)
#define FLAG_USB_DEFAULT (1 << 1)
#define FLAG_USB_ADDRESS (1 << 2)
#define FLAG_USB_CONFIGURATION1 (1 << 3)
#define FLAG_USB_SUSPEND (1 << 6)
#define FLAG_USB_HIGH_SPEED (1 << 7)

/* request type & mask
 * request direction
 */
#define REQUEST_ANSWER_DIRECTION (0x80)
#define REQUEST_ANSWER_DIRECTION_HOST_TO_DEVICE (0x0)
#define REQUEST_ANSWER_DIRECTION_DEVICE_TO_HOST (0x80)

#define REQUEST_TYPE_MASK (0x60)
#define STANDARD_REQUEST (0x00)
#define CLASS_REQUEST (0x20)
#define VENDOR_REQUEST (0x40)
/* request mask */
#define REQUEST_MASK (0x0F)

/* standard request codes */
#define STD_GET (0x00)
#define STD_CLEAR_FEATURE (0x01)
#define STD_SET_FEATURE (0x03)
#define STD_SET_ADDRESS (0x05)
#define STD_GET_DESCRIPTOR (0x06)
#define STD_SET_DESCRIPTOR (0x07)
#define STD_GET_CONFIGURATION (0x08)
#define STD_SET_CONFIGURATION (0x09)
#define STD_GET_INTERFACE (0x0A)
#define STD_SET_INTERFACE (0x0B)
#define STD_SYNC_FRAME (0x0C)
/* device address mask */
#define ADDRESS_MASK (0x7F)

#define MAXIMUM_USB_STRING_LENGTH (255)
/* values for the bits returned by the USB GET_STATUS command */
#define USB_GETSTATUS_SELF_POWERED (0x01)
#define USB_GETSTATUS_REMOTE_WAKEUP_ENABLED (0x02)

#define STR_INDEX_LANGUAGE (0x00)
#define STR_INDEX_MANUFACTURER (0x01)
#define STR_INDEX_PRODUCT (0x02)
#define STR_INDEX_SERIALNUMBER (0x03)
#define STR_INDEX_CONFIGURATION (0x04)
#define STR_INDEX_INTERFACE (0x05)

/* Values for bmAttributes field of an endpoint descriptor */
#define USB_ENDPOINT_TYPE_MASK (0x03)

#define USB_ENDPOINT_TYPE_CONTROL (0x00)
#define USB_ENDPOINT_TYPE_ISOCHRONOUS (0x01)
#define USB_ENDPOINT_TYPE_BULK (0x02)
#define USB_ENDPOINT_TYPE_INTERRUPT (0x03)
/* definitions for bits in the bmAttributes field of a configuration descriptor. */
#define USB_CONFIG_POWERED_MASK (0xC0)

#define USB_CONFIG_BUS_POWERED (0x80)
#define USB_CONFIG_SELF_POWERED (0x40)
#define USB_CONFIG_REMOTE_WAKEUP (0x20)
/* Endpoint direction bit, stored in address */
#define USB_ENDPOINT_DIRECTION_MASK (0x80)
/* usb device classes */
#define USB_DEVICE_CLASS_RESERVED (0x00)
#define USB_DEVICE_CLASS_AUDIO (0x01)
#define USB_DEVICE_CLASS_COMMUNICATIONS (0x02)
#define USB_DEVICE_CLASS_HUMAN_INTERFACE (0x03)
#define USB_DEVICE_CLASS_MONITOR (0x04)
#define USB_DEVICE_CLASS_PHYSICAL_INTERFACE (0x05)
#define USB_DEVICE_CLASS_POWER (0x06)
#define USB_DEVICE_CLASS_PRINTER (0x07)
#define USB_DEVICE_CLASS_STORAGE (0x08)
#define USB_DEVICE_CLASS_HUB (0x09)
#define USB_DEVICE_CLASS_VENDOR_SPECIFIC (0xFF)
/* USB defined Feature selectors */
#define FEATURE_ENDPOINT_HALT (0x0000)
#define FEATURE_REMOTE_WAKEUP (0x0001)
#define FEATURE_TEST_MODE (0x0002)

#define FEATURE_TEST_J (0x01)
#define FEATURE_TEST_K (0x02)
#define FEATURE_TEST_SE0_NAK (0x03)
#define FEATURE_TEST_Packet (0x04)
#define FEATURE_TEST_Force_Enable (0x05)    /* for USB2.0 HUB downstream port only */


/* event handler referance-table */
#define BOOT_EVENT_HANDLER ((void(*)(void))BOOT_EVENT_HANDLER_BASE)
/* standard request referance-table */
#define BOOT_STD_REQUEST ((void(*)(void))BOOT_STD_REQUEST_BASE)
/* vendor request referance-table */
#define BOOT_VEN_REQUEST ((void(*)(void))BOOT_VEN_REQUEST_BASE)

/*    ISP     */
#define AccessISPRegWrite(x) PG_PINS_CLR(USBDevConf_PORTG_READY_WRITE_EDGE);            \
                             x;                                                         \
                             while(!(PG_PINS_DATA&USBDevConf_PORTG_READY_WRITE_EDGE));  \
                             ssync();                                                   \
                             while(!(PG_PINS_DATA&USBDevConf_PORTG_READY_WRITE_LEV));

#define DisableISPInterrupt AccessISPRegWrite(ISP1583.D14_MODE.VALUE&=~(1<<3))
#define EnableISPInterrupt AccessISPRegWrite(ISP1583.D14_MODE.VALUE|=(1<<3))

#define DisableISPDMAClk AccessISPRegWrite(ISP1583.D14_MODE.VALUE&=~(1<<9))
#define EnableISPDMAClk AccessISPRegWrite(ISP1583.D14_MODE.VALUE|=(1<<9))

#define USB_DESCRIPTOR_MAKE_TYPE_AND_INDEX(d, i) ((USHORT)((USHORT)d<<8 | i))
/*================================================================================================*/

/*================================================================================================*/
typedef enum
    {
	SEL_AVR_DM = 0x82,
	SEL_AVR_PM = 0x83,
	SEL_DMA_TEST_FLAG = 0x84,
	SEL_FLASH_BUFFER = 0x85,
	SEL_FPGA_DATA = 0x86,
	SEL_ALTERA_FLAGS = 0x87,
	SEL_FLASH_DSP = 0x88,
	SEL_FLASH_STATUS = 0x89,
	SEL_DSP = 0x90,
	SEL_DSP_SPI = 0x91,
	SEL_DSP_SPI_BOOT = 0x92,
	SEL_DSP_MEM = 0x93
    } t_LTR030_Get_Put_Array_Destination;
/*================================================================================================*/

/*================================================================================================*/
extern USB_request_Handler USBEventHandler[];


extern int g_need_reset_USB;
/*================================================================================================*/

/*================================================================================================*/
void ep0rx_data(BYTE sel, DWORD ptr, WORD length);
void ep0tx_data(BYTE sel, DWORD ptr, WORD length);
void ep2tx_data(BYTE sel, DWORD ptr, DWORD length);
void InitDMA(void);
void InitISP1583DMAC(void);
void put_ep2data(t_uint16 data[], unsigned long data_len);
void ReadEP0(BYTE * data, WORD DataLength);
void WriteEP0(BYTE * data, WORD DataLength);


void get_ep2rx(t_uint16 data_len);
/*================================================================================================*/

#endif /*#ifndef USB_H_*/
