#include "../hardware.h" 
#include "GlobDefs.h"

/*================================================================================================*/
#define SN_LENGTH 16
/* Definitions for bits in the bmAttributes field of  */
/*   (Table 9-10) Standard Configuration Descriptor   */
/* Reserved (reset to zero) */                    /* D4...D0 */
#define USB_CONFIG_BUS_NOWAKEUP           0x80 /* D5=0 + D6=0 + D7=1 */
#define USB_CONFIG_SELF_NOWAKEUP          0xC0 /* D5=0 + D6=1 + D7=1 */
#define USB_CONFIG_BUS_WAKEUP             0xA0 /* D5=1 + D6=0 + D7=1 */
#define USB_CONFIG_SELF_WAKEUP            0xE0 /* D5=1 + D6=1 + D7=1 */
/* Reserved (set to one) */                       /*   D7    */

#define CURRENT_CONSUMPTION_MA  400

#pragma pack(1)

#define SN_LENGTH  16

/*================================================================================================*/

segment ("sdram0_bank1") BYTE FS_DESCRIPTOR[] =
    {
    /* -> FS_DESCRIPTOR_DEVICE */
    DESCRIPTOR_DEVICE_SIZE,                 /* length */
    DESCRIPTOR_DEVICE,                      /* bDescriptorType */
    W2B( 0x0110 ),                          /* Version 1.1 USB spec Word */
    0xFF,                                   /* Class */
    0x00,                                   /* bDeviceSubClass */
    0x00,                                   /* bDeviceProtocol */
    0x40,                                   /* bMaxPacketSize0 */
    W2B( 0x0471 ),                          /* idVendor */
#if defined(LTR030_ENDPOINT)
    W2B( 0x3030 ),                          /* idProduct for Filter Driver */
#elif defined(LTR031_ENDPOINT)
    W2B( 0x3131 ),                          /* idProduct for Filter Driver */
#else
  #error Compile endpoint not specified
#endif
    W2B( 0x0010 ),                          /* bcdDevice */
    0x01,                                   /* iManufacturer */
    0x02,                                   /* iProduct */
    0x00,                                   /* iSerialNumber */
    0x01,                                   /* bNumConfigurations */
    /* -> FS_DESCRIPTOR_CONFIGURATION */
    DESCRIPTOR_CONFIGURATION_SIZE,          /* Length of the configuration Descriptor */
    DESCRIPTOR_CONFIGURATION,               /* Configuration Descriptor */
    W2B( CONFIG_LENGTH ),                   /* Total length of the configurations,interface and
                                             * class/endpoint
                                             */
    0x01,                                   /* No. of interface supported */
    0x01,                                   /* Configuration Value */
    0x03,                                   /* Index of string descriptor */
    0xC0,                                   /* Configurations Characteristic */
    0xC8,                                   /* Maximun Power */
    /* -> FS_DESCRIPTOR_INTERFACE */
    DESCRIPTOR_INTERFACE_SIZE,              /* Length of standard interface descriptor */
    DESCRIPTOR_INTERFACE,                   /* Standard Interface Type */
    0x00,                                   /* No of Interface */
    0x00,                                   /* Alternate Setting Selector */
    0x04,                                   /* No of Endpoints */
    0x00,                                   /* Interface Class */
    0x00,                                   /* ATAPI Interface Sub Class */
    0x00,                                   /* Bulk Only Interface Protocol */
    0x04,                                   /* Index of interface string desriptor */
    /* -> FS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x01,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> FS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x81,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> FS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x02,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( FS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> FS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x82,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( FS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00                                    /* Interval for Polling Endpoint for data transfer */
    };

segment ("sdram0_bank1") BYTE HS_DESCRIPTOR[] =
    {
    /* -> HS_DESCRIPTOR_DEVICE */
    DESCRIPTOR_DEVICE_SIZE,                 /* length */
    DESCRIPTOR_DEVICE,                      /* bDescriptorType */
    W2B( 0x0200 ),                          /* Version 2.0 USB spec Word */
    0xFF,                                   /* Class */
    0x00,                                   /* bDeviceSubClass */
    0x00,                                   /* bDeviceProtocol */
    0x40,                                   /* bMaxPacketSize0 */
    W2B( 0x0471 ),                          /* idVendor */
#if defined(LTR030_ENDPOINT)
    W2B( 0x3030 ),                          /* idProduct for Filter Driver */
#elif defined(LTR031_ENDPOINT)
    W2B( 0x3131 ),                          /* idProduct for Filter Driver */
#else
  #error Compile endpoint not specified
#endif
    W2B( 0x0010 ),                          /* bcdDevice */
    0x01,                                   /* iManufacturer */
    0x02,                                   /* iProduct */
    0x00,                                   /* iSerialNumber */
    0x01,                                   /* bNumConfigurations */
    /* -> HS_DESCRIPTOR_CONFIGURATION */
    DESCRIPTOR_CONFIGURATION_SIZE,          /* Length of the configuration Descriptor */
    DESCRIPTOR_CONFIGURATION,               /* Configuration Descriptor */
    W2B( CONFIG_LENGTH ),                   /* Total length of the configurations,interface and
                                             * class/endpoint
                                             */
    0x01,                                   /* No. of interface supported */
    0x01,                                   /* Configuration Value */
    0x03,                                   /* Index of string descriptor */
    0xC0,                                   /* Configurations Characteristic */
    0xC8,                                   /* Maximum Power */
    /* -> HS_DESCRIPTOR_INTERFACE */
    DESCRIPTOR_INTERFACE_SIZE,              /* Length of standard interface descriptor */
    DESCRIPTOR_INTERFACE,                   /* Standard Interface Type */
    0x00,                                   /* No of Interface */
    0x00,                                   /* Alternate Setting Selector */
    0x04,                                   /* No of Endpoints */
    0x00,                                   /* Interface Class */
    0x00,                                   /* ATAPI Interface Sub Class */
    0x00,                                   /* Bulk Only Interface Protocol */
    0x04,                                   /* Index of interface string desriptor */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x01,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x81,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x02,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( HS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x82,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( HS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00                                    /* Interval for Polling Endpoint for data transfer */
    };

segment ("sdram0_bank1") BYTE OTHER_DESCRIPTOR[] =
    {
    /* -> DEVICE_QUALIFIER */
    DESCRIPTOR_DEVICE_QUALIFIER_SIZE,       /* length of HS Device Descriptor */
    DESCRIPTOR_DEVICE_QUALIFIER,            /* HS Device Qualifier Type */
    W2B( 0x0200 ),                          /* USB 2.0 version */
    0xFF,                                   /* Device class */
    0x00,                                   /* Device SubClass */
    0x00,                                   /* Device Protocol Code */
    0x40,                                   /* Maximum Packet SIze for other speed */
    0x01,                                   /* Number of Other speed configurations */
    0x00,                                   /* Reserved */
    /* -> DESCRIPTOR_OTHER_SPEED_CONFIG */
    /* length of other speed configuration descriptor */
    DESCRIPTOR_OTHER_SPEED_CONFIGURATION_SIZE,
    DESCRIPTOR_OTHER_SPEED_CONFIGURATION,   /* Other speed configuration Type */
    W2B( CONFIG_LENGTH ),                   /* Total length */
    0x01,                                   /* No. of interface supported */
    0x01,                                   /* Configuration Value */
    0x03,                                   /* Index of string descriptor */
    0xC0,                                   /* Configurations Characteristic */
    0xC8,                                   /* Maximun Power */
    /* -> HS_DESCRIPTOR_INTERFACE */
    DESCRIPTOR_INTERFACE_SIZE,              /* Length of standard interface descriptor */
    DESCRIPTOR_INTERFACE,                   /* Standard Interface Type */
    0x00,                                   /* No of Interface */
    0x00,                                   /* Alternate Setting Selector */
    0x04,                                   /* No of Endpoints */
    0x00,                                   /* Interface Class */
    0x00,                                   /* ATAPI Interface Sub Class */
    0x00,                                   /* Bulk Only Interface Protocol */
    0x04,                                   /* Index of interface string desriptor */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x01,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x81,                                   /* Set Int endpoint to 2 for ATAPI mode */
    0x03,                                   /* Endpoint Characteristic */
    W2B( 0x0040 ),                          /* Endpoint Max Packet Size */
    0x01,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x02,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( FS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00,                                   /* Interval for Polling Endpoint for data transfer */
    /* -> HS_DESCRIPTOR_ENDPOINT */
    DESCRIPTOR_ENDPOINT_SIZE,               /* Length of Standard Endpoint Descriptor */
    DESCRIPTOR_ENDPOINT,                    /* Standard Endpoint Type */
    0x82,                                   /* Set Bulk endpoint to 1 for ATAPI mode */
    0x02,                                   /* Endpoint Characteristic */
    W2B( FS_BULK_MAX_PACK_SZ ),             /* Endpoint Max Packet Size */
    0x00                                    /* Interval for Polling Endpoint for data transfer */
    };

segment ("sdram0_bank1") BYTE MFG_STR_LAT[] =
    {
    22,
    DESCRIPTOR_STRING,
    UNI_LAT( 'L' ), UNI_LAT( '-' ), UNI_LAT( 'C' ), UNI_LAT( 'A' ), UNI_LAT( 'R' ), UNI_LAT( 'D' ),
    UNI_LAT( ' ' ),
    UNI_LAT( 'L' ),
    UNI_LAT( 'T' ),
    UNI_LAT( 'D' )
    };
segment ("sdram0_bank1") BYTE MFG_STR_CYR[] =
    {
    22,
    DESCRIPTOR_STRING,
    UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_LAT( ' ' ),
    UNI_CYR( '' ), UNI_LAT( '-' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' )
    };

segment ("sdram0_bank1") BYTE FS_PID_STR_LAT[] =
    {
    38,
    DESCRIPTOR_STRING,
    UNI_LAT( 'C' ), UNI_LAT( 'r' ), UNI_LAT( 'a' ), UNI_LAT( 't' ), UNI_LAT( 'e' ), UNI_LAT( ' ' ),
    UNI_LAT( 'L' ), UNI_LAT( 'T' ), UNI_LAT( 'R' ),
    UNI_LAT( '(' ),
    UNI_LAT( 'F' ), UNI_LAT( 'S' ), UNI_LAT( ' ' ),
    UNI_LAT( 'M' ), UNI_LAT( 'o' ), UNI_LAT( 'd' ), UNI_LAT( 'e' ),
    UNI_LAT( ')' )
    };
segment ("sdram0_bank1") BYTE HS_PID_STR_LAT[] =
    {
   38,
   DESCRIPTOR_STRING,
   UNI_LAT( 'C' ), UNI_LAT( 'r' ), UNI_LAT( 'a' ), UNI_LAT( 't' ), UNI_LAT( 'e' ), UNI_LAT( ' ' ),
   UNI_LAT( 'L' ), UNI_LAT( 'T' ), UNI_LAT( 'R' ),
   UNI_LAT( '(' ),
   UNI_LAT( 'H' ), UNI_LAT( 'S' ), UNI_LAT( ' ' ),
   UNI_LAT( 'M' ), UNI_LAT( 'o' ), UNI_LAT( 'd' ), UNI_LAT( 'e' ),
   UNI_LAT( ')' )
    };
segment ("sdram0_bank1") BYTE FS_PID_STR_CYR[] =
    {
    40,
    DESCRIPTOR_STRING,
    UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_LAT( ' ' ),
    UNI_LAT( 'L' ), UNI_LAT( 'T' ), UNI_LAT( 'R' ),
    UNI_LAT( '(' ),
    UNI_LAT( 'F' ), UNI_LAT( 'S' ), UNI_LAT( ' ' ),
    UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ),
    UNI_LAT( ')' )
    };
segment ("sdram0_bank1") BYTE HS_PID_STR_CYR[] =
    {
    40,
    DESCRIPTOR_STRING,
    UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ),
    UNI_LAT( ' ' ),
    UNI_LAT( 'L' ), UNI_LAT('T'), UNI_LAT('R'),
    UNI_LAT( '(' ),
    UNI_LAT( 'H' ), UNI_LAT( 'S' ),
    UNI_LAT( ' ' ),
    UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ), UNI_CYR( '' ),
    UNI_LAT( ')' )
    };

segment ("sdram0_bank1") BYTE CFG_STR_LAT[] =
    {
    46,
    DESCRIPTOR_STRING,
    UNI_LAT( 'S' ),
    UNI_LAT( 't' ),
    UNI_LAT( 'a' ),
    UNI_LAT( 'n' ),
    UNI_LAT( 'd' ),
    UNI_LAT( 'a' ),
    UNI_LAT( 'r' ),
    UNI_LAT( 'd' ),
    UNI_LAT( ' ' ),
    UNI_LAT( 'C' ),
    UNI_LAT( 'o' ),
    UNI_LAT( 'n' ),
    UNI_LAT( 'f' ),
    UNI_LAT( 'i' ),
    UNI_LAT( 'g' ),
    UNI_LAT( 'u' ),
    UNI_LAT( 'r' ),
    UNI_LAT( 'a' ),
    UNI_LAT( 't' ),
    UNI_LAT( 'i' ),
    UNI_LAT( 'o' ),
    UNI_LAT( 'n' )
    };
segment ("sdram0_bank1") BYTE CFG_STR_CYR[] =
    {
    50,
    DESCRIPTOR_STRING,
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_LAT( ' ' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' )
    };

segment ("sdram0_bank1") BYTE INT_STR_LAT[] =
    {
    54,
    DESCRIPTOR_STRING,
    UNI_LAT( 'C' ),
    UNI_LAT( 'r' ),
    UNI_LAT( 'a' ),
    UNI_LAT( 't' ),
    UNI_LAT( 'e' ),
    UNI_LAT( ' ' ),
    UNI_LAT( 'c' ),
    UNI_LAT( 'o' ),
    UNI_LAT( 'n' ),
    UNI_LAT( 't' ),
    UNI_LAT( 'r' ),
    UNI_LAT( 'o' ),
    UNI_LAT( 'l' ),
    UNI_LAT( 'l' ),
    UNI_LAT( 'e' ),
    UNI_LAT( 'r' ),
    UNI_LAT( ' ' ),
    UNI_LAT( 'i' ),
    UNI_LAT( 'n' ),
    UNI_LAT( 't' ),
    UNI_LAT( 'e' ),
    UNI_LAT( 'r' ),
    UNI_LAT( 'f' ),
    UNI_LAT( 'a' ),
    UNI_LAT( 'c' ),
    UNI_LAT( 'e' )
    };
segment ("sdram0_bank1") BYTE INT_STR_CYR[] =
    {
    56,
    DESCRIPTOR_STRING,
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_LAT( ' ' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_LAT( '-' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' ),
    UNI_CYR( '' )
    };

segment ("sdram0_bank1") BYTE STRING_ID[] =
    {
    0x06,
    DESCRIPTOR_STRING,
    0x09, 0x04,                             /* LAT */
    0x19, 0x04                              /* CYR */
    };                                              //
segment ("sdram0_bank1") BYTE SERIAL_NUMBER_UNI[] =
    {
    14,
    DESCRIPTOR_STRING,
    UNI_LAT( '0' ), UNI_LAT( '0' ), UNI_LAT( '0' ), UNI_LAT( '1' ),UNI_LAT( 'A' ),UNI_LAT( 'B' )
    };


segment ("sdram0_bank1") BYTE *STRINGS[ 4 ][ 6 ] =
    {
    {STRING_ID, MFG_STR_LAT, FS_PID_STR_LAT, CFG_STR_LAT, INT_STR_LAT, SERIAL_NUMBER_UNI},
    {STRING_ID, MFG_STR_CYR, FS_PID_STR_CYR, CFG_STR_CYR, INT_STR_CYR, SERIAL_NUMBER_UNI},
    {STRING_ID, MFG_STR_LAT, HS_PID_STR_LAT, CFG_STR_LAT, INT_STR_LAT, SERIAL_NUMBER_UNI},
    {STRING_ID, MFG_STR_CYR, HS_PID_STR_CYR, CFG_STR_CYR, INT_STR_CYR, SERIAL_NUMBER_UNI}
    };
/*================================================================================================*/

#pragma pack()
