#include "..\hardware.h" 


/*================================================================================================*/
/* standard request-handler referance-table */
USB_request_Handler StandardDeviceRequest[] =
    {
    std_get_status,                              /* 0 */
    std_clear_feature,                           /* 1 */
    std_undetected,                              /* 2 */
    std_set_feature,                             /* 3 */

    std_undetected,                              /* 4 */
    std_set_address,                             /* 5 */
    std_get_descriptor,                          /* 6 */
    std_set_descriptor,                          /* 7 */

    std_get_configuration,                       /* 8 */
    std_set_configuration,                       /* 9 */
    std_get_interface,                           /* 10 */
    std_set_interface,                           /* 11 */

    std_undetected,                              /* 12 */
    std_undetected,                              /* 13 */
    std_undetected,                              /* 14 */
    std_undetected                               /* 15 */
    };
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void std_clear_feature
    (
    void
    )
    {
	/* standard request -> clear feature (DIOC_RESET_PIPEx) */
	
    switch (USBDevConf->dreq.bmRequestType & RECIPIENT_MASK)
        {
        case RECIPIENT_ENDPOINT:
            if (USBDevConf->dreq.wValue != FEATURE_ENDPOINT_HALT)
                {
                STALL( 1 );
                }
            else
                {
                switch ((BYTE)USBDevConf->dreq.wIndex)
                    {
                    case 0x00:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_ADDRESS))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x01:
                    case 0x81:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep1.state = EP_IDLE;
                            UNSTALL( (1 << 1) + 0 );
                            UNSTALL( (1 << 1) + 1 );
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x02:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep2rx.state = EP_IDLE;
                            // InitISP1583DMAC();                    
                            UNSTALL( (2 << 1) + 0 );
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x82:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep2tx.state = EP_IDLE;
                            // InitISP1583DMAC();
                            UNSTALL( (2 << 1) + 1 );
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    default:
                        STALL( 1 );
                    }
                }
            break;
        default:
            STALL( 1 );
        } /*switch (USBDevConf->dreq.bmRequestType & RECIPIENT_MASK)*/
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_get_configuration
    (
    void
    )
    {
	/* standard request -> get configuration */
    if (!(USBDevConf->USB_FLAGS & FLAG_USB_ADDRESS))
        {
        STALL(1);
        }
    else
        {
        *(BYTE *)&USBDevConf->dreq = (USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1) ? 1 : 0;
        ep0tx_data( SEL_DSP_MEM, (DWORD)&USBDevConf->dreq, 1 );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_get_descriptor
    (
    void
    )
    {
	/* standard request -> get descriptor */
    BYTE dIndex = LSB( USBDevConf->dreq.wValue );
	BYTE dType = MSB( USBDevConf->dreq.wValue );
    DWORD ptr = (DWORD)((USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED) ?
        HS_DESCRIPTOR : FS_DESCRIPTOR);
    
    switch (dType)
        {
        case DESCRIPTOR_DEVICE:
            ep0tx_data( SEL_DSP_MEM, ptr + 0, DESCRIPTOR_DEVICE_SIZE );
            break;
        case DESCRIPTOR_CONFIGURATION:
            ep0tx_data( SEL_DSP_MEM, ptr + DESCRIPTOR_DEVICE_SIZE, CONFIG_LENGTH );
            break;
        case DESCRIPTOR_STRING:
            ptr = (DWORD)STRINGS[ ((USBDevConf->USB_FLAGS & FLAG_USB_HIGH_SPEED) ? 2 : 0)+
                ((USBDevConf->dreq.wIndex == 0x0409) ? 0 : 1) ][ dIndex ];
            ep0tx_data( SEL_DSP_MEM, ptr, *(BYTE *)ptr );
            break;
        //case DESCRIPTOR_INTERFACE :
        //     STALL(1);
        //   break;
        case DESCRIPTOR_ENDPOINT:
            STALL( 1 );
            break;
        case DESCRIPTOR_DEVICE_QUALIFIER:
            ep0tx_data( SEL_DSP_MEM, (DWORD)OTHER_DESCRIPTOR, DESCRIPTOR_DEVICE_QUALIFIER_SIZE );
            break;
        case DESCRIPTOR_OTHER_SPEED_CONFIGURATION:
            ep0tx_data( SEL_DSP_MEM, (DWORD)OTHER_DESCRIPTOR + DESCRIPTOR_DEVICE_QUALIFIER_SIZE,
                CONFIG_LENGTH );
            break;
        default:
            STALL( 1 );
            break;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_get_interface
    (
    void
    )
    {
	/* standard request -> get interface */
    STALL( 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_get_status
    (
    void
    )
    {
    /* standard request -> get status */
    switch (USBDevConf->dreq.bmRequestType & RECIPIENT_MASK)
        {
        case RECIPIENT_DEVICE:
            if (!(USBDevConf->USB_FLAGS & FLAG_USB_ADDRESS))
                {
                STALL( 1 );
                }
            else
                {
                *(DWORD *)&USBDevConf->dreq = 0;
                ep0tx_data( SEL_DSP_MEM, (DWORD)&USBDevConf->dreq, 2 );
                }
            break;
        case RECIPIENT_INTERFACE:
            if  (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                {
                STALL( 1 );
                }
            else
                {
                *(DWORD *)&USBDevConf->dreq = 0;
                ep0tx_data( SEL_DSP_MEM, (DWORD)&USBDevConf->dreq, 2 );
                }
            break;
        case RECIPIENT_ENDPOINT:
            if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                {
                STALL( 1 );
                }
            else
                {
                switch ((BYTE)USBDevConf->dreq.wIndex)
                    {
                    case 0x00:
                        *(WORD *)&USBDevConf->dreq = (USBDevConf->ep0.state == EP_HALTED) ? 1 : 0;
                        break;
                    case 0x01:
                    case 0x81:
                        *(WORD *)&USBDevConf->dreq = (USBDevConf->ep1.state == EP_HALTED) ? 1 : 0;
                        break;
                    case 0x02:
                        *(WORD *)&USBDevConf->dreq = (USBDevConf->ep2rx.state == EP_HALTED) ? 1 : 0;
                        break;
                    case 0x82:
                        *(WORD *)&USBDevConf->dreq = (USBDevConf->ep2tx.state == EP_HALTED) ? 1 : 0;
                        break;
                    }
                ep0tx_data( SEL_DSP_MEM, (DWORD)&USBDevConf->dreq, 2 );
            }
            break;
        default:
            STALL( 1 );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_set_address
    (
    void
    )
    {
	/* standard request -> set address */
    if (USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1)
        {
        STALL( 1 );
        }
    else
        {
        BYTE devaddr = USBDevConf->dreq.wValue;
        AccessISPRegWrite( ISP1583.D14_ADDRESS.BITS.DEVADDR = devaddr );
        ConfigUSB( (!devaddr) ? FLAG_USB_DEFAULT : FLAG_USB_ADDRESS );
        ACKNOWLEDGE( 1 );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_set_configuration
    (
    void
    )
    {
	/* standard request -> set configuration */
    BYTE config = USBDevConf->dreq.wValue;
    
    if (!(USBDevConf->USB_FLAGS & FLAG_USB_ADDRESS) || (config > 1))
        {
        STALL( 1 );
        }
    else
        {
        ConfigUSB( (!config) ? FLAG_USB_ADDRESS : FLAG_USB_CONFIGURATION1 );
        ACKNOWLEDGE( 1 );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_set_descriptor
    (
    void
    )
    {
	/* standard request -> set descriptor */
    STALL( 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_set_feature
    (
    void
    )
    {
    /* standard request -> set feature */
    switch (USBDevConf->dreq.bmRequestType & RECIPIENT_MASK)
        {
        case RECIPIENT_ENDPOINT:
            if (USBDevConf->dreq.wValue != FEATURE_ENDPOINT_HALT)
                {
                STALL( 1 );
                }
            else
                {
                switch ((BYTE)USBDevConf->dreq.wIndex)
                    {
                    case 0x00:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_ADDRESS))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x01:
                    case 0x81:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep1.state = EP_HALTED;
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x02:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep2rx.state = EP_HALTED;
                            // InitISP1583DMAC();
                            STALL( (2 << 1) + 0 );
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    case 0x82:
                        if (!(USBDevConf->USB_FLAGS & FLAG_USB_CONFIGURATION1))
                            {
                            STALL( 1 );
                            }
                        else
                            {
                            USBDevConf->ep2tx.state = EP_HALTED;
                            // InitISP1583DMAC();                  
                            STALL( (2 << 1) + 1 );
                            ACKNOWLEDGE( 1 );
                            }
                        break;
                    default:
                        STALL( 1 );
                    }
                }
            break;
        default : STALL(1);
        } /*switch (USBDevConf->dreq.bmRequestType & RECIPIENT_MASK)*/
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_set_interface
    (
    void
    )
    {
	/* standard request -> set interface */
    STALL( 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void std_undetected
    (
    void
    )
    {
	/* undetected standard request -> stall endpoint */
    STALL( 1 );
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
