#include "../hardware.h" 
#include "GlobDefs.h"


static volatile int abc_flg = 0;


/*================================================================================================*/
static void f_event_bus_reset(void);
static void f_event_bus_resume(void);
static void f_event_bus_suspend(void);
static void f_event_DMA(void);
static void f_event_ep0rx(void);
static void f_event_ep0tx(void);
static void f_event_ep0setup(void);
static void f_event_ep2rx(void);
static void f_event_ep2tx(void);
static void f_event_high_speed_status(void);
static void f_event_unhandled(void);
/*================================================================================================*/


/*================================================================================================*/
extern volatile int DMAErrorFlag = 0;
extern volatile unsigned Traffic = 0;
/* usb-event handler referance-table */
USB_request_Handler USBEventHandler[] =
    {
    f_event_bus_reset,                      /* 0 */
    f_event_unhandled,                      /* 1 */
    f_event_unhandled,                      /* 2 */
    f_event_bus_suspend,                    /* 3 */
    f_event_bus_resume,                     /* 4 */
    f_event_high_speed_status,              /* 5 */
    f_event_DMA,                            /* 6 */
    f_event_unhandled,                      /* 7 */
    f_event_ep0setup,                       /* 8 */
    f_event_unhandled,                      /* 9 */
    f_event_ep0rx,                          /* 10 */
    f_event_ep0tx,                          /* 11 */
    f_event_unhandled,                      /* 12 */
    f_event_unhandled,                      /* 13 */
    f_event_ep2rx,                          /* 14 */
    f_event_ep2tx                           /* 15 */
    };
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void ep0rx_data
    (
    BYTE sel,
    DWORD ptr,
    WORD length
    )
    {
    if (sel == SEL_AVR_PM)
        {
        /*    AVR */
        if ((0x2FF0 <= ptr) && (ptr < 0x3FF0))
            {
        	GetSystemStatusToStatusStruct();
            ptr -= 0x2FF0;
            ptr += (DWORD)&Descr;
            }
        sel = SEL_DSP_MEM;
        }
    else if (sel == SEL_AVR_DM)
        {
        if ((0x8000 <= ptr) && (ptr < 0x8020))
            {
            /*   */
            GetSystemRegistersToStatusStruct();
            ptr -= 0x8000;
            ptr += (DWORD)&ltr030Conf.LTR010_Emulation;
            USBDevConf->ChangeSystemRegisters = true;
            }
        sel = SEL_DSP_MEM;
        }
    else if (sel == SEL_FPGA_DATA)
        {
        length /= 2;
        }
    
    USBDevConf->ep0.sel = sel;
    USBDevConf->ep0.ptr = ptr;
    USBDevConf->ep0.cntr = min( length, USBDevConf->dreq.wLength );
    USBDevConf->ep0.state = EP_WAIT_RX;
    
    if (USBDevConf->ep0.cntr > 0)
        {
        AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 0 );
        AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.DSEN = 1 );
        }
    //f_event_ep0rx();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ep0tx_data
    (
    BYTE sel,
    DWORD ptr,
    WORD length
    )
    {
    if (sel == SEL_AVR_PM)
        {
        /*    AVR */
        if ((0x2FF0 <= ptr) && (ptr < 0x3FF0))
            {
        	GetSystemStatusToStatusStruct();
            ptr -= 0x2FF0;
            ptr += (DWORD)&Descr;
            }
        sel = SEL_DSP_MEM;
        }
    else if (sel == SEL_AVR_DM)
        {
        if ((0x8000 <= ptr) && (ptr < 0x8020))
            {
            /*   */
            GetSystemRegistersToStatusStruct();
            ptr -= 0x8000;
            ptr += (DWORD)&ltr030Conf.LTR010_Emulation;
            }
        sel = SEL_DSP_MEM;
        }
    else if (sel == SEL_DSP)
        {
        ;
        }
    else if (sel == SEL_FPGA_DATA)
        {
        return;
        }

    USBDevConf->ep0.sel = sel;
    USBDevConf->ep0.ptr = ptr;
    USBDevConf->ep0.cntr = min( length, USBDevConf->dreq.wLength );
    USBDevConf->ep0.state = EP_WAIT_TX;
    f_event_ep0tx();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ep2tx_data
    (
    BYTE sel,
    DWORD ptr,
    DWORD length
    )
    {
    USBDevConf->ep2tx.sel = sel;
    USBDevConf->ep2tx.ptr = ptr;
    USBDevConf->ep2tx.cntr = length;
    USBDevConf->ep2tx.state = EP_WAIT_TX;   
    AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE |= (1 << 15) );
    f_event_ep2tx();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_bus_reset
    (
    void
    )
    {
	/* bus reset event */
    ConfigUSB( FLAG_USB_DEFAULT );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_bus_resume
    (
    void
    )
    {
	/* bus resume event */
    USBDevConf->USB_FLAGS &= ~FLAG_USB_SUSPEND;
    g_mode_ULED = e_LED_Red;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_bus_suspend
    (
    void
    )
    {
	/* bus suspend event */
    USBDevConf->USB_FLAGS |= FLAG_USB_SUSPEND;
    g_mode_ULED = e_LED_Orange;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_DMA
    (
    void
    )
    {
	/*   ISP */
	if (USBDevConf->epDMA.cntr == 0)
	    {
	    if (USBDevConf->DataTransmittedFromPPI > 0)
	        {
            CommitDataFromPPIBuffer( USBDevConf->DataTransmittedFromPPI );
            USBDevConf->DataTransmittedFromPPI = 0;
	        }
        
        USBDevConf->DMAReady = true;
	    }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_ep0rx
    (
    void
    )
    {
	/* ep0rx event */
	t_uint16 epN_save;
	t_uint8 rx_size;
	WORD sz;
	
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 0 );
    rx_size = ISP1583.D14_BUFFER_LENGTH;
    
    DelayNs( 190 );
    
    sz = min( USBDevConf->ep0.cntr, rx_size );
    /* select memory space */
    switch (USBDevConf->ep0.sel)
        {     
        case SEL_DSP_MEM :        
            ReadAsync_asm( (WORD *)&ISP1583.D14_DATA_PORT.VALUE, (WORD*)USBDevConf->ep0.ptr, sz );
            break;
        case SEL_FPGA_DATA:
            if (USBDevConf->ep0.ptr + sz <= ALTERA_REGS_NUMBER)
                {
                int n = sz;
                struct altera_register *preg = &g_altera_reg[ USBDevConf->ep0.ptr ];
                while (n-- > 0)
                    {
                    preg->value = ISP1583.D14_DATA_PORT.VALUE;
                    preg->is_updated = 1;
                    preg++;
                    }
                }
            else
                {
                STALL( 1 );
                }
            break;
        default:
            STALL( 1 );
            break;
        }
    
    USBDevConf->ep0.ptr += sz;
    USBDevConf->ep0.cntr -= sz;
    
    if (USBDevConf->ep0.cntr == 0)
        {
        ACKNOWLEDGE( 1 );
        if (USBDevConf->ChangeSystemRegisters)
            {
            USBDevConf->ChangeSystemRegisters = false;
            SetSystemRegistersFromStatusStruct();
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_ep0setup
    (
    void
    )
    {
	/* ep0setup processing */
	t_uint16 epN_save;
	unsigned short b;
	
	//DelayNs( 300 );
	//epN_save = ISP1583.D14_ENDPT_INDEX;
	
    /* select setup endpoint */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 0x20 );

    /* read device request */
    for (b = 0; (b < 4); b++)
        {
        ((WORD *)&USBDevConf->dreq)[ b ] = ISP1583.D14_DATA_PORT.VALUE;
        }

    USBDevConf->dreq.bRequest &= REQUEST_MASK;
    
    switch (USBDevConf->dreq.bmRequestType & REQUEST_TYPE_MASK)
        {
        case STANDARD_REQUEST:
            StandardDeviceRequest[ USBDevConf->dreq.bRequest ]();
            break;
        case VENDOR_REQUEST:
            VendorDeviceRequest[ USBDevConf->dreq.bRequest ]();
            break;
        default:
            STALL( 1 );
            break;
        }
        
        
        
    //AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = epN_save );
    //DelayNs( 190 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_ep0tx
    (
    void
    )
    {
    t_uint16 epN_save;
    WORD *ptr;
    WORD sz;
    WORD Wordsz;
    	
    if (USBDevConf->ep0.cntr == 0)
        {
        ACKNOWLEDGE( 0 );
        return;
        }
        
    //DelayNs( 300 );
	//epN_save = ISP1583.D14_ENDPT_INDEX;
        
    /* select endpoint & packet size */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 1 );
    sz = min( USBDevConf->ep0.cntr, EP0_PACKET_SIZE );
    AccessISPRegWrite( ISP1583.D14_BUFFER_LENGTH = sz );
    Wordsz = sz >> 1;
    
    DelayNs( 100 );
    /* select memory space */
    switch (USBDevConf->ep0.sel)
        {    
        case SEL_DSP_MEM:
            ptr = (WORD *)USBDevConf->ep0.ptr;
            while (Wordsz-- > 0)
                {
            	AccessISPRegWrite( ISP1583.D14_DATA_PORT.VALUE = GetWORDFromAddr_asm( ptr++ ) );
                }
            if (sz & 1)
                {
                /*     -    */
            	AccessISPRegWrite( ISP1583.D14_DATA_PORT.VALUE = *(BYTE *)ptr );
                }
            break;
        case SEL_DSP:
            if (USBDevConf->ep0.ptr == 0x8000)
                {
                t_uint8 digin;
                if (g_board_version == 0)
                    {
                    digin = (*pPORTFIO >> 1) & 0x01 | (*pPORTGIO >> 12) & 0x02;
                    }
                else
                    {
                    digin = *pPORTFIO & 0x01 | (*pPORTGIO >> 12) & 0x02;
                    }
                AccessISPRegWrite( ISP1583.D14_DATA_PORT.VALUE = digin );
                }
            else if (USBDevConf->ep0.ptr == 0x8001)
                {
                AccessISPRegWrite( ISP1583.D14_DATA_PORT.VALUE = g_board_version );
                }
            else
                {
                STALL( 0 );
                }
            break;
        default:
            STALL( 0 );
            break;
        }
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.DSEN = 1 );
    USBDevConf->ep0.ptr += sz;
    USBDevConf->ep0.cntr -= sz;
    
    
    //AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = epN_save );
    //DelayNs( 190 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_ep2rx
    (
    void
    )
    {
    t_uint32 buf_remaind;
    unsigned BufferLengthBulkRx;
    //t_uint16 epN_save;
    unsigned iData;
    t_uint16 *pRXData;
    
    unsigned char bbs;
    volatile t_uint16 tmp_data;    
    
	
	//epN_save = ISP1583.D14_ENDPT_INDEX;
	AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
	BufferLengthBulkRx = ISP1583.D14_BUFFER_LENGTH;
	
	if (BufferLengthBulkRx == 0)
	    USBDevConf->ep2rxAbort = 1;
	
	BufferLengthBulkRx /= 2;	
	
	/*
	bbs = ISP1583.D14_BUFFER_STATUS;
	
	BufferLengthBulkRx /= 2;
	
	while (abc_flg)
	    {
	    tmp_data = ISP1583.D14_DATA_PORT.VALUE;
	    }
	*/
	
    if (USBDevConf->ep2rxEn && (USBDevConf->BufferLengthBulkRx[ 0 ] == 0))
	    {
    	get_ep2rx( BufferLengthBulkRx );
	
#if 0
    	pRXData = &SPORT_Tran_Data[ ltr030Conf.AlteraDev.CurSndBufferPtr ];
    	buf_remaind = SPORT_Transmit_Data_Size_Word - ltr030Conf.AlteraDev.CurSndBufferPtr;
        if (buf_remaind >= BufferLengthBulkRx)
    	    {
    	    iData = BufferLengthBulkRx;
    	    }
    	else
    	    {
    	    iData = buf_remaind;
    	    while (iData-- > 0)
    	        {
    	        *pRXData++ = ISP1583.D14_DATA_PORT.VALUE;
    	        }
    	    pRXData = &SPORT_Tran_Data[ 0 ];
    	    iData = BufferLengthBulkRx - buf_remaind;
    	    }
    	while (iData-- > 0)
    	    {
    	    *pRXData++ = ISP1583.D14_DATA_PORT.VALUE;
    	    }
	    
        if (buf_remaind > BufferLengthBulkRx)
            {
            ltr030Conf.AlteraDev.CurSndBufferPtr += BufferLengthBulkRx;
            }
        else
            {
            ltr030Conf.AlteraDev.CurSndBufferPtr = BufferLengthBulkRx - buf_remaind;
            }
	    
    	ltr030Conf.AlteraDev.SndBufAvail -= BufferLengthBulkRx;
        if (ltr030Conf.AlteraDev.SndBufAvail < USBDevConf->MaxBulkSize / 2)
            {
            USBDevConf->ep2rxEn = 0;
            //AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE &= ~(1U << 14) );
            }
#endif
	    }
	else
	    {
	    USBDevConf->BufferLengthBulkRx[ 0 ] += BufferLengthBulkRx;
	    }
    /*
	else if (USBDevConf->BufferLengthBulkRx[ 1 ] == 0)
	    {
	    USBDevConf->BufferLengthBulkRx[ 1 ] = BufferLengthBulkRx;
	    }
	*/
#if 0
	else
	    {
	    DelayNs( 100 );
	    }
#endif
		
	//AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = epN_save );
	//DelayNs( 190 );
    }	
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_ep2tx
    (
    void
    )
    {
    return;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_high_speed_status
    (
    void
    )
    {
	/* high speed status event */
    USBDevConf->USB_FLAGS |= FLAG_USB_HIGH_SPEED;
    USBDevConf->MaxBulkSize = HS_BULK_MAX_PACK_SZ;
    g_mode_ULED = e_LED_Green;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_event_unhandled
    (
    void
    )
    {
	/* unhendled event */
    STALL( 0 );
    }
/*------------------------------------------------------------------------------------------------*/
unsigned long dbg_cnt = 0;
/*------------------------------------------------------------------------------------------------*/
void get_ep2rx
    (
    t_uint16 data_len
    )
    {
     t_uint32 buf_remaind;
    unsigned BufferLengthBulkRx;
    unsigned iData;
    t_uint16 *pRXData;
    	
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
    DelayNs( 190 );
    
    pRXData = &SPORT_Tran_Data[ ltr030Conf.AlteraDev.CurSndBufferPtr ];
	buf_remaind = SPORT_Transmit_Data_Size_Word - ltr030Conf.AlteraDev.CurSndBufferPtr;
    if (buf_remaind >= data_len)
	    {
	    iData = data_len;
	    }
	else
	    {
	    iData = buf_remaind;
	    while (iData-- > 0)
	        {
	        *pRXData++ = ISP1583.D14_DATA_PORT.VALUE;
	        }
	    pRXData = &SPORT_Tran_Data[ 0 ];
	    iData = data_len - buf_remaind;
	    }
	    
	dbg_cnt += iData;
	    
	while (iData-- > 0)
	    {
	    *pRXData++ = ISP1583.D14_DATA_PORT.VALUE;
	    }
    
    if (buf_remaind > data_len)
        {
        ltr030Conf.AlteraDev.CurSndBufferPtr += data_len;
        }
    else
        {
        ltr030Conf.AlteraDev.CurSndBufferPtr = data_len - buf_remaind;
        }
    
	ltr030Conf.AlteraDev.SndBufAvail -= data_len * 2;
    if (ltr030Conf.AlteraDev.SndBufAvail < USBDevConf->MaxBulkSize)
        {
        USBDevConf->ep2rxEn = 0;
        //AccessISPRegWrite( ISP1583.D14_INT_ENABLE.VALUE &= ~(1U << 14) );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitDMA
    (
    void
    )
    {
    unsigned i;	
    extern int tx_en;
    
    if (!USBDevConf->DMAReady)
        {
        /* DMA   ,   DMA */
        AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = GDMA_STOP );
        USBDevConf->DMAReady = true;
        }
        
    /*      */
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 3 );

    /*   (    RX buffer) */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );

    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    AccessISPRegWrite( ISP1583.D14_CONTROL_FUNCTION.BITS.CLBUF = 1 );

    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 2 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void InitISP1583DMAC
    (
    void
    )
    {
    /* globale interrupt disable */
    DisableISPInterrupt;                    /* disable Interrupt */

	/*  endpidx=1, ep0setup=EP0 data buffer, dir=RX */
	/*  DMA_ENDPOINT        */
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 2 );
	
	/* reset dma=0x11 */
    AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = DMA_RESET );
	
	/*   DMA */
    
	/* width=1 -   16 , mode=2 -  DACK, 
	 * ata_mode=0 -   ata, dma_mode=0 -  mdma mode 0
	 */
    AccessISPRegWrite( ISP1583.D14_DMA_CONFIG.VALUE = 1 | (2 << 2) );

    /* master=0 - GDMA slave mode, eot_pol=dreq_pol=dacq_pol=1 - , xxx_pol=0 */
    AccessISPRegWrite( ISP1583.D14_DMA_HARDWARE.VALUE = (1 << 5) | (1 << 2) | (1 << 3) );

    /* enable DMA interrupts - XFER_OK -   -   ,    
     *    
     */
	AccessISPRegWrite( ISP1583.D14_DMA_INT_ENABLE.VALUE = (1 << 8) );

	/* 512    -    */
    AccessISPRegWrite( ISP1583.D14_DMA_BURST_COUNTER = USBDevConf->MaxBulkSize);

    /* globale interrupt enable */
    EnableISPInterrupt;                     /* enable Interrupt */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void put_ep2data
    (
    t_uint16 data[],
    unsigned long data_len
    )
    {
    unsigned st;
        
        
    if (data_len == 0)
        return;
        
    
    PF_PINS_INTERRUPT_B_CLR( USBDevConf_PORTF_INT | USBDevConf_PORTF_DREQ );
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 5 );
    
    st = ISP1583.D14_BUFFER_STATUS;
    
    AccessISPRegWrite( ISP1583.D14_ENDPT_INDEX = 4 );
    
    
    USBDevConf->epDMA.ptr = data;
	USBDevConf->epDMA.cntr = data_len;
    
           	  
	USBDevConf->DMAReady = false;
	
	ssync();
	
	/*  PF8 */
	PF_PINS_CLR( USBDevConf_PORTF_DREQ );
	
	/* start read DMA ISP */
	/* endpoint=5   Bulk */
    AccessISPRegWrite( ISP1583.D14_DMA_ENDPOINT = 5 );
	/*  DSP   ,    */
    AccessISPRegWrite( ISP1583.D14_DMA_TRANSFER_COUNTER = (data_len * 2) );
    /*  DMA */
    AccessISPRegWrite( ISP1583.D14_DMA_COMMAND = GDMA_Read_Command );
    
	/*   */
	PF_PINS_INTERRUPT_B_SET( USBDevConf_PORTF_INT | USBDevConf_PORTF_DREQ );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ReadEP0
    (
    BYTE *Data,
    WORD NumData
    )
    {
    ep0rx_data( SEL_DSP_MEM, (DWORD)Data, NumData );
    while (USBDevConf->ep0.cntr && !GlobalStop)
        {
        continue;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WriteEP0
    (
    BYTE *Data,
    WORD NumData
    )
    {
    ep0tx_data( SEL_DSP_MEM, (DWORD)Data, NumData );
    while (USBDevConf->ep0.cntr && !GlobalStop)
        {
        continue;
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
