#include "../hardware.h" 


/*==============================================================================================*/
int g_altera_reset = 0;
/*==============================================================================================*/


/*==============================================================================================*/
static void f_ven_boot_loader_get_array(void);
static void f_ven_boot_loader_put_array(void);
static void f_ven_call_application(void);
static void f_ven_Command_DSP(void);
static void f_ven_get_array(void);
static void f_ven_get_module_name(void);
static void f_ven_GetArrayFromFlash(void);
static void f_ven_GetTestInfo(void);
static void f_ven_Init_ISP_DMA(void);
static void f_ven_put_array(void);
static void f_ven_PutArrayToFlash(void);
static void f_ven_reset_Altera(void);

static void f_ven_reset_bulkUSB(void);

static void f_ven_ResetDSP(void);
static void f_ven_SpecialFlashCommand(void);
static void f_ven_undetected(void);
/*================================================================================================*/

/*================================================================================================*/
/* vendor request-handler referance-table */
USB_request_Handler VendorDeviceRequest[] =
    {
    f_ven_reset_Altera,                     /* 0 */
    f_ven_put_array,                        /* 1 */
    f_ven_get_array,                        /* 2 */
    f_ven_GetTestInfo,                      /* 3 */

    f_ven_Init_ISP_DMA,                     /* 4 */
    f_ven_Command_DSP,                      /* 5 */
    f_ven_undetected,                       /* 6 */
    f_ven_undetected,                       /* 7 */

    f_ven_SpecialFlashCommand,              /* 8 */
    f_ven_PutArrayToFlash,                  /* 9 */
    f_ven_GetArrayFromFlash,                /* 10 */
    f_ven_get_module_name,                  /* 11 */

    f_ven_ResetDSP,                         /* 12 */
    f_ven_boot_loader_put_array,            /* 13 */
    f_ven_boot_loader_get_array,            /* 14 */
    f_ven_call_application                 /* 15 */
    
    //f_ven_reset_bulkUSB                     /* 16 */
    };
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
static void f_ven_boot_loader_get_array
    (
    void
    )
    {
	/* vendor request -> get array */
    ep0tx_data( (BYTE)(USBDevConf->dreq.wIndex >> 8), USBDevConf->dreq.wValue,
        USBDevConf->dreq.wLength );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_boot_loader_put_array
    (
    void
    )
    {
    /* vendor request -> put array */
    ep0rx_data( (BYTE)(USBDevConf->dreq.wIndex >> 8), USBDevConf->dreq.wValue,
        USBDevConf->dreq.wLength );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_call_application
    (
    void
    )
    {
	/* vendor request -> call application */
    if(/**(WORD *)*/USBDevConf->dreq.wValue != 0xFFFF)
        {
        ACKNOWLEDGE( 1 );                                           
        }
    else
        {
        STALL( 1 );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_Command_DSP
    (
    void
    )
    {
	/*  DSP */
    t_ControlRequest ControlRequest =
        {0, USBDevConf->dreq.wValue, USBDevConf->dreq.wLength, USBDevConf->dreq.wIndex, USB_Device};
        
    ltr030Conf.AddNewCommandHandler( &ControlRequest, NULL, 0 );
  
    if (!USBDevConf->dreq.wLength)
        {
        if (    (USBDevConf->dreq.bmRequestType & REQUEST_ANSWER_DIRECTION)
             == REQUEST_ANSWER_DIRECTION_HOST_TO_DEVICE
           )
            {
            /*     DSP */
            ACKNOWLEDGE( 1 );
            }
        else
            {
            /*     DSP */
            ACKNOWLEDGE( 0 );
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_get_array
    (
    void
    )
    {
	/* vendor request -> get array */
    ep0tx_data( (BYTE)(USBDevConf->dreq.wIndex >> 8), USBDevConf->dreq.wValue,
        USBDevConf->dreq.wLength );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_get_module_name
    (
    void
    )
    {
    /* vendor request -> get module name */
    ep0tx_data( SEL_DSP_MEM, (DWORD)&Descr.Descr, DESC_MODULE_NAME_SIZE );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_GetArrayFromFlash
    (
    void
    )
    {
	/*    */
	DWORD Address = USBDevConf->dreq.wValue | ((DWORD)USBDevConf->dreq.wIndex << 16);
	t_ControlRequest ControlRequest = 
	    {0, ltr030_CC_ReadBootFlash, USBDevConf->dreq.wLength, Address, USB_Device};
	ltr030Conf.AddNewCommandHandler( &ControlRequest, NULL, 0 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_GetTestInfo
    (
    void
    )
    {
	/* vendor request 3 - GetTestInfo */
    unsigned TestInfoArray[ 3 ];
    
    TestInfoArray[ 0 ] = AlteraReadCommand( 0x14 );

    TestInfoArray[ 1 ] = (unsigned)AlteraReadCommand( 0x10 ) |
        ((unsigned)AlteraReadCommand( 0x11 ) << 8) | ((unsigned)AlteraReadCommand( 0x12 ) << 16) |
        ((unsigned)AlteraReadCommand( 0x13 ) << 24);
	    			
    TestInfoArray[ 2 ] = (unsigned)AlteraReadCommand( 0x18 ) |
        ((unsigned)AlteraReadCommand( 0x19 ) << 8) | ((unsigned)AlteraReadCommand( 0x1A ) << 16) |
        ((unsigned)AlteraReadCommand( 0x1B ) << 24);
    					
    					
    ep0tx_data( SEL_DSP_MEM, (DWORD)TestInfoArray, sizeof TestInfoArray );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_Init_ISP_DMA
    (
    void
    )
    {
//    InitDMA();
    //g_need_reset_USB = 1;
    ACKNOWLEDGE( 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_put_array
    (
    void
    )
    {
	/* vendor request -> put array */
    ep0rx_data( (BYTE)(USBDevConf->dreq.wIndex >> 8), USBDevConf->dreq.wValue,
        USBDevConf->dreq.wLength );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_PutArrayToFlash
    (
    void
    )
    {
	/*  -   */
	DWORD Address = USBDevConf->dreq.wValue | ((DWORD)USBDevConf->dreq.wIndex << 16);
	t_ControlRequest ControlRequest = 
	    {0, ltr030_CC_WriteBootFlash, USBDevConf->dreq.wLength, Address, USB_Device};	
	ltr030Conf.AddNewCommandHandler( &ControlRequest, NULL, 0 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_reset_Altera
    (
    void
    )
    {
    //AlteraDev->NewModulesAppearence = true;
    g_altera_reset = 1;
    ACKNOWLEDGE( 1 );
    
    
    //AlteraWriteCommand( 0x00, 0x00 );
    //InitDMA();
    //AlteraWriteCommand( 0x00, 0x01 );
    
    /*
    AlteraWriteCommand( 0x00, 0x00 );
    InitDMA();
    AlteraWriteCommand( 0x00, 0x01 );
    CloseAlteraDataInterface();
    SetDefaultAlteraConfiguration( &ltr030Conf.AlteraDev );
    OpenAlteraDataInterface( &ltr030Conf.AlteraDev );
    ltr030Conf.USBDev.ep2rxEn = 1;
    */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_reset_bulkUSB
    (
    void
    )
    {
    g_need_reset_USB = 1;
    ACKNOWLEDGE( 1 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_ResetDSP
    (
    void
    )
    {
    /*  DMA   USB	*/
    if (!USBDevConf->dreq.wValue)
        {
        /*   */
        t_ControlRequest ControlRequest = {0, ltr030_CC_SystemReset, 0, 0, USB_Device };
        ACKNOWLEDGE( 1 );
        GlobalStop = 1;
        ltr030Conf.AddNewCommandHandler( &ControlRequest, NULL, 0 );
        }
    else
        {
        /*  DMA -   */
        ACKNOWLEDGE( 1 );
        ep2tx_data( SEL_DSP_MEM, (DWORD)&ISP1583.D14_DMA_TRANSFER_COUNTER, 8 );
        InitDMA();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_SpecialFlashCommand
    (
    void
    )
    {
	/*     */
	t_ControlRequest ControlRequest =
	    {0,
	    ltr030_CC_SpecBootFlashCmd,
	    USBDevConf->dreq.wLength,
	    USBDevConf->dreq.wIndex,
	    USB_Device
	    };	
	ltr030Conf.AddNewCommandHandler( &ControlRequest, NULL, 0 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ven_undetected
    (
    void
    )
    {
	/* undetected vendor request -> stall endpoint */
    STALL( 0 );
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
