#ifndef LTR030_H_
#define LTR030_H_


/*================================================================================================*/
/* SDRAM */
#define	SDRAM_ADDRESS ((unsigned short *)0x00)
#define	SDRAM_BANK0 ((unsigned short *)0x000000000)
#define	SDRAM_BANK1 ((unsigned short *)0x000800000)
#define	SDRAM_BANK2 ((unsigned short *)0x001000000)
#define	SDRAM_BANK3 ((unsigned short *)0x001800000)

/*   0 */
/*    ISP -   */
#define ISP1583 (*((volatile D14_CNTRL_REG *)0x20000000))

/*   1 */
/*    ISP  DMA */
#define pBULK_DATA ((volatile WORD *)0x20100000)

/*   2 */
/*  2 */
#define pBANK2_DATA ((volatile WORD *)0x20200000)

/*   3 */
/*  3 */
#define pBANK3_DATA ((volatile WORD *)0x20300000)

/*  */
#define ISP1583_DREQ_DMAR_F PF0
#define ISP1583_INT_F PF2

#define ALTERA_REG_CS_F PF5

#define ISP1583_DREQ_F PF8

#define PPI_FS1_F PF9	
			
#define ATFlash_CS_F PF10

#define PPI_CLK_F PF15

#define ALTERA_CONF_DONE_F PF14

#define ALTERA_NCONF_G PG8
#define ALTERA_NSTAT_G PG9

#define ISP1583_RESET_G PG10
#define ALTERA_POS_WRITE_G PG11
#define ALTERA_MODULE_CHANGE PG12
#define ISP1583_READY_G PG15
#define ISP1583_READY2_G PG14

#define FAN_NUM (4)

#define ALTERAREG_FANCTRL (6)

#define DEVICE_CONFIG_FLASH_START_ADDRESS (0x1E6E00)


#ifdef LTR030DEBUBG
  #define LTR030DEBUG_PRINT(x)  printf x
#else
  #define LTR030DEBUG_PRINT(x)
#endif

#ifdef LTR030DEUBG_STAT
  #define LTR030DEBUG_STAT_PRINT(x) printf x
#else
  #define LTR030DEBUG_STAT_PRINT(x)
#endif
/*================================================================================================*/

/*================================================================================================*/
#pragma pack(1)
typedef struct
    {
    BYTE Name[16];
    BYTE SerialNumber[16];
    BYTE CpuType[16];
    BYTE Revision;
    long QuartzFrequency;
    BYTE Unused[128-53];    
    } t_DEVICE_DESCR;
typedef struct
    {
    BYTE   CPUName[16];
    DWORD  CurrCPUFreq;
    DWORD  CurrSystemFreq;
    WORD   SDRAM_Size_Mb;
    WORD   SD_Card_Presense;
    WORD   Temperature;
    BYTE Unused[64-30];    
    } t_DEVICE_STATUS;
typedef struct
    {
	DWORD FirmwareVersion;
	BYTE Unused[12];
    t_DEVICE_DESCR Descr;
    t_DEVICE_STATUS Status;
    } t_FULL_DEVICE_DESCR_STATUS;
typedef struct 
    {
    BYTE PhillipsDMA;
    BYTE SDRAM;
    BYTE SpecialTechno;
    BYTE LowFIFOSize;
    BYTE FIFOStatus;
    BYTE ErrorStatus;
    BYTE SpeedSerial_Low;
    BYTE SpeedSerial_High;
    BYTE Module_Presence_Low;
    BYTE Module_Presence_High;
    } t_LTR10_Hardware_Registers_Struct;
#pragma pack()

typedef enum {USB_Device = 1, Ether_Device = 2} t_CommandDeviceType;

#pragma pack(2)
typedef struct 
    {
    WORD OveralLength;
    WORD CommandCode;
    WORD ReturnBytes;
    DWORD Parameter;
    t_CommandDeviceType DeviceRequested;
    } t_ControlRequest;
#pragma pack()

typedef bool (*CommandHandler) (t_ControlRequest *ControlRequest, BYTE *Data, DWORD NumData);

#pragma pack(4)
typedef struct
    {	
	t_USB_DevConfStruct USBDev;
    t_ETHERNET_BF537_CONF EtherDev; 
    t_AlteraDataStruct AlteraDev;  
    t_LTR10_Hardware_Registers_Struct LTR010_Emulation;  
    int DataClientSocket;    
    CommandHandler AddNewCommandHandler;
    DWORD CommandDataWritePointer;
    DWORD CommandDataReadPointer;
    DWORD CommandDataCounter;
    } t_ltr030_Conf;
#pragma pack()

typedef enum
    {
    ltr030_CC_GlobalStop = 0,
    ltr030_CC_GetName = 1,
    ltr030_CC_GetDescriptor = 2,
    ltr030_CC_StartSpeedTest = 3,
    ltr030_CC_StartDataThreads = 4,
    ltr030_CC_WriteBootFlash = 5,
    ltr030_CC_ReadBootFlash = 6,
    ltr030_CC_SpecBootFlashCmd = 7,
    ltr030_CC_SystemReset = 8,
    ltr030_CC_TestWriteRingControl = 9,
    ltr030_CC_TestReadRingControl = 10,
    ltr030_CC_TestRingBulk = 11,
    ltr030_CC_TestWriteBulk = 12,
    ltr030_CC_TestReadBulk = 13,
    ltr030_CC_GetDebugStatus = 14,
    ltr030_CC_GetThermo = 15,
    ltr030_CC_GetFanSpeed1 = 16,
    ltr030_CC_GetFanSpeed2 = 17,
    ltr030_CC_GetFanSpeed3 = 18,
    ltr030_CC_GetFanSpeed4 = 19,
    ltr030_CC_StartSDTest = 20,
    ltr030_CC_GetSDTestResult = 21,
    ltr030_CC_SetTime = 22,
    ltr030_CC_GetTime = 23,
    ltr030_CC_GetSDStatus = 24,
    ltr030_CC_UARTTest = 25,
    ltr030_CC_TestBulkFrame = 26
    } t_ltr030ControlCommand;  
/*================================================================================================*/

/*================================================================================================*/
extern const DWORD CrateSygnalBufferOverflow[ 2 ];
extern const DWORD CrateSygnalDataNewModule[ 2 ];
extern const DWORD CrateSygnalDMABufferOverflow[ 2 ];
extern t_FULL_DEVICE_DESCR_STATUS Descr;
extern t_int16 g_crate_temperature;
extern t_uint32 g_fan_speed[ FAN_NUM ];
extern t_ltr030_Conf ltr030Conf;
/*================================================================================================*/

/*================================================================================================*/
void SetSystemRegistersFromStatusStruct(void);
void GetSystemRegistersToStatusStruct(void);
void GetSystemStatusToStatusStruct(void);
/*================================================================================================*/

#endif /*#ifndef LTR030_H_*/
