#include "GlobDefs.h" 
#include "interfaces/bf537_spi.h"

#include "rtc.h"

#include "ifuart.h"

#if defined(LTR030_ENDPOINT)
#include "fanning.h"
#endif


int g_bulkframe_size = 0;


/*================================================================================================*/
#define CONTROL_QUEUE_SIZE (128*1024)// ControlQueueSize (128*1024)
/*================================================================================================*/

/*================================================================================================*/
static void f_GetCommandMutex(void);
static int f_GetCommandRequest(t_ControlRequest *ControlRequest, BYTE *Data, DWORD NumData);
static int f_ProcessCommandEther(t_ControlRequest *ControlRequest);
static int f_ProcessCommandUSB(t_ControlRequest *ControlRequest);
static void f_ReleaseCommandMutex(void);
/*================================================================================================*/

/*================================================================================================*/
static int f_CommandInterruptLatch[ 10 ];
static t_uint16 f_CommandInterruptLatchDeepCounter;
segment ("sdram0_bank1") BYTE ProcessCommandQueue[ CONTROL_QUEUE_SIZE ];
segment ("sdram0_bank1") WORD RequestData[ 4096 ];
segment ("sdram0_bank1") WORD TempArray[ 128*1024 ];

static hwInterface f_h_sd;
static t_uint8 f_sdtest_buf[ 512 ];
static t_uint16 f_sdtest_result;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
bool AddNewCommandRequest
    (
    t_ControlRequest *ControlRequest,
    BYTE *Data,
    DWORD NumData
    )
    {
    unsigned long i;
    DWORD TotalSize = NumData;
    
    if (ControlRequest != NULL)
        {
    	TotalSize += sizeof(t_ControlRequest);
    	ControlRequest->OveralLength = TotalSize;
    	if (ControlRequest->CommandCode == ltr030_CC_GlobalStop)
    	    GlobalStop = 1;
        }

    if (ltr030Conf.CommandDataCounter + TotalSize > CONTROL_QUEUE_SIZE)
        return false;

    f_GetCommandMutex();	

    
    if (ControlRequest != NULL)
        {
        BYTE *ptr = (BYTE *)ControlRequest;
        for (i = 0; (i < sizeof(t_ControlRequest)); i++)
            {
            ProcessCommandQueue[ ltr030Conf.CommandDataWritePointer++ ] = *ptr++;
            if (ltr030Conf.CommandDataWritePointer >= CONTROL_QUEUE_SIZE)
                ltr030Conf.CommandDataWritePointer = 0;
            }
        }

    for (i = 0; (i < NumData); i++)
        {
        ProcessCommandQueue[ ltr030Conf.CommandDataWritePointer++ ] = *Data++;
        if (ltr030Conf.CommandDataWritePointer >= CONTROL_QUEUE_SIZE)
            ltr030Conf.CommandDataWritePointer = 0;
        }
    ltr030Conf.CommandDataCounter += TotalSize;
    
#ifdef _VDK_
	VDK_C_ISR_PostSemaphore( kNewCommandAdded );
#endif        

    f_ReleaseCommandMutex();
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_GetCommandMutex
    (
    void
    )
    {	
    f_CommandInterruptLatch[ f_CommandInterruptLatchDeepCounter++ ] = disable_interrupt;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_GetCommandRequest
    (
    t_ControlRequest *ControlRequest,
    BYTE *Data,
    DWORD NumData
    )
    {
    unsigned long i;
    DWORD TotalSize = NumData;
    
    if (ControlRequest != NULL)
        TotalSize += sizeof(t_ControlRequest);
    if (ltr030Conf.CommandDataCounter < TotalSize)
        return 0;

    f_GetCommandMutex();	

    if (ControlRequest != NULL)
        {
        BYTE *ptr = (BYTE *)ControlRequest;
        for (i = 0; (i < sizeof(t_ControlRequest)); i++)
            {
            *ptr++ = ProcessCommandQueue[ ltr030Conf.CommandDataReadPointer++ ];
            if (ltr030Conf.CommandDataReadPointer >= CONTROL_QUEUE_SIZE)
                ltr030Conf.CommandDataReadPointer = 0;
            }
        }

    for (i = 0; (i < NumData); i++)
        {
        *Data++ = ProcessCommandQueue[ ltr030Conf.CommandDataReadPointer++ ]; 
        if (ltr030Conf.CommandDataReadPointer >= CONTROL_QUEUE_SIZE)
            ltr030Conf.CommandDataReadPointer = 0;
        }

    ltr030Conf.CommandDataCounter -= TotalSize;
    f_ReleaseCommandMutex();

    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_ProcessCommandEther
    (
    t_ControlRequest *ControlRequest
    )
    {
    int Answer = 0x10101010;
	int ClientSocket = ControlRequest->Parameter;
	
#ifdef _USE_LWIP_        
    switch (ControlRequest->CommandCode)
        {
        case ltr030_CC_GlobalStop:
            GlobalStop = 1;
            break;
        case ltr030_CC_GetName:        
            send( ClientSocket, &Descr, ControlRequest->ReturnBytes, 0 );
            break;
        case ltr030_CC_GetDescriptor:        
            send( ClientSocket, &Descr, ControlRequest->ReturnBytes, 0 );
            break;
        case ltr030_CC_StartSpeedTest:
            TestSpeedWriteRunning = 0;
            TestSpeedReadRunning = 0;

            while (!VDK_PendSemaphore( kSendTestSema, 0 ))
                {
                continue;
                }
            while (!VDK_PendSemaphore( kRecvTestSema, 0 ))
                {
                continue;
                }			    	
        
            write( ClientSocket, (BYTE *)&Answer, sizeof Answer );

            TestSpeedWriteRunning = RequestData[ 0 ];
            TestSpeedReadRunning = RequestData[ 0 ] >> 8;

            NumTestSpeedData = RequestData[ 1 ];

            if (TestClientSocket != INVALID_SOCKET)
                close( TestClientSocket );
            TestClientSocket = ClientSocket;
            ClientSocket = INVALID_SOCKET;

            VDK_CreateThread( kTestStackThreadSendType );
            VDK_CreateThread( kTestStackThreadRecvType );
            break;
        case ltr030_CC_StartDataThreads:        
            DataSendToPCRunning = 0;
            DataRecvFromPCRunning = 0;

            while (!VDK_PendSemaphore( kDataRecvFromPCSema, 0 ))
                {
                continue;
                }			    	    			    	    
            while (!VDK_PendSemaphore( kDataSendToPCSema, 0 ))
                {
                continue;
                }			    	    

            DataSendToPCRunning = RequestData[ 0 ];
            DataRecvFromPCRunning = RequestData[ 0 ] >> 8;

            if (ltr030Conf.DataClientSocket != INVALID_SOCKET)
                close( ltr030Conf.DataClientSocket );
            ltr030Conf.DataClientSocket = ClientSocket;
            ClientSocket = INVALID_SOCKET;

            VDK_CreateThread( kDataSendToPC );
            VDK_CreateThread( kDataRecvFromPC );
            break;
        case ltr030_CC_WriteBootFlash:
            break;
        case ltr030_CC_ReadBootFlash:
            break;
        case ltr030_CC_SpecBootFlashCmd:
            break;
        case ltr030_CC_SystemReset:
            SystemReset( false );
            break;
        default:
            LTR030DEBUG_PRINT( ("Unknown Command\n") );
            return 0;
        } /*switch (ControlRequest->CommandCode)*/
    
    if (ClientSocket != INVALID_SOCKET)
        close( ClientSocket );
#endif    
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_ProcessCommandUSB
    (
    t_ControlRequest *ControlRequest
    )
    {
    unsigned i;
    int j;
    t_uint16 sd_status;
    	
	switch (ControlRequest->CommandCode)
        {
        case ltr030_CC_GlobalStop:
            GlobalStop = 1;
            break;
        case ltr030_CC_GetName:        
            break;
        case ltr030_CC_GetDescriptor:        
            break;
        case ltr030_CC_StartSpeedTest:		            		    	
            break;
        case ltr030_CC_StartDataThreads:    
            break;
        case ltr030_CC_WriteBootFlash:                
          	ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            OpenATFlashInterface();
            ProgrammFlash( ControlRequest->Parameter, (BYTE *)RequestData,
                ControlRequest->ReturnBytes );
            CloseATFlashInterface();                    
            break;
        case ltr030_CC_ReadBootFlash:                                 
            OpenATFlashInterface();        
            ContiniousReadFlash( ControlRequest->Parameter, (BYTE *)RequestData,
                ControlRequest->ReturnBytes );
            CloseATFlashInterface();
            WriteEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_SpecBootFlashCmd:
            OpenATFlashInterface();  
            switch(ControlRequest->Parameter)
                {
                case 1:                /*  flash    2 */
                    SetFlashToPowerOfTwo();
                    /*  acknowledge */
                    ReadEP0( NULL, 0 );
                    break;
                case 2:                /*   */
                    GetFlashStatus( &((BYTE *)RequestData)[ 0 ] );
                    ((BYTE *)RequestData)[ 1 ] = GetFlashSectorProtectionRegister();
                    ((BYTE *)RequestData)[ 2 ] = GetFlashSectorLockDownRegister();
                    ((BYTE *)RequestData)[ 3 ] = GetFlashSecurityRegister();
                    GetFlashChipId( &((BYTE *)RequestData)[ 4 ] );
                    WriteEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
                    break;
                case 3:                /*     */
                    for (i = 32; (i <= 255); i += 32)
                        {
                        EraseFlashSector( i & 0xFF );
                        }
                    /*  acknowledge */
                    ReadEP0( NULL,0 );
                    break;
                default:
                    break;
                }            
            CloseATFlashInterface();
            break;
        case ltr030_CC_SystemReset:
            SystemReset( false );
            break;
        case ltr030_CC_TestWriteRingControl:
            ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_TestReadRingControl:
            WriteEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_TestRingBulk:          
            ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
        	InitDMA();
    	
    //    	AlteraWriteCommand(0x10,0xFFFF);	//  AWE  ARE
    	
            /*    -    */
        	while(!GlobalStop)
        	    {
        		//start_readDMA_ISP( TempArray, ControlRequest->Parameter );
        		while (ISP_DMA_Running() && !GlobalStop)
        		    {
        		    continue;
        		    }
        		DelayCycles( *(DWORD *)RequestData );
    //    		start_writeDMA_ISP_DMAR(TempArray,ControlRequest->Parameter);
        		//start_writeDMA_ISP( TempArray, ControlRequest->Parameter );
    			while (ISP_DMA_Running() && !GlobalStop)
    			    {
    			    continue;
    			    }
        		DelayCycles( *(DWORD *)RequestData );			
        	    }
        
            break;
        case ltr030_CC_TestWriteBulk:
            ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            InitDMA();
        
    //    	AlteraWriteCommand(0x10,0xFFFF);	//  AWE  ARE        
        
            /*    -    */
    		while(!GlobalStop)
    		    {
    			//start_readDMA_ISP( TempArray, ControlRequest->Parameter );
    			while (ISP_DMA_Running() && !GlobalStop)
    			    {
    			    continue;
    			    }
    			DelayCycles( *(DWORD *)RequestData );
    		    }
            break;
        case ltr030_CC_TestReadBulk:
        	ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            InitDMA();
    		for (i = 0; (i < ControlRequest->Parameter); i++)
    		    {
    		    TempArray[ i ] = i;
    		    }
    		/*    -    */
    		while (!GlobalStop)
    		    {
//    			start_writeDMA_ISP_DMAR(TempArray,ControlRequest->Parameter);
				//start_writeDMA_ISP( TempArray,ControlRequest->Parameter );
    			while (ISP_DMA_Running() && !GlobalStop)
    			    {
    			    continue;
    			    }
    			DelayCycles( *(DWORD *)RequestData );
    		    }
        break;
        case ltr030_CC_GetDebugStatus:
    //    	WriteEP0((BYTE *)&ErrorCounter, ControlRequest->ReturnBytes);         
            break;                    
        case ltr030_CC_StartSDTest:
            portDISABLE_INTERRUPTS();
            g_altera_reg[ 0x01 ].value |= 0x80;
            AlteraWriteCommand( 0x01, g_altera_reg[ 0x01 ].value );
            portENABLE_INTERRUPTS();
            
            for (i = 0; (i < 10000); i++)
                {
                continue;
                }
            
            f_sdtest_result = 0;
            if (if_initInterface( &f_h_sd, NULL ) != 0)
                f_sdtest_result = 1;
            
            if (f_sdtest_result == 0)
                {
                for (i = 0; (i < 512); i++)
                    {
                    f_sdtest_buf[ i ] = i & 0xFF;
                    }
                for (j = 0; (j < 2048); j++)
                    {
                    clear_extwdt();
                    if (if_writeBuf( &f_h_sd, 1, f_sdtest_buf ) != 0)
                        {
                        f_sdtest_result = 2;
                        break;
                        }
                    }
                }
            if (f_sdtest_result == 0)
                {
                for (j = 0; (j < 2048); j++)
                    {
                    clear_extwdt();
                    if (if_readBuf( &f_h_sd, 1, f_sdtest_buf ) != 0)
                        f_sdtest_result = 3;
                    for (i = 0; ((i < 512) && (f_sdtest_result == 0)); i++)
                        {
                        if (f_sdtest_buf[ i ] != (i & 0xFF))
                            {
                            f_sdtest_result = 4;
                            break;
                            }
                        }
                    if (f_sdtest_result != 0)
                        break;
                    }
                }
            
                
            portDISABLE_INTERRUPTS();
            g_altera_reg[ 0x01 ].value &= ~0x80;
            AlteraWriteCommand( 0x01, g_altera_reg[ 0x01 ].value );
            portENABLE_INTERRUPTS();
            
            break;
        case ltr030_CC_GetSDTestResult:
            WriteEP0( (BYTE *)&f_sdtest_result, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_SetTime:
            ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            if (g_board_version != 0)
                {
                enum RTCErr e;
                BYTE *pbyte = (BYTE *)RequestData;
                struct rtc_tm t;
                
                t.sec = pbyte[ 0 ];
                t.min = pbyte[ 1 ];
                t.hour = pbyte[ 2 ];
                t.day = pbyte[ 3 ];
                t.mon = pbyte[ 4 ];
                t.year = pbyte[5 ];
                set_rtc( &t, &e );
                }
            break;
        case ltr030_CC_GetTime:
            if (g_board_version != 0)
                {
                BYTE data[ 6 ];
                enum RTCErr e;
                struct rtc_tm t;
                    
                get_rtc( &t, &e );
                data[ 0 ] = t.sec;
                data[ 1 ] = t.min;
                data[ 2 ] = t.hour;
                data[ 3 ] = t.day;
                data[ 4 ] = t.mon;
                data[ 5 ] = t.year;
                WriteEP0( data, ControlRequest->ReturnBytes );
                }
            break;
        case ltr030_CC_GetSDStatus:
            if (g_board_version != 0)
                {
                portDISABLE_INTERRUPTS();
                sd_status = ((AlteraReadCommand( 0x01 ) & 0x80) != 0);
                portENABLE_INTERRUPTS();
                }
            else
                {
                sd_status = 0;
                }
            WriteEP0( (BYTE *)&sd_status, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_UARTTest:
            if (g_board_version != 0)
                {
                uart_init();
                sd_status = test_uart();
                uart_deinit();
                }
            else
                {
                sd_status = 0;
                }
            WriteEP0( (BYTE *)&sd_status, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_TestBulkFrame:
            ReadEP0( (BYTE *)RequestData, ControlRequest->ReturnBytes );
            g_bulkframe_size = RequestData[ 0 ];
            break;
#if defined(LTR030_ENDPOINT)
        case ltr030_CC_GetThermo:
            WriteEP0( (BYTE *)&g_crate_temperature, ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_GetFanSpeed1:
            WriteEP0 ( (BYTE *)&g_fan_speed[ 0 ], ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_GetFanSpeed2:
            WriteEP0 ( (BYTE *)&g_fan_speed[ 1 ], ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_GetFanSpeed3:
            WriteEP0 ( (BYTE *)&g_fan_speed[ 2 ], ControlRequest->ReturnBytes );
            break;
        case ltr030_CC_GetFanSpeed4:
            WriteEP0 ( (BYTE *)&g_fan_speed[ 3 ], ControlRequest->ReturnBytes );
            break;
#endif
        default:        
            LTR030DEBUG_PRINT( ("Unknown Command\n") );
            return 0;        
        } /*switch (ControlRequest->CommandCode)*/
    
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ReleaseCommandMutex
    (
    void
    )
    {	    
    enable_interrupt( f_CommandInterruptLatch[ --f_CommandInterruptLatchDeepCounter ] );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int ProcessCommand
    (
    void
    )
    {
	t_ControlRequest ControlRequest = {0, 0, 0};
    	
    if (!f_GetCommandRequest( &ControlRequest, NULL, 0 ))
        return 0;
            
    if (!f_GetCommandRequest( NULL, (BYTE *)RequestData, ControlRequest.OveralLength -
            sizeof(t_ControlRequest))
       )
        {       
        return 0;
        }
        
    GlobalStop = 0;	    	    
    
    if (ControlRequest.DeviceRequested == USB_Device)
        {
        return f_ProcessCommandUSB( &ControlRequest );
        }
    else if (ControlRequest.DeviceRequested == Ether_Device)
        {
    	return f_ProcessCommandEther( &ControlRequest );
        }
    
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
