/*
 *         .
 *         g_datahook_cfg ( . 
 *  datahhok.h). ,    ,   datahook.  
 * datahook       .    
 *          .
 *          .
 */

#include "GlobDefs.h" 
#include "ethernet.h"
#include "datahook.h"


#if DATA_HOOK_EN

/*================================================================================================*/
#define SLOT_NUM(cmd) (((cmd) >> 8) & 0x0F)
/*================================================================================================*/

/*================================================================================================*/
static void f_send_host_data(int data_size);
/*================================================================================================*/

/*================================================================================================*/
static t_uint16 f_host_databuf[ MAX_DATA_SIZE*2 ];
static t_uint16 f_modules_databuf[ N_CRATE_MODULES ][ MAX_DATA_SIZE ];
extern const struct datahook_cfg g_datahook_cfg[ N_CRATE_MODULES ];
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void datahook
    (
    unsigned long n_data
    )
    {
    /*
     *     .
     * 
     *      ,        ( )
     *       .
     */
    static int databuf_idx[ N_CRATE_MODULES ];
    int i;
    long recv_data_idx;
    
    recv_data_idx = ltr030Conf.AlteraDev.CurRecvBufPtrProcesed;
    /*     */
    while (n_data > 0)
        {
        const t_uint16 ct_cmd_flag = 0x8000U;
        if ((PPI_Recv_Data[ recv_data_idx ] & ct_cmd_flag) == 0)
            {
            /*   */
            int slot_num = SLOT_NUM( PPI_Recv_Data[ recv_data_idx ] );
            /*     */
            if (++recv_data_idx >= PPI_Recv_Data_Size_Word)
                recv_data_idx = 0;
            
            if (g_datahook_cfg[ slot_num ].module != e_Module_None)
                {
                int data_size;
                
                f_modules_databuf[ slot_num ][ databuf_idx[ slot_num ] ] =
                    PPI_Recv_Data[ recv_data_idx ];
                data_size = (g_datahook_cfg[ slot_num ].data_size <= MAX_DATA_SIZE) ?
                    g_datahook_cfg[ slot_num ].data_size : MAX_DATA_SIZE;
                if (++databuf_idx[ slot_num ] >= data_size)
                    {
                    if (g_datahook_cfg[ slot_num ].cb_hook != NULL)
                        {
                        g_datahook_cfg[ slot_num ].cb_hook( g_datahook_cfg[ slot_num ].module,
                            data_size, f_modules_databuf[ slot_num ] );
                        }
                    databuf_idx[ slot_num ] = 0;
                    }
                }
            }
         else
            {
            /*     */
            if (++recv_data_idx >= PPI_Recv_Data_Size_Word)
                recv_data_idx = 0;
            }
        /*     */
        if (++recv_data_idx >= PPI_Recv_Data_Size_Word)
            recv_data_idx = 0;
        n_data -= 2;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_send_host_data
    (
    int data_size                           /* () -    2-  */
    )
    {
    /*
     *   .
     * 
     *        USB  Ethernet     .
     */
    
    if (g_device_mode == e_DeviceMode_ETH)
        {
        putdata_ethernet( (t_uint8 *)f_host_databuf, data_size*2 );
        }
    else
        {
        put_ep2data( f_host_databuf, data_size );
        while (ISP_DMA_Running())
            {
            continue;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void put_host_data
    (
    const t_uint16 *data,                   /* () -  */
    int data_size                           /* () -    16- 
                                             *        [1..MAX_DATA_SIZE]
                                             */
    )
    {
    /*
     *   .
     * 
     *    2-   4-     - 
     *     .
     */
    const t_uint16 cmd_word = 0x4000;
    int i_data;
    t_uint16 *pbuf = f_host_databuf;
    
    if (data_size <= 0)
        return;
    
    if (data_size >= MAX_DATA_SIZE)
        data_size = MAX_DATA_SIZE;
        
    for (i_data = data_size; (i_data > 0); i_data--)
        {
        *pbuf++ = cmd_word;
        *pbuf++ = *data++;
        }
    
    f_send_host_data( data_size*2 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void put_module_data
    (
    int module_num,                         /* () -   [0..N_CRATE_MODULES) */
    const t_uint16 *data,                   /* () -  */
    int data_size                           /* () -    16-  */
    )
    {
    /*
     *   .
     * 
     *     2-   4-       SPORT.
     */
    unsigned buf_block_size;
    const t_uint16 cmd_word = (module_num & 0x0F) << 8;
    int i_data;
    WORD *p_buf;
    
    while (ltr030Conf.AlteraDev.SndBufAvail < data_size * 2)
        {
        continue;
        }
    
    if (data_size > MAX_DATA_SIZE)
        data_size = MAX_DATA_SIZE;
    
    p_buf = &SPORT_Tran_Data[ ltr030Conf.AlteraDev.CurSndBufferPtr ];
    for (i_data = data_size; (i_data > 0); i_data--)
        {
        *p_buf = cmd_word;
        if (++p_buf >= &SPORT_Tran_Data[ SPORT_Transmit_Data_Size_Word ])
            p_buf = SPORT_Tran_Data;
        *p_buf = *data++;
        if (++p_buf >= &SPORT_Tran_Data[ SPORT_Transmit_Data_Size_Word ])
            p_buf = SPORT_Tran_Data;
        }
    /*     2 , ..      */
    data_size *= 2;
    portDISABLE_INTERRUPTS();
    if (SPORT_Transmit_Data_Size_Word - ltr030Conf.AlteraDev.CurSndBufferPtr > data_size)
        {
        ltr030Conf.AlteraDev.CurSndBufferPtr += data_size;
        }
    else
        {
        ltr030Conf.AlteraDev.CurSndBufferPtr = data_size - (SPORT_Transmit_Data_Size_Word -
            ltr030Conf.AlteraDev.CurSndBufferPtr);
        }
    ltr030Conf.AlteraDev.SndBufAvail -= data_size * 2;
    buf_block_size = (g_device_mode == e_DeviceMode_USB) ? ltr030Conf.USBDev.MaxBulkSize :
        ETH_DATAPART;
    if (ltr030Conf.AlteraDev.SndBufAvail < buf_block_size)
        ltr030Conf.USBDev.ep2rxEn = 0;
    portENABLE_INTERRUPTS();
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /*#if DATA_HOOK_EN*/
