#include "GlobDefs.h"

#include "ethernet.h"
#include "datahook.h"


int state;


/*=================================================================================================*/
#define LTRETH_CMD_PORT (11113)
#define LTRETH_DATA_PORT (11110)
#define ETH_COMMAND_LEN (16)
#define ETH_HEADER_LEN (20)

#define ETHREQIDX_SIGNATURE (0)
#define ETHREQIDX_CMDCODE   (1)
#define ETHREQIDX_CMDPARAM  (2)
#define ETHREQIDX_DATALEN   (3)
#define ETHREQIDX_REPLYLEN  (4)
#define ETHREQIDX_STARTDATA (5)


#define ETHCMD_TASK_PRIO (5)
#define ETHERDATARECV_TASK_PRIO (8)

#define ETHERR_PACKET_LEN (1)

#define ETHRETCODE_OK (0)
#define ETHRETCODE_ERROR (1)

#define ETHCMD_NOP (0)
#define ETHCMD_GETNAME (0x00000001)
#define ETHCMD_GETARRAY (0x00000002)
#define ETHCMD_PUTARRAY (0x00000003)
#define ETHCMD_INIT (0x80000000)
/*=================================================================================================*/

/*=================================================================================================*/
portTASK_FUNCTION_PROTO( ethernet_datarecv_task, pvParameters );
static int f_eth_read(int sockfd, int len, t_uint8 *data);
static void f_eth_send_command(int sockfd, t_uint32 cmd_code, int len, const t_uint8 *data);
static void f_eth_send_error(t_uint32 errCode);
static int f_eth_write(int sockfd, int len, const t_uint8 *data);
static void f_get_array_cmd(int sockfd, int mem_type, t_uint32 mem_addr, unsigned long len);
static void f_process_ethcommand(int sockfd);
static void f_put_array_cmd(int sockfd, int mem_type, t_uint32 mem_addr, unsigned long len);
static int f_recv_server_data(void);
static int f_send_server_data(void);
/*=================================================================================================*/

/*=================================================================================================*/
static int f_cmdthread_run = 0;
static int f_dataconn = 0;
static t_uint32 f_ethcommand[ 128 ];
static const t_uint32 f_ethcmd_signature = 0x314C5443;
static xSemaphoreHandle f_h_cmd_smphr = NULL;
static int f_h_cmd_sock = -1;
static int f_h_data_sock = -1;
static int f_init_require = 0;
static unsigned long f_n_data_ready;
static int f_n_datacycle = 0;
static int f_recv_shift = 0;
struct ip_addr g_gatewayaddr;
struct ip_addr g_devipaddr;
struct ip_addr g_netmask;
//const t_uint8 g_gatewayaddr[ 4 ] = {192, 168, 12, 1};
//const t_uint8 g_ipaddr[ 4 ] = {192, 168, 12, 247};
//const t_uint8 g_netmask[ 4 ] = {255, 255, 255, 0};
/*=================================================================================================*/

/*=================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
portTASK_FUNCTION( ethcmd_task, pvParameters )
    {
    while (1)
        {
        while (xSemaphoreTake( f_h_cmd_smphr, portMAX_DELAY ) != pdTRUE)
            {
            continue;
            }
        while (1)
            {
            int msg_len;
            int nBytes;
            nBytes = f_eth_read( f_h_cmd_sock, ETH_HEADER_LEN, (t_uint8 *)f_ethcommand );
        
            if (   (nBytes != ETH_HEADER_LEN)
                || (f_ethcmd_signature != f_ethcommand[ ETHREQIDX_SIGNATURE ])
               )
                {
                break;
                }               
            
            msg_len = f_ethcommand[ ETHREQIDX_DATALEN ];
            if (msg_len > 0)
                {
                if (msg_len + ETH_HEADER_LEN >= sizeof f_ethcommand)
                    {
                    f_eth_send_error( ETHERR_PACKET_LEN );
                    break;
                    }
                nBytes = f_eth_read( f_h_cmd_sock, msg_len, (t_uint8 *)f_ethcommand + ETH_HEADER_LEN );
                if (nBytes != msg_len)
                    break;
                }
    
            f_process_ethcommand( f_h_cmd_sock );
            }
        if (f_dataconn)
            f_init_require = 1;
        close( f_h_cmd_sock );
        f_cmdthread_run = 0;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
portTASK_FUNCTION( ethernet_task, pvParameters )
    {
    extern err_t ethernetif_init( struct netif *netif );
    static struct netif EMAC_if;
    int res_s;
    struct sockaddr_in sa;
    int socket_fd;
    struct ip_addr xGateway;
    struct ip_addr xIpAddr;
    struct ip_addr xNetMast;
    
    
    tcpip_init( NULL, NULL );
    /* Create and configure the EMAC interface. */
//	IP4_ADDR( &xIpAddr, g_ipaddr[ 0 ], g_ipaddr[ 1 ], g_ipaddr[ 2 ], g_ipaddr[ 3 ] );
//	IP4_ADDR( &xNetMast, g_netmask[ 0 ], g_netmask[ 1 ], g_netmask[ 2 ], g_netmask[ 3 ] );
//	IP4_ADDR( &xGateway, g_gatewayaddr[ 0 ], g_gatewayaddr[ 1 ], g_gatewayaddr[ 2 ],
//	    g_gatewayaddr[ 3 ] );
	//netif_add( &EMAC_if, &xIpAddr, &xNetMast, &xGateway, NULL, ethernetif_init, tcpip_input );
	
	netif_add( &EMAC_if, &g_devipaddr, &g_netmask, &g_gatewayaddr, NULL, ethernetif_init,
	    tcpip_input );
	    
//	while (1)
//        ;
	
	/* make it the default interface */
    netif_set_default(&EMAC_if);

	/* bring it up */
    netif_set_up(&EMAC_if);
    
    printf( "TCP ready\n" );
    g_mode_ELED = e_LED_Green;
    
    vSemaphoreCreateBinary( f_h_cmd_smphr );
    
    sys_thread_new( "ETHDR", ethernet_datarecv_task, NULL, 2048, ETHERDATARECV_TASK_PRIO );
    sys_thread_new( "ETHCMD", ethcmd_task, NULL, 2048, ETHCMD_TASK_PRIO );
    
    res_s = -1;
    while (res_s != 0)
        {
        while ((socket_fd = socket( PF_INET, SOCK_STREAM, 0 )) < 0)
            {
            continue;
            }
    	memset(&sa, 0, sizeof(struct sockaddr_in));
        sa.sin_family = AF_INET;
        sa.sin_addr.s_addr = htonl( INADDR_ANY );
        sa.sin_port = htons( LTRETH_CMD_PORT );
    
        res_s = bind( socket_fd, (struct sockaddr *)&sa, sizeof(sa) );
        if (res_s != 0)
            {
            close( socket_fd );
            continue;
            }

        res_s = listen( socket_fd, 0 );
        if (res_s != 0)
            {
            close( socket_fd );
            continue;
            }
        }
    
    while (1)
        {
        int accept_fd;
        u32_t addr_size;
        struct sockaddr_in isa;
        
        addr_size = sizeof(isa);
        accept_fd = accept( socket_fd, (struct sockaddr*)&isa, &addr_size );
        if (!f_cmdthread_run)
            {
            int sockopt = 30000;
            setsockopt( accept_fd, SOL_SOCKET, SO_RCVTIMEO, &sockopt, sizeof sockopt );
            f_h_cmd_sock = accept_fd;
            f_cmdthread_run = 1;
            
            xSemaphoreGive( f_h_cmd_smphr );
            }
        else
            {
            close( accept_fd );
            }
        } /*while (1)*/
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
portTASK_FUNCTION( ethernet_datarecv_task, pvParameters )
    {
    /*
     *      / .
     */
    u32_t addr_size;
    int h_socket;
    struct sockaddr_in sa;
    
    h_socket = socket( PF_INET, SOCK_STREAM, 0 );
    
    memset( &sa, 0, sizeof(struct sockaddr_in) );
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = htonl( INADDR_ANY );
    sa.sin_port = htons( LTRETH_DATA_PORT );
    
    bind( h_socket, (struct sockaddr *)&sa, sizeof sa );
    listen( h_socket, 5 );
    
    while (1)
        {
        struct sockaddr_in isa;
        u32_t nonblock = 1;
        
        f_n_datacycle = 0;
        ltr030Conf.AlteraDev.NewModulesAppearence = true;
                
        addr_size = sizeof isa;
        
        state = 1;
        
        f_h_data_sock = accept( h_socket, (struct sockaddr *)&isa, &addr_size );
        ioctlsocket( f_h_data_sock, FIONBIO, &nonblock );
        
        state = 2;
        
        f_recv_shift = 0;
        
        f_dataconn = 1;
        
        while (1)
            {
#if DATA_HOOK_EN
            unsigned long buf_remaind;
#endif
            int n_ready_sock;
            fd_set rset;
            struct timeval t_out;
            fd_set wset;
            
            
/*            int n;
            
            
            n = recv( h_data_sock, data, 8192, 0 );
            if (n <= 0)
                break;
            send( h_data_sock, data, n, 0 );
            continue;*/
            
            /*        */
            FD_ZERO( &rset );
            FD_ZERO( &wset );
            f_n_data_ready = GetNumPPIDataReady() & 0xFFFFFFFE;
            
#if DATA_HOOK_EN
            buf_remaind = PPI_Recv_Data_Size_Word - ltr030Conf.AlteraDev.CurRecvBufPtrProcesed;
            buf_remaind &= 0xFFFFFFFE;
            if (f_n_data_ready > buf_remaind)
                f_n_data_ready = buf_remaind;
            if (f_n_data_ready > 32764 / 2)
                f_n_data_ready = 32764 / 2;
#endif
            
            
            FD_SET( f_h_data_sock, &rset );
            t_out.tv_sec = 0;
            t_out.tv_usec = 2000;
            
            state = 3;
            
            if ((f_n_datacycle < 1) || (f_n_data_ready > 0))
                {
                FD_SET( f_h_data_sock, &wset );
                n_ready_sock = select( f_h_data_sock+1, &rset, &wset, NULL, &t_out );
                }
            else
                {
                n_ready_sock = select( f_h_data_sock+1, &rset, NULL, NULL, &t_out );
                }
                
            state = 4;
                
            if (n_ready_sock < 0)
                break;
            
            state = 5;
                
            if (n_ready_sock > 0)
                {
                if (FD_ISSET( f_h_data_sock, &wset ))
                    {
#if DATA_HOOK_EN
                    if (f_n_data_ready > 0)
                        datahook( f_n_data_ready );
#endif

                    state = 6;
                    
                    /*    */
                    if (f_send_server_data() != 0)
                        break;
                        
                    state = 7;
                    }
                if (FD_ISSET( f_h_data_sock, &rset ))
                    {
                    /*     */
                    
                    state = 8;
                    if (f_recv_server_data() != 0)
                        break;
                        
                    state = 9;
                    }
                }
                
                        
            if (AlteraSPORTReady())
                {
                int buf_empty;
                buf_empty = (ltr030Conf.AlteraDev.SndBufAvail / 2 >= SPORT_Transmit_Data_Size_Word);
                if (!buf_empty)
                    InitiateSPORTTransfer();
                }
            
            if (f_init_require)
                {
                f_init_require = 0;
                break;
                }
                
            state = 10;
            } /*while (1)*/
            
        f_dataconn = 0;
            
        close( f_h_data_sock );
        f_h_data_sock = -1;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_eth_send_command
    (
    int sockfd,                             /* () -   */
    t_uint32 cmd_code,                      /* () -   */
    int len,                                /* () -    */
    const t_uint8 *data                     /* () -   */
    )
    {
    /*
     *    ethernet.
     */
    t_uint32 header[ 3 ];
    header[ 0 ] = f_ethcmd_signature;
    header[ 1 ] = cmd_code;
    header[ 2 ] = len;
    
    f_eth_write( sockfd, sizeof header, (t_uint8 *)header );
    if ((len > 0) && (data != NULL))
        f_eth_write( sockfd, len, data );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_eth_send_error
    (
    t_uint32 errCode
    )
    {
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_eth_read
    (
    int sockfd,                             /* () -   */
    int len,                                /* () -    */
    t_uint8 *data                           /* () -   */
    )
    {
    /*
     *      ethernet.
     * 
     *    recv  API     ,  . 
     *             
     *    .
     *  
     *   < 0 -   ;
     *   0   -    ;
     *   > 0 -   .
     */
    int size = 0;
    int ret_val = 0;
    while (size < len)
        {
        int n = recv( sockfd, data + size, len - size, 0 );
        if (n <= 0)
            {
            ret_val = (n < 0) ? n : size;
            break;
            }
        
        size += n;
        ret_val = size;
        }
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_eth_write
    (
    int sockfd,                             /* () -   */
    int len,                                /* () -    */
    const t_uint8 *data                     /* () -   */
    )
    {
    /*
     *      ethernet.
     * 
     *    send  API     ,  .  
     *              .
     *  
     *   <= 0 -  ;
     *   > 0 -   .
     */
    int size = 0;
    int ret_val = 0;
    while (size < len)
        {
        int n = send( sockfd, data + size, len - size, 0 );
        if (n <= 0)
            {
            ret_val = n;
            break;
            }
        
        size += n;
        ret_val = size;
        }
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_get_array_cmd
    (
    int sockfd,                             /* () -   */
    int mem_type,                           /* () -  ,     */
    t_uint32 mem_addr,                      /* () - ,     */
    unsigned long len                       /* () -      */
    )
    {
    /*
     *   .
     * 
     *           AVR,     
     *   Blackfin.
     */
    int addr_ok = 0;
    t_uint8 *ptr;
    t_uint8 tx_data[ 1 ];
    
    switch (mem_type)
        {
        case SEL_AVR_PM:
            if ((0x2FF0 <= mem_addr) && (mem_addr < 0x3FF0))
                {
                GetSystemStatusToStatusStruct();
                mem_addr -= 0x2FF0;
                ptr = (t_uint8 *)&Descr + mem_addr;
                }
            else
                {
                ptr = (t_uint8 *)mem_addr;
                }
            addr_ok = 1;
            break;
        case SEL_AVR_DM:
            if ((0x8000 <= mem_addr) && (mem_addr < 0x8020))
                {
                GetSystemRegistersToStatusStruct();
                mem_addr -= 0x8000U;
                ptr = (t_uint8 *)&ltr030Conf.LTR010_Emulation + mem_addr;
                }
            else
                {
                ptr = (t_uint8 *)mem_addr;
                }
            addr_ok = 1;
            break;
        case SEL_DSP:
            if (mem_addr == 0x8000)
                {
                if (g_board_version == 0)
                    {
                    tx_data[ 0 ] = (*pPORTFIO >> 1) & 0x01 | (*pPORTGIO >> 12) & 0x02;
                    }
                else
                    {
                    tx_data[ 0 ] = *pPORTFIO & 0x01 | (*pPORTGIO >> 12) & 0x02;
                    }
                ptr = tx_data;
                addr_ok = 1;
                }
            else if (mem_addr == 0x8001)
                {
                tx_data[ 0 ] = g_board_version;
                ptr = tx_data;
                addr_ok = 1;
                }
            break;
        default:
            ptr = (t_uint8 *)mem_addr;
            addr_ok = 1;
        }
        
        if (addr_ok)
            {
            f_eth_send_command( sockfd, ETHRETCODE_OK, len, ptr );
            }
        else
            {
            f_eth_send_command( sockfd, ETHRETCODE_ERROR, 0, NULL );
            }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_process_ethcommand
    (
    int sockfd                              /* () -   */
    )
    {
    /*    Ethernet . */
    switch (f_ethcommand[ ETHREQIDX_CMDCODE ])
        {
        case ETHCMD_INIT:
            if (f_dataconn)
                f_init_require = 1;
            f_eth_send_command( sockfd, ETHRETCODE_OK, 510, (t_uint8 *)g_altera_info );
            break;
        case ETHCMD_NOP:
            f_eth_send_command( sockfd, ETHRETCODE_OK, 0, NULL );
            break;
        case ETHCMD_GETNAME:
            f_eth_send_command( sockfd, ETHRETCODE_OK, DESC_MODULE_NAME_SIZE,
                (t_uint8 *)Descr.Descr.Name );
            break;
        case ETHCMD_GETARRAY:
            f_get_array_cmd( sockfd, f_ethcommand[ ETHREQIDX_CMDPARAM ] >> 24,
                f_ethcommand[ ETHREQIDX_CMDPARAM ] & 0xFFFFFF, f_ethcommand[ ETHREQIDX_REPLYLEN ] );
            break;
        case ETHCMD_PUTARRAY:
            f_put_array_cmd( sockfd, f_ethcommand[ ETHREQIDX_CMDPARAM ] >> 24,
                f_ethcommand[ ETHREQIDX_CMDPARAM ] & 0xFFFFFF, f_ethcommand[ ETHREQIDX_DATALEN ] );
            break;
        default:
            break;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_put_array_cmd
    (
    int sockfd,                             /* () -   */
    int mem_type,                           /* () -  ,     */
    t_uint32 mem_addr,                      /* () - ,    
                                             *         */
    unsigned long len                       /* () -      */
    )
    {
    /*
     *   .
     * 
     *           AVR,     
     *   Blackfin.
     */
    int addr_ok = 0;
    int need_copy = 0;
    t_uint8 *ptr;
    t_uint8 tx_data[ 1 ];
    
    switch (mem_type)
        {
        case SEL_AVR_PM:
            if ((0x2FF0 <= mem_addr) && (mem_addr < 0x3FF0))
                {
                GetSystemStatusToStatusStruct();
                mem_addr -= 0x2FF0;
                ptr = (t_uint8 *)&Descr + mem_addr;
                }
            else
                {
                ptr = (t_uint8 *)mem_addr;
                }
            need_copy = 1;
            addr_ok = 1;
            break;
        case SEL_AVR_DM:
            if ((0x8000 <= mem_addr) && (mem_addr < 0x8020))
                {
                GetSystemRegistersToStatusStruct();
                mem_addr -= 0x8000U;
                ptr = (t_uint8 *)&ltr030Conf.LTR010_Emulation + mem_addr;
                (void)memcpy( ptr, &f_ethcommand[ ETHREQIDX_STARTDATA ], len );
                SetSystemRegistersFromStatusStruct();
                }
            else
                {
                ptr = (t_uint8 *)mem_addr;
                need_copy = 1;
                }
            addr_ok = 1;
            break;
        case SEL_FPGA_DATA:
            if (mem_addr + len / 2 <= ALTERA_REGS_NUMBER)
                {
                int n = len / 2;
                t_uint16 *p = (t_uint16 *)&f_ethcommand[ ETHREQIDX_STARTDATA ];
                struct altera_register *preg = &g_altera_reg[ mem_addr ];
                while (n-- > 0)
                    {
                    preg->value = *p++;
                    preg->is_updated = 1;
                    preg++;
                    }
                addr_ok = 1;
                }
            break;
        default:
            ptr = (t_uint8 *)mem_addr;
            need_copy = 1;
        }
        
        if (addr_ok)
            {
            if (need_copy)
                {
                (void)memcpy( ptr, &f_ethcommand[ ETHREQIDX_STARTDATA ], len );
                }
            f_eth_send_command( sockfd, ETHRETCODE_OK, 0, NULL );
            }
        else
            {
            f_eth_send_command( sockfd, ETHRETCODE_ERROR, 0, NULL );
            }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_recv_server_data
    (
    void
    )
    {
    /*
     *    .
     * 
     *            .
     */
    /*         */
    static t_uint8 recv_buf[ 4 ];
    int recv_status = 0;
    
    if (ltr030Conf.USBDev.ep2rxEn)
        {
        t_uint32 buf_remaind;
        int data_size;
        int i_data;
        t_uint8 *p_recv_data;
        t_uint8 *p_rx_data = NULL;
        int recv_size;
        
        /* ,      ( ) */
        buf_remaind = (SPORT_Transmit_Data_Size_Word - ltr030Conf.AlteraDev.CurSndBufferPtr) * 2;
        /*   2       */
        recv_size = (buf_remaind >= ETH_DATAPART * 2) ? ETH_DATAPART * 2 : buf_remaind;
        p_recv_data = p_rx_data =
            (t_uint8 *)&SPORT_Tran_Data[ ltr030Conf.AlteraDev.CurSndBufferPtr ];
        for (i_data = 0; (i_data < f_recv_shift); i_data++)
            {
            *p_rx_data++ = recv_buf[ i_data ];
            }
        
        data_size = recv( f_h_data_sock, p_rx_data, recv_size-f_recv_shift, 0 );
        if (data_size == 0)
            recv_status = -1;
            
        if (recv_status == 0)
            {
            if (data_size < 0)
                {
                int err;
                socklen_t sz = sizeof(int);
                getsockopt( f_h_data_sock, SOL_SOCKET, SO_ERROR, &err, &sz );
                if (err != EWOULDBLOCK)
                    recv_status = -2;
                }
            }
        if ((recv_status == 0) && (data_size > 0))
            {
            int sum_bytes = data_size + f_recv_shift;
            int rem4_bytes = sum_bytes % 4;
            int sum4_bytes = sum_bytes - rem4_bytes;
            portDISABLE_INTERRUPTS();
            ltr030Conf.AlteraDev.CurSndBufferPtr += sum4_bytes / 2;
            if (ltr030Conf.AlteraDev.CurSndBufferPtr >= SPORT_Transmit_Data_Size_Word)
                ltr030Conf.AlteraDev.CurSndBufferPtr = 0;
                
            ltr030Conf.AlteraDev.SndBufAvail -= sum4_bytes;
            if (ltr030Conf.AlteraDev.SndBufAvail < ETH_DATAPART)
                ltr030Conf.USBDev.ep2rxEn = 0;
            portENABLE_INTERRUPTS();
            for (i_data = 0; (i_data < rem4_bytes); i_data++)
                {
                recv_buf[ i_data ] = p_recv_data[ sum4_bytes+i_data ];
                }
            f_recv_shift = rem4_bytes;
            }
        }
        
        return recv_status;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_send_server_data
    (
    void
    )
    {
    /*     
     *
     */
    unsigned long num_data_ready;
    int send_status = 0;
    
     
    state = 11;
    
    if (f_n_datacycle == 0)
        {
        f_eth_write( f_h_data_sock, 8, (t_uint8 *)CrateSygnalDataNewModule );
        f_n_datacycle = 2;
        g_power_on = 0;
        }
        
    
    state = 12;
        
#if DATA_HOOK_EN
    num_data_ready = f_n_data_ready;
#else
    num_data_ready = GetNumPPIDataReady() & 0xFFFFFFFE;
#endif
    if (num_data_ready > 0)
        {
#if !DATA_HOOK_EN
        unsigned long buf_remaind;
        unsigned long n;
#endif
        int n_data;
        int n_send_data;

#if DATA_HOOK_EN
        n_data = num_data_ready;
#else        
        buf_remaind = PPI_Recv_Data_Size_Word - ltr030Conf.AlteraDev.CurRecvBufPtrProcesed;
        buf_remaind &= 0xFFFFFFFE;
        n = min( buf_remaind, num_data_ready );
        n_data = min( n, 32764 / 2 );
#endif
        
        state = 13;

        n_send_data = f_eth_write( f_h_data_sock, n_data * 2,
            (t_uint8 *)&PPI_Recv_Data[ ltr030Conf.AlteraDev.CurRecvBufPtrProcesed ] );
            
        state = 14;
        if (n_send_data > 0)
            {
            portDISABLE_INTERRUPTS();
            CommitDataFromPPIBuffer( n_data );
            portENABLE_INTERRUPTS();
            }
        else
            {
            send_status = -1;
            }
        }
        
    state = 15;
    
    return send_status;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void putdata_ethernet
    (
    const t_uint8 *data,                    /* () -  */
    int data_size                           /* () -     */
    )
    {
    /*
     *    Ethernet.
     * 
     *         (  ).
     */
    if (f_h_data_sock >= 0)
        f_eth_write( f_h_data_sock, data_size, data );
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
