#include "GlobDefs.h"

#if defined(LTR030_ENDPOINT)

#include "fanning.h"
#include "hardware/hardware.h"


/*================================================================================================*/
#define TEMP_SPI_CS (FLG6)
/*================================================================================================*/

/*================================================================================================*/
static void f_control_thermo(void);
static void f_dec_fanspeed(int n);
static t_uint32 f_fanspeed(void);
static void f_get_fanspeed(void);
static void f_get_temperature(int *t);
static void f_inc_fanspeed(int n);
static void f_start_fanspeed_measuring(void);
static void f_stop_fanspeed_measuring(void);
/*================================================================================================*/

/*================================================================================================*/
static unsigned long f_fan_avgspeed = 0;
static int f_fannum_measured = 0;
t_int16 g_crate_temperature;
t_uint32 g_fan_speed[ FAN_NUM ];
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
static void f_control_thermo
    (
    void
    )
    {
    const int ct_min_fanspeed = 100;
    const int ct_ref_temperature = 40;
    const int ct_temp_nullzone = 2;
    int t;
    
    f_get_temperature( &t );
    g_crate_temperature = (t_int16)t;
    if (t > ct_ref_temperature + ct_temp_nullzone)
        {
        f_inc_fanspeed(1);
        }
    else if ((t < ct_ref_temperature - ct_temp_nullzone) && (f_fan_avgspeed > ct_min_fanspeed))
        {
        f_dec_fanspeed(1);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_dec_fanspeed
    (
    int n                                   /* () -    
                                             *   
                                             */
    )
    {
    /*
     *    .
     * 
     *            
     *     .
     */
    const t_uint16 ct_cmd_dec = 0x20;

    while (n-- > 0)
        {
        AlteraWriteCommand( ALTERAREG_FANCTRL, ct_cmd_dec | f_fannum_measured );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static t_uint32 f_fanspeed
    (
    void
    )
    {
    /*
     *    .
     * 
     *      TIMER2,      
     *   .
     *  
     *       .       ,
     *       .
     */
    t_uint32 v;
    v = *pTIMER2_COUNTER;
    return v;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_get_fanspeed
    (
    void
    )
    {
    /*
     *    .
     * 
     *         TIMER2,    
     *     .
     *           TIMER2 
     *        .
     *  
     *          1   .
     */
    static unsigned long fan_sumspeed;
    static int is_measuring = 0;
    static int n_fan = 0;
    
    if (!is_measuring)
        {
        f_start_fanspeed_measuring();
        is_measuring = 1;
        }
    else
        {
        g_fan_speed[ f_fannum_measured ] = f_fanspeed();
        f_stop_fanspeed_measuring();
        is_measuring = 0;
        
        if (f_fannum_measured == 0)
            {
            fan_sumspeed = 0;
            n_fan = 0;
            }
        if (g_fan_speed[ f_fannum_measured ] > 0)
            {
            fan_sumspeed += g_fan_speed[ f_fannum_measured ];
            n_fan++;
            }
            
        if (++f_fannum_measured >= FAN_NUM)
            {
            f_fannum_measured = 0;
            if (n_fan > 0) f_fan_avgspeed = fan_sumspeed / n_fan;
            else           f_fan_avgspeed = 0;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_get_temperature
    (
    int *t                                  /* () -     */
    )
    {
    const t_uint32 ct_spi_clk = 15000000;
    
    DumpSPISettingsStore( SPI_DEVICE_THERMO );
    
    DisableSPI;
    PF_PINS_SET( PF4|PF5|PF6|PF10 );
    ConfigureSPIBaud( ct_spi_clk );
    ConfigureSPI( START_TRANS_WITH_WRITE, true, false, false, true,	true, false, true, true, true,
        false);
    EnableSPI;
    Select_CS_Slave( TEMP_SPI_CS );
    ssync();
    *t = ExchangeSPI( 0 );
    Select_CS_Slave( 0 );
    
    UnDumpSPISettingsStore( SPI_DEVICE_THERMO );
    
    if (*t >= 8192)
        *t -= 16384;
    *t /= 32;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_inc_fanspeed
    (
    int n                                   /* () -     */
    )
    {
    /*
     *    .
     * 
     *            
     *     .
     */
    const t_uint16 ct_cmd_inc = 0x10;

    while (n-- > 0)
        {
        AlteraWriteCommand( ALTERAREG_FANCTRL, ct_cmd_inc | f_fannum_measured );
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_start_fanspeed_measuring
    (
    void
    )
    {
    /*
     *     .
     */
    AlteraWriteCommand( ALTERAREG_FANCTRL, f_fannum_measured );
    *pTIMER2_CONFIG = EMU_RUN | EXT_CLK;
    *pTIMER2_PERIOD = 0xFFFFFFFFU;
    *pTIMER_ENABLE = TIMEN2;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_stop_fanspeed_measuring
    (
    void
    )
    {
    /*
     *     .
     * 
     *     TIMER2.
     */
    *pTIMER_DISABLE = TIMDIS2;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void process_fanning
	(
	void
	)
	{
	f_control_thermo();
	f_get_fanspeed();
	}
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /*#if defined(LTR030_ENDPOINT)*/
