/*
 *   I2C.
 */
 
#include <cdefBF537.h>
#include <ccblkfn.h>
#include "i2c.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void init_i2c
    (
    void
    )
    {
    /*
     *   I2C.
     */
    *pTWI_CONTROL = 14 | TWI_ENA;
    *pTWI_CLKDIV = CLKHI( 30 ) | CLKLOW( 65 );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void transmitreceive_i2c
    (
    unsigned char addr,                     /* () -   */
    unsigned char tx_len,                   /* () -    */
    const unsigned char *tx_data,           /* () -   */
    unsigned char rx_len,                   /* () -    */
    unsigned char *rx_data,                 /* () -   */
    enum I2CErr *err                        /* () -   */
    )
    {
    /*
     * -    I2C.
     * 
     *         (   )  .
     */
    int i_data;
    const unsigned char *pcdata;
     
    *err = I2CErr_Ok;
     
    *pTWI_MASTER_ADDR = addr;
    *pTWI_FIFO_CTL = 0;
    *pTWI_MASTER_CTL = ((tx_len & 0xFF) << 6) | MEN;
    
    pcdata = tx_data;
    for (i_data = tx_len; (i_data > 0); i_data--)
        {
        while (*pTWI_FIFO_STAT == XMT_FULL)
            {
            ssync();
            }
        *pTWI_XMT_DATA8 = *pcdata++;
        ssync();
        }
    
    while ((*pTWI_MASTER_CTL & DCNT) != 0)
        {
        ssync();
        }
        
    *pTWI_MASTER_CTL |= MDIR | RSTART;
    
    while ((*pTWI_INT_STAT & (MCOMP | MERR)) == 0)
        {
        ssync();
        }
        
    if (*pTWI_INT_STAT & MERR)
        *err = I2CErr_tx;
        
    *pTWI_INT_STAT = XMTSERV | MCOMP | MERR;
    
    
    if (*err == I2CErr_Ok)
        {
        unsigned char *pdata = rx_data;
        *pTWI_MASTER_CTL = ((rx_len & 0xFF) << 6) | MEN | MDIR;
        
        for (i_data = rx_len; (i_data > 0); i_data--)
        	{ 
    		while (*pTWI_FIFO_STAT == RCV_EMPTY)
    		    {
    			ssync();
    		    }

    		*pdata++ = *pTWI_RCV_DATA8;
    		ssync();
        	}
        	
        while ((*pTWI_INT_STAT & (MCOMP | MERR)) == 0)
            {
            ssync();
            }
        if (*pTWI_INT_STAT & MERR)
            *err = I2CErr_rx;
        *pTWI_INT_STAT = RCVSERV | MCOMP | MERR;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void transmit_i2c
    (
    unsigned char addr,                     /* () -   */
    unsigned char tx_len,                   /* () -    */
    const unsigned char *tx_data,           /* () -  */
    enum I2CErr *err                        /* () -   */
    )
    {
    /*
     *     I2C.
     */
    int i_data;
    const unsigned char *pcdata;
     
    *err = I2CErr_Ok;
     
    *pTWI_MASTER_ADDR = addr;
    *pTWI_FIFO_CTL = 0;
    *pTWI_MASTER_CTL = ((tx_len & 0xFF) << 6) | MEN;
    
    pcdata = tx_data;
    for (i_data = tx_len; (i_data > 0); i_data--)
        {
        while (*pTWI_FIFO_STAT == XMT_FULL)
            {
            ssync();
            }
        *pTWI_XMT_DATA8 = *pcdata++;
        ssync();
        }
    
    while ((*pTWI_MASTER_CTL & DCNT) != 0)
        {
        ssync();
        }
        
    *pTWI_MASTER_CTL |= RSTART;
    
    while ((*pTWI_INT_STAT & (MCOMP | MERR)) == 0)
        {
        ssync();
        }
        
    if (*pTWI_INT_STAT & MERR)
        *err = I2CErr_tx;
        
    *pTWI_INT_STAT = XMTSERV | MCOMP | MERR;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
