#include "GlobDefs.h"

#include "ethernet.h"
#include "datahook.h"
#include "lib/emac_bf/emac_bf.h"

#include "wdt.h"

#include "rtc.h"

#if defined(LTR030_ENDPOINT)
#include "fanning.h"
#include "interfaces/bf537_spi.h"
#endif

#include "ifuart.h"


/*================================================================================================*/
#define ETHER_TASK_PRIO (4)
#define MAIN_TASK_PRIO (9)

#define DEVMODE_USB (0)
#define DEVMODE_ETH (1)
/*================================================================================================*/

/*================================================================================================*/
static void f_read_device_config(void);
static void f_update_alteraregs(void);
portTASK_FUNCTION_PROTO( main_task, pvParameters );
/*================================================================================================*/

/*================================================================================================*/
segment ("sdram0_bank1") t_uint8 g_altera_info[ 510 ];
enum e_LED g_mode_ELED = e_LED_Red;
enum e_LED g_mode_ULED = e_LED_Orange;
static t_uint8 f_sd_buf[ 512 ];
enum e_DeviceMode g_device_mode = e_DeviceMode_USB;
unsigned char g_board_version = 0;
int g_ring_mode = 0;
int g_power_on = 1;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void clear_extwdt
    (
    void
    )
    {
    volatile int tmp;
    tmp = *((int *)0x20200000);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_read_device_config
    (
    void
    )
    {
    /*
     *     FLASH-.
     * 
     *        (     "
     *   "):
     *             
     *   0..1           
     *   2..5          IP  
     *   6..9          IP  
     *   10..13         
     */
#define CONFIGREC_SZ (20)
    t_uint8 config_rec[ CONFIGREC_SZ ];
    t_uint16 dev_mode;
    
    OpenATFlashInterface();
    
    ContiniousReadFlash( DEVICE_CONFIG_FLASH_START_ADDRESS, config_rec, sizeof config_rec );
    
    dev_mode = config_rec[ 0 ] & 0xFF | ((config_rec[ 1 ] & 0xFF) << 8);
    if (dev_mode == DEVMODE_ETH) g_device_mode = e_DeviceMode_ETH;
    else                         g_device_mode = e_DeviceMode_USB;
        
    g_devipaddr.addr = (u32_t)config_rec[ 2 ] & 0xFF | (((u32_t)config_rec[ 3 ] & 0xFF) << 8) |
        (((u32_t)config_rec[ 4 ] & 0xFF) << 16) | (((u32_t)config_rec[ 5 ] & 0xFF) << 24);
    g_gatewayaddr.addr = (u32_t)config_rec[ 6 ] & 0xFF | (((u32_t)config_rec[ 7 ] & 0xFF) << 8) |
        (((u32_t)config_rec[ 8 ] & 0xFF) << 16) | (((u32_t)config_rec[ 9 ] & 0xFF) << 24);
    g_netmask.addr = (u32_t)config_rec[ 10 ] & 0xFF | (((u32_t)config_rec[ 11 ] & 0xFF) << 8) |
        (((u32_t)config_rec[ 12 ] & 0xFF) << 16) | (((u32_t)config_rec[ 13 ] & 0xFF) << 24);
    (void)memcpy( g_emac_hwaddr, &config_rec[ 14 ], EMAC_HWADDR_SZ );
    
    CloseATFlashInterface();
    
#undef CONFIGREC_SZ
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_update_alteraregs
    (
    void
    )
    {
    int i_reg;
    
    for (i_reg = 0; (i_reg < ALTERA_REGS_NUMBER); i_reg++)
        {
        if (g_altera_reg[ i_reg ].is_updated)
            {
            portDISABLE_INTERRUPTS();
            AlteraWriteCommand( i_reg, g_altera_reg[ i_reg ].value );
            portENABLE_INTERRUPTS();
            if (i_reg == 0x01)
                g_ring_mode = ((g_altera_reg[ i_reg ].value & 0x01) != 0);
            g_altera_reg[ i_reg ].is_updated = 0;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int main
	(
	void
	)
    {
    DWORD BroachingSize;
    int i;
    const DWORD ct_serial_addr = 131072;
    static int uled_color = e_LED_Red;
#if defined(LTR030_ENDPOINT)
    const long ct_fanning_period = 100;
    static long fanning_timer = 1;
    hwInterface h_sd;
    esint8 res;
#endif

    
    g_power_on = 1;
    
    InitializeHardware();
    
    xTaskCreate( main_task, "MAIN", 2048, NULL, MAIN_TASK_PRIO, NULL );
	vTaskStartScheduler();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
portTASK_FUNCTION( main_task, pvParameters )
    {
    DWORD BroachingSize;
    int i;
    const DWORD ct_serial_addr = 131072;
    static int gled_color = e_LED_Red;
    static int uled_color = e_LED_Red;
    const int wdt_period = 10000;
#if defined(LTR030_ENDPOINT)
    const long ct_fanning_period = 100;
    static long fanning_timer = 1;
    hwInterface h_sd;
    esint8 res;
#endif

    InitializeSoftware();
    
    ltr030Conf.AlteraDev.NewModulesAppearence = false;
    
    ltr030Conf.AddNewCommandHandler = AddNewCommandRequest;
    SetDefaultUSBParameters( &ltr030Conf.USBDev );
	SetDefaultAlteraConfiguration( &ltr030Conf.AlteraDev );
    
	do
	    {
	    BroachingSize = LoadAlteraBroachingFromFlash( ProcessCommandQueue );
	    }
	while (BroachingSize == 0xFFFFFFFF);
	
	while (!BurnAltera( ProcessCommandQueue, BroachingSize ))
	    {
	    continue;
	    }
	
	//if (BroachingSize == 0xFFFFFFFF)
	//    AddErrorCounter( LTR030_ERROR_LOAD_ALTERA_BROACHING );
    //if (!BurnAltera( ProcessCommandQueue, BroachingSize ))
    //    AddErrorCounter( LTR030_ERROR_BURN_ALTERA_BROACHING );
        
    f_read_device_config();
         
    OpenAlteraDataInterface( &ltr030Conf.AlteraDev );
    
    portDISABLE_INTERRUPTS();
    AlteraWriteCommand( 0x00, 0 );
    /*   */
    AlteraWriteCommand( 0x00, 1 );
    AlteraWriteCommand( 0x01, 0x00 );
    g_altera_reg[ 0x01 ].value = 0;
    portENABLE_INTERRUPTS();
    
    /*    DMA1 (EMAC RX)  DMA2 (EMAC TX)  IVG8 */
    *pSIC_IAR2 &= 0xFFFFF00F;
    *pSIC_IAR2 |= 0x00000110;
    
    OpenATFlashInterface();        
    ContiniousReadFlash( ct_serial_addr, Descr.Descr.SerialNumber, 15 );
    Descr.Descr.SerialNumber[ 15 ] = '\0';
    CloseATFlashInterface();
    
    portDISABLE_INTERRUPTS();
    AlteraWriteCommand( 0x08, uled_color );
    portENABLE_INTERRUPTS();
    
    //TestAlteraInterface();        		    	    
    
	
    OpenISP1583( &ltr030Conf.USBDev );
    
    portDISABLE_INTERRUPTS();
    g_board_version = AlteraReadCommand( 0x07 );
    portENABLE_INTERRUPTS();
    

    if (g_board_version != 0)
        init_rtc();
    
    for (i = 0; (i < 510); i += 2)
        {
        portDISABLE_INTERRUPTS();
        g_altera_info[ i ] = AlteraReadCommand( 0x00 );
        portENABLE_INTERRUPTS();
        g_altera_info[ i+1 ] = 0xE0;
        }
        
        

/*        
    WORD val = 0;
    portDISABLE_INTERRUPTS();
    AlteraWriteCommand( 0x0C, 0 );
    AlteraWriteCommand( 0x0D, 1 );
    portENABLE_INTERRUPTS();
    while (1)
        {
        portDISABLE_INTERRUPTS();
        AlteraWriteCommand( 0x0A, val );
        AlteraWriteCommand( 0x0B, val );
        portENABLE_INTERRUPTS();
        val = (val == 0) ? 1 : 0;
        }
*/
    
    
	//Set_Enable_InterruptIVG_Perif( Timer3_Interrupt, IVG12, timer3_ISR );
	
    ConfiGureTimerPeriodAndWidth( TIMER1, (CurSystClkHz / 1000) * 10, 0 );
    EnableTimers( TIMER1 );
    
#if 0
    /*  UART */
    /*portDISABLE_INTERRUPTS();
    AlteraWriteCommand( 0x09, 0x01 );
    AlteraWriteCommand( 0x0A, 2 );
    AlteraWriteCommand( 0x0B, 3 );
    AlteraWriteCommand( 0x0C, 0x00 );
    g_altera_reg[ 0x01 ].value = 0;
    portENABLE_INTERRUPTS();*/
    
    uart_init();
    test_uart();
#endif
    
    
    /*     */
//	AlteraWriteCommand( 0x02, 0xFF );               /*   */
//	AlteraWriteCommand( 0x03, 0xFF );               /*    */
//	AlteraWriteCommand( 0x01, 0x01 );               /*   */
//	DelayMs( 10 );                                  /*   */
    
#if 0
    res = if_initInterface( &h_sd, NULL );
    for (i = 0; (i < 512); i++)
        {
        f_sd_buf[ i ] = i & 0xFF;
        }
//    res = if_writeBuf( &h_sd, 2, f_sd_buf );
    for (i = 0; (i < 512); i++)
        {
        f_sd_buf[ i ] = 0xFF;
        }
    while (1)
        res = if_readBuf( &h_sd, 0, f_sd_buf );
    res = if_readBuf( &h_sd, 1, f_sd_buf );
    res = if_readBuf( &h_sd, 2, f_sd_buf );
#endif

    //initialize_wdt( wdt_period );

    ltr030Conf.USBDev.ep2rxEn = 1;
    if (g_device_mode == e_DeviceMode_ETH)
        sys_thread_new( "ETH", ethernet_task, NULL, 1024, ETHER_TASK_PRIO );
        
	while (1)
        { 
        clear_extwdt();
            
	    if (ltr030Conf.CommandDataCounter >= sizeof(t_ControlRequest))
            {
	    	ProcessCommand();
            }
	    
	    f_update_alteraregs();
	    
		if (g_mode_ULED == e_LED_Orange)
    	    {
            uled_color = (uled_color == e_LED_Red) ? e_LED_Green : e_LED_Red;
            uled_color |= (g_mode_ELED & 0x03) << 2;
            portDISABLE_INTERRUPTS();
    		AlteraWriteCommand( 0x08, uled_color );
    		portENABLE_INTERRUPTS();
    		uled_color &= 0x03;
		    }
		else if (g_mode_ULED != uled_color)
		    {
		    WORD led_ctrl = g_mode_ULED | ((g_mode_ELED & 0x03) << 2);
		    uled_color = g_mode_ULED;
            portDISABLE_INTERRUPTS();
            AlteraWriteCommand( 0x08, led_ctrl );
            portENABLE_INTERRUPTS();
		    }
		 else if (g_mode_ELED != gled_color)
		    {
		    WORD led_ctrl = g_mode_ULED | ((g_mode_ELED & 0x03) << 2);
		    gled_color = g_mode_ELED;
            portDISABLE_INTERRUPTS();
            AlteraWriteCommand( 0x08, led_ctrl );
            portENABLE_INTERRUPTS();
		    }
#if defined(LTR030_ENDPOINT)
        if (GetOverflowStatus( TIMER1 ))
            {
    		ClearOverflowStatus( TIMER1 );
            if (--fanning_timer <= 0)
                {
                process_fanning();
                fanning_timer = ct_fanning_period;
                }
            }
#endif
	   if (g_device_mode == e_DeviceMode_ETH)
            {
	        vTaskDelay( 10 );
	        }
	    else
	        {
	        ProcessData();
	        }
	        
	    clear_wdt();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void prvSetupTimerInterrupt
    (
    void
    )
    {
    const unsigned ct_counter = 2000;
    const unsigned ct_scale = 199;
    
    /* Turn on timer, TMPWR */
    *pTCNTL = 1;
	*pTSCALE = ct_scale;
	*pTCOUNT = ct_counter;
	*pTPERIOD = ct_counter;
	/* Register Timer ISR and enable IVG6 (Timer) */
	register_handler_ex( ik_timer, (ex_handler_fn )vTickISR, EX_INT_ENABLE );
    /* Interrupts are disabled when this is called so the timer can be started here. */
	*pTCNTL   = 0x7;                                 // Start Timer and set Auto-reload
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
